<?php
require_once ZEND_SERVER_GUI_PATH . '/library/Zwas/Text.php';
require_once ZEND_SERVER_GUI_PATH . '/library/Zwas/Translate.php';

class CheckDependencies_CE {
	/**
	 * a flag signaling the directive should not be empty
	 * @var string
	 */
	const NOT_EMPTY = '#NOTEMPTY#';
	/**
	 * a flag signaling the directive should be greater than zero
	 * @var string
	 */
	const GREATER_THAN_ZERO = '#GREATERTHANZERO#';
	/**
	 * an array with the different values for the true value
	 * @var array
	 */
	protected static $trueValues = array(true, 1, '1', 'on', 'true', 'yes');
	/**
	 * an array with the different values for the false value
	 * @var array
	 */
	protected static $falseValues = array(false, 0, '0', 'off', 'false', 'no', '', null);
	
	/**
	 * an error message explaining which check failed.
	 * @var string
	 */
	private static $errorMessage = null;

	/**
	 * an array with the needed directives and their wanted values for this version (CE/PE/EE).
	 * child classes can add to this list using the addDirectives method
	 * @var array
	 */
	protected $directives = array(
		'tidy.clean_output' 			=> array('required' => false, 'value' => false),
		'session.auto_start'			=> array('required' => true,  'value' => false),
		'auto_append_file'				=> array('required' => false, 'value' => ''),
		'auto_prepend_file'				=> array('required' => false, 'value' => ''),
		'variables_order'				=> array('required' => true,  'value' => 'GPCS'),
		'doc_root'						=> array('required' => false, 'value' => ''),
		'pcre.backtrack_limit'			=> array('required' => true,  'value' => self::GREATER_THAN_ZERO),
	);

    /**
	 * an array with the needed extensions for this version (CE/PE/EE).
	 * child classes can add to this list using the addExtensions method
	 * @var array
	 */
	protected $extensions = array('PDO', 'pdo_sqlite', 'SimpleXML', 'DOM', 'ctype', 'JSON', 'iconv');

	public function __construct() {
		if (extension_loaded('Zend Optimizer+')) {
			$this->addDirectives(array('zend_optimizerplus.save_comments' => array('required' => true,  'value' => true)));
		}
	}
	/**
	 * Performs a check for the existent and/or value of directives and extensions (case insensitive) needed for ZS to run.
	 * @return bool
	 */
	public function check() {
		return ((self::checkDirectives(ini_get_all(), $this->directives)) &&
				(self::checkExtensions(get_loaded_extensions(), $this->extensions)));
	}
	
	/**
	 * returns the error message set by the report method, when a dependency check failed
	 * @return string
	 */
	public static function getErrorMessage(){
		return self::$errorMessage;
	}
	
	/**
	 * static method for checking the needed directives and their values against the server values.
	 * public for testing reasons.
	 * @param array $set the server directives and their values
	 * @param array $wanted the needed directives and their values
	 * @return bool whether the needed directives and their values are set correct
	 */
	public static function checkDirectives(array $set, array $wanted) {
		foreach ($wanted as $wantedDirectiveName => $wantedDirectiveData) {
			if (!isset($set[$wantedDirectiveName])) {
				if ($wantedDirectiveData['required']) {
					self::report('Directive %s is required', array($wantedDirectiveName));
					return false;
				} else {
					continue;
				}
			}
			$wantedDirectiveValue = $wantedDirectiveData['value'];
			$actualDirectiveValue = $set[$wantedDirectiveName]['local_value'];
			switch (true) {
				case is_bool($wantedDirectiveValue):
					if (($wantedDirectiveValue === true)  && (!in_array($actualDirectiveValue, self::$trueValues,  true))) {
						self::report('Directive %s should be set to true', array($wantedDirectiveName));
						return false;
					}
					if (($wantedDirectiveValue === false) && (!in_array($actualDirectiveValue, self::$falseValues, true))) {
						self::report('Directive %s should be set to false', array($wantedDirectiveName));
						return false;
					}
					break;

				case ($wantedDirectiveValue == self::GREATER_THAN_ZERO):
					if (!($actualDirectiveValue > 0)) {
						self::report('Directive %s should be greater than 0 but is set to %s', array($wantedDirectiveName, $actualDirectiveValue));
						return false;
					}
					break;

				case ($wantedDirectiveValue == self::NOT_EMPTY):
					if (empty($actualDirectiveValue)) {
						self::report('Directive %s should not be empty', array($wantedDirectiveName));
						return false;
					}
					break;

				default:
					if ($actualDirectiveValue != $wantedDirectiveValue) {
						if (empty($wantedDirectiveValue)){
							$wantedDirectiveValue = '<empty>';
						}
						self::report('Directive %s should be set to %s but is set to %s', array($wantedDirectiveName, $wantedDirectiveValue, $actualDirectiveValue));
						return false;
					}
			}
		}
		return true;
	}

	/**
	 * static method for checking the needed extensions against the loaded ones.
	 * public for testing reasons.
	 * @param array $loaded the loaded extensions
	 * @param array $wanted the needed extensions
	 * @return bool whether the needed extensions are loaded
	 */
	public static function checkExtensions($loaded, $wanted) {
		$loaded = array_map('strtolower', $loaded);
		$wanted = array_map('strtolower', $wanted);
		foreach ($wanted as $name) {
			if (!in_array($name, $loaded, true)){
				self::report('Extension %s should be loaded', array($name));
				return false;
			}
		}
		return true;
	}

	/**
	 * helper method for adding directives to the list of checked directives
	 * @param array $directives
	 */
	protected function addDirectives($directives) {
		$this->directives = array_merge($this->directives, $directives);
	}

	/**
	 * helper method for adding extensions to the list of checked extensions
	 * @param array $extensions
	 */
	protected function addExtensions($extensions) {
		$this->extensions = array_merge($this->extensions, $extensions);
	}
	
	/**
	 * store the error message in a class var, and writes it to an error log
	 * @param string $message error message
	 */
	private static function report($message, $args){
		self::$errorMessage = new Zwas_Text(Zwas_Translate::_($message), $args);
		error_log(self::$errorMessage);
	}
}