<?php
class Common_ApplicationModel {

	const DATE_FORMAT_SHORT		= 'short';
	const DATE_FORMAT_MEDIUM	= 'medium';
	const DATE_FORMAT_LONG		= 'long';
	
	/**
	 * Set the accept terms flag
	 */
	public static function setAcceptedTerms() {
		self::setConfigVar(array('acceptedTerms' => '1'));
	}
	
	/**
	 * Return if the user has accepted the EULA
	 * @return boolean
	 */
	public static function hasAcceptedTerms() {
		$acceptedTerms = (string)self::getConfigVar('registration', 'acceptedTerms', true);
		return ('1' === $acceptedTerms);
	}
	
	/**
	 * Set the registration complete flag
	 */
	public static function setRegistrationComplete() {
		self::setConfigVar(array('completed' => '1'));
	}
	
	/**
	 * Remove the brandNewInstallation directive from zend-server-user.ini
	 */
	public static function scrubNewInstallationFlag() {
		self::deleteConfigVar(array('brandNewInstallation'));
	}
	
	/**
	 * Return if the user completed the registration process
	 * @return boolean
	 */
	public static function isRegistrationComplete() {
		$complete = (string)self::getConfigVar('registration', 'completed', true);
		if ('1' === $complete) {
			$complete = true;
		} else {
			$complete = false;
			try {
				/// attempt to retrieve the new installation marker
				/// if all's well, complete should remain false - we need registration
				(string)self::getConfigVar('zendServer', 'brandNewInstallation', true);
			} catch (Zwas_Exception $e) {
				/// if the marker is not found, this is an upgrade - assume registration is complete
				$complete = true;
				self::setRegistrationComplete();
			}
		}
		
		return $complete;
	}
	
	/**
	 * Return if the application's password is set
	 * @return boolean
	 */
	public static function isPasswordSet() {
		
		$password = self::getPassword();
		
		if ('INSTALLATION_PLACEHOLDER_GUI_PASSWORD' == $password) {
			return false;
		}
		return true;
	}
	
	/**
	 * Save the new password to zend-server-user.ini if it is valid
	 * @param string $newPassword
	 * @param string $confirmPassword
	 * @throws Zwas_Exception
	 */
	public static function savePassword($newPassword, $confirmPassword) {
		
		if ($confirmPassword !== $newPassword) {
			throw new Zwas_Exception(Zwas_Translate::_('Your new password entries do not match'));
		}
		
		$validatePassword = new Zwas_Validate_Password();
		if (! $validatePassword->isValid($newPassword)) {
			throw new Zwas_Exception(Zwas_Translate::_('The password you have entered is not valid. Passwords must be 4-20 characters long  and cannot include spaces'));
		}
				
		// make sure the directive exists
		if (! self::passwordDirectiveExists()) {
			throw new Zwas_Exception(Zwas_Translate::_('zend-server-user.ini is missing the password directive under the [authentication] section'), Zwas_Exception::ERROR);
		}
		
		self::setConfigVar(array('password' => md5($newPassword)));
		
		// change the password marker
		if (Zend_Auth::getInstance()->hasIdentity()) {
			Authentication_Marker::setMarker();
			Zend_Auth::getInstance()->getIdentity()->updateMarker();
		}
	}

	/**
	 * Reset to zend-server-user.ini
	 * @throws Zwas_Exception
	 */
	public static function resetPassword() {
		self::setConfigVar(array('password' => 'INSTALLATION_PLACEHOLDER_GUI_PASSWORD'));
	}
	
	/**
	 * Check that the configuration object has an authentication section and
	 * password directive
	 *
	 * @return boolean
	 */
	public static function passwordDirectiveExists() {
		try {
			self::getPassword();
			
			return true;
		} catch (Zwas_Exception $e) {
			// Ignore the exception, passowrd cannot be retrieved
		}
		return false;
	}
	
	/**
	 * Load the configuration from the configurations files and set it to the global
	 * registry
	 * @throws Zwas_Exception
	 */
	public static function loadConfiguration() {
		try {
			
			$confFile = Zwas_Path::create(ZEND_SERVER_CONFIGURATION_FILE);
			// use FileObject to check that the configuration file can be read
			Context_FileSystem::getFileObject($confFile, 'r');
			$configuration = Context_FileSystem::getConfigReader($confFile, array('allowModifications' => true));

			// add the user configuration directives
			$userConfFile = Zwas_Path::create(ZEND_SERVER_USER_CONFIGURATION_FILE);
			Context_FileSystem::getFileObject($userConfFile, 'r');
			$configuration->merge(Context_FileSystem::getConfigReader($userConfFile));
			
			Zend_Registry::set('configuration', $configuration);
			
		} catch (Exception $e) {
			throw new Zwas_Exception($e->getMessage());
		}
	}
	
	/**
	 * Get the password directive
	 *
	 * @return string
	 * @throws Zwas_Exception if not set
	 */
    public static function getPassword() {
    	return self::getConfigVar('authentication', 'password', true);
    }
    
	/**
	 * Get the phpMyAdmin URL directive
	 *
	 * @return string a valid url, or null if directive value is empty or equal to the PLACEHOLDER
	 * @throws Zwas_Exception if not set or invalid url
	 */
    public static function getPhpMyAdminUrl() {
    	$url = self::getConfigVar('monitor', 'phpMyAdminUrl', true);
    	
    	if (empty($url) || ('INSTALLATION_PLACEHOLDER_PHPMYADMIN_URL' == $url)) {
			return null;
		}
		
    	if (! Zend_Uri_Http::check($url)) {	// full path
    		if ((0 !== strpos($url, '/')) || (! Zend_Uri_Http::check('http://localhost' . $url))) { // relative path
    			Zwas_Log::logMessage('Invalid phpMyAdmin URL value: ' . $url);
    			return '';
    		}
		}
		return $url;
    }
    
	/**
	 * Get the Zend Server version directive
	 *
	 * @return string
	 * @throws Zwas_Exception if not set or empty
	 */
    public static function getZendServerVersion() {
    	return self::getConfigVar('zendServer', 'version', false);
    }
    
	/**
	 * Get the path of the file that contains the log files for monitoring
	 *
	 * @return string
	 * @throws Zwas_Exception if not set or empty
	 */
    public static function getLogsFile() {
    	return self::getConfigVar('monitor', 'logfile', false);
    }
    
	/**
	 * Get the auto log file refresh interval
	 *
	 * @return int Number of seconds
	 * @throws Zwas_Exception if not set or empty
	 */
    public static function getLogfileRefreshInterval() {
    	return (int)self::getConfigVar('monitor', 'logfileRefreshInterval', false);
    }
    
    /**
     * @return boolean
     */
    public static function getDisplayWelcome() {
    	$value = trim(self::getConfigVar('monitor', 'displayWelcome', true));
    	return ('1' === $value);
    }
    
    /**
     * @param boolean $status
     */
    public static function setDisplayWelcome($status) {
    	if (true === $status) {
    		$value = array('displayWelcome' => 1);
    	} else {
    		$value = array('displayWelcome' => 0);
    	}
   		self::setConfigVar($value);
    }
    
	/**
	 * Get the user server url
	 *
	 * @return string a valid url
	 * @throws Zwas_Exception if not set, empty or invalid url
	 */
    public static function getUserServerUrl() {
    	$url = self::getConfigVar('userServer', 'url', false);
    	
    	$url .= '/index.php';
    	if (! Zend_Uri_Http::check($url)) {
    		Zwas_Log::logMessage('zend-server.ini holds an invalid URL for the url directive under the [userServer] section: ' . $url);
			throw new Zwas_Exception(	new Zwas_Text(Zwas_Translate::_('An internal error has occurred, retry your last action and if that fails reinstall %s'), array(ZEND_SERVER_APPLICATION_NAME)),
										Zwas_Exception::ERROR);
    	}
		
    	return $url;
    }
    
    /**
     * @return boolean
     */
    public static function isRunningAsDedicatedServer() {
    	$userServerUrl = self::getConfigVar('userServer', 'url', true);
   		return (('INSTALLATION_PLACEHOLDER_USER_SERVER_URL' !== $userServerUrl) &&
   				('' !== $userServerUrl));
    }
    
	/**
	 * Get the package build identifier
	 *
	 * @return string
	 */
    public static function getPackageBuild() {
    	$value = (string)self::getConfigVar('zendServer', 'packageBuild', true);
    	
    	if ('INSTALLATION_PLACEHOLDER_PACKAGE_BUILD_NUMBER' === $value) {
    		return '';
    	}
    	
    	return $value;
    }
    
	/**
	 * @return string
	 */
    public static function getGuiTag() {
    	$value = (string)self::getConfigVar('zendServer', 'build', true);
    	
    	if ('INSTALLATION_PLACEHOLDER_BUILD_TAG' === $value) {
    		return '';
    	}
    	
    	return $value;
    }
        
	/**
	 * Get whether or not the rewriting is enabled
	 *
	 * @return bool
	 * @throws Zwas_Exception if not set
	 */
    public static function getIsRewriteEnabled() {
    	$value = self::getConfigVar('zendServer', 'rewriteEnabled', true);
    	
    	if (('' == $value) || ('off' == strtolower($value))) {
    		return false;
    	}
    	return true;
    }
    
    public static function getLogDirectorypath() {
    	$logDir = get_cfg_var('zend.log_dir');
		if (! $logDir) {
			$logDir = sys_get_temp_dir();
		}
		return $logDir;
    }
    
    /**
     * Get the writer stream for the logger
     * @return Zend_Log_Writer_Abstract
     */
    public static function getErrorLoggerStream() {
    	$logDir = self::getLogDirectorypath();
			
		if ($logDir && is_dir($logDir)) {
			try {
				return new Zend_Log_Writer_Stream(Zwas_Path::create($logDir, Zwas_Log::LOGFILE));
			} catch (Exception $e) {
				// nothing interesting to do
			}
		}
		return new Zend_Log_Writer_Null();
    }
    
    /**
     * Get the temp folder dedicated for the GUI relative to the install path
     * Fallback for system temp directory
     * @return string - path to temp directory
     */
    public static function getGuiTempDir() {
    	$tempPath = Zwas_Path::create(ZEND_SERVER_INSTALL_PATH, 'tmp/gui');
    	if (! is_dir($tempPath)) {
    		// TODO: if the mkdir fails, it will keep happening on each request
    		if (! @mkdir($tempPath, 0700, true)) {
    			return sys_get_temp_dir();
    		}
    	}
    	if (! is_writable($tempPath)) {
    		return sys_get_temp_dir();
    	}
    	return $tempPath;
    }
    
    /**
     * @param string $format
     * @return string a date format matching date() function's definitions
     */
    public static function getDateFormat($format) {
    	try {
    		return self::getConfigVar('dateFormats', $format, false);
    	} catch (Exception $e) {
    		Zwas_Log::logException('zend-server.ini is missing a dateFormat directive', $e);
    		
    		// Return default date formats
    		switch ($format) {
    			case self::DATE_FORMAT_SHORT:
    				return "d-M-Y";
    			case self::DATE_FORMAT_MEDIUM:
					return "d-M H:i";
    			case self::DATE_FORMAT_LONG:
					return "d-M-Y H:i";
    			default:
    				throw new Zwas_Exception('Invalid date format requested', Zwas_Exception::ASSERT);
    		}
    	}
    }
    
    /**
     * @param string $email
     */
    public static function setNewsletterEmail($email) {
    	self::setConfigVar(array('newsletterEmail' => (string)$email));
    }
    
    /**
     * @return string
     */
    public static function getNewsletterEmail() {
    	return (string)self::getConfigVar('registration', 'newsletterEmail', true);
    }
    
    /**
     * @return integer one of Zend_Log priority level constants
     * @see http://framework.zend.com/manual/en/zend.log.overview.html
     */
    public static function getLogVerbosityLevel() {
    	$logLevel = Zend_Log::ERR;
    	try {
    		$logConstant = 'Zend_Log::' . (string)self::getConfigVar('zendServer', 'logVerbosity', false);
    		if (defined($logConstant)) {
    			$logLevel = constant($logConstant);
    		}
    	} catch (Zwas_Exception $e) {
    		// do nothing
    	}
    	return $logLevel;
    }
    
	/**
     * Return a directive from the configuration of the application
     *
     * @param string $section The section where the directive exists in
     * @param string $directive
     * @param bool $emptyIsValid Whether or not the directive value can be empty or not
     * @return string
     *
     * @throws Zwas_Exception If section or directive doesn't exist or if the directive value is empty and its not allowed
     */
    protected static function getConfigVar($section, $directive, $emptyIsValid) {
    	try {
    		$configuration = Zend_Registry::get('configuration');
    	} catch (Exception $e) {
    		throw new Zwas_Exception(Zwas_Translate::_('Failed to load the Administration Interface configuration file'), Zwas_Exception::ASSERT);
    	}
    	
    	if ((! isset($configuration->{$section})) ||
    		(! isset($configuration->{$section}->{$directive}))) {
    		throw new Zwas_Exception(
    			new Zwas_Text(
    				Zwas_Translate::_("zend-server-user.ini or zend-server.ini is missing the %s directive under the [%s] section"),
    				array($directive, $section)),
    			Zwas_Exception::ASSERT);
    	}
    	$value = trim($configuration->{$section}->{$directive});
    	
    	if ((! $emptyIsValid) && ('' === $value)) {
    		throw new Zwas_Exception(
    			new Zwas_Text(
    				Zwas_Translate::_("%s directive value is empty"),
    				array($directive)),
    			Zwas_Exception::ASSERT);
    	}
    	return $value;
    }
    
    /**
     * Writes directives to the zend-server-user.ini configuration file
     * @param array $directive
     * @throws Zwas_Exception
     */
    protected static function setConfigVar(array $directives) {
    	try {
			$zendServerIni = Context_FileSystem::getConfigReader(ZEND_SERVER_CONFIGURATION_FILE, array('allowModifications' => true));
    	
			// create a mapping of what directive goes in which section
			$directiveMapping = array();
			foreach($zendServerIni as $section => $obj) {
				foreach($obj as $directive => $value) {
					$directiveMapping[ $directive ] = $section;
				}
			}
			
			$zendServerUserIni = Context_FileSystem::getConfigReader(
														ZEND_SERVER_USER_CONFIGURATION_FILE,
	                              						array(	'skipExtends'        => true,
	                                    						'allowModifications' => true));
	
			foreach($directives as $name => $value) {
				if (! isset($directiveMapping[$name])) {
					throw new Zwas_Exception(new Zwas_Text(
						Zwas_Translate::_('zend-server.ini is missing the requested directive %s'),
						$name));
				}
				
				$section = $directiveMapping[$name];
				
				if (! isset($zendServerUserIni->{$section})) {
					$zendServerUserIni->{$section} = array();
				}
				$zendServerUserIni->{$section}->{$name} = $value;
			}
			
			$writer = Context_FileSystem::getConfigWriter(
				ZEND_SERVER_USER_CONFIGURATION_FILE,
				array(	'config'	=> $zendServerUserIni)
			);
			
			try {
				$writer->write();
			} catch (Exception $e) {
				throw new Zwas_Exception($e->getMessage(), Zwas_Exception::ERROR);
			}
			
			// reload the configurations into the memory as it has changed
			self::loadConfiguration();
			
    	} catch (Zwas_Exception $e) {
    		Zwas_Log::logException('Failed in ' . __CLASS__ . '::' . __FUNCTION__, $e);
    		throw new Zwas_Exception(	new Zwas_Text(Zwas_Translate::_('An internal error has occurred, retry your last action and if that fails reinstall %s'), array(ZEND_SERVER_APPLICATION_NAME)),
										Zwas_Exception::ERROR);
    	}
    }
    
    /**
     * Deletes directives from the zend-server-user.ini configuration file
     * 
     * This method does not use zend-server.ini as a mesh, it acts only on
     * zend-server-user.ini
     * 
     * @uses Zend_Config_Reader
     * @uses Zend_Config_Writer
     * @param array $directives Directive names to delete
     * @throws Zwas_Exception
     */
    protected static function deleteConfigVar(array $directives) {
    	try {
			$zendServerUserIni = Context_FileSystem::getConfigReader(
							ZEND_SERVER_USER_CONFIGURATION_FILE,
                            array('skipExtends' 		=> true,
                            	'allowModifications' 	=> true));
	
	                              						
    		foreach($zendServerUserIni as $section => $obj) {
				foreach($obj as $directive => $value) {
					if (in_array($directive, $directives)) {
						unset($obj->{$directive});
					}
				}
				
				if (0 == $obj->count()) {
					unset($zendServerUserIni->{$section});
				}
			}
			
			$writer = Context_FileSystem::getConfigWriter(
				ZEND_SERVER_USER_CONFIGURATION_FILE,
				array(	'config'	=> $zendServerUserIni)
			);
			
			try {
				$writer->write();
			} catch (Exception $e) {
				throw new Zwas_Exception($e->getMessage(), Zwas_Exception::ERROR);
			}
			
			// reload the configurations into the memory as it has changed
			self::loadConfiguration();
			
    	} catch (Zwas_Exception $e) {
    		Zwas_Log::logException('Failed in ' . __CLASS__ . '::' . __FUNCTION__, $e);
    		throw new Zwas_Exception(	new Zwas_Text(Zwas_Translate::_('An internal error has occurred, retry your last action and if that fails reinstall %s'), array(ZEND_SERVER_APPLICATION_NAME)),
										Zwas_Exception::ERROR);
    	}
    }
}
