<?php

class Configuration_IniModifier {

	const BACKUP_EXTENSION = '-bak';
	/**
	 * @var Zwas_FileObject
	 */
	private $stream;
	/**
	 * @var array
	 */
	private $content;
	/**
	 * @var array
	 */
	private $directives;
	/**
	 * @var array
	 */
	private $extensions;
	/**
	 * @var array
	 */
	private $zemExtensions;
	
	/**
	 * Initialize the Configuration_IniModifier object
	 *
	 * @param Zwas_FileObject $stream
	 */
	public function __construct(SplFileObject $stream) {
		$this->stream = $stream;
		$iniParser = $this->createIniParser();
		$this->content = $iniParser->getContent();
		$this->directives = $iniParser->getDirectives();
		$this->extensions = $iniParser->getExtensions();
		$this->zemExtensions = $iniParser->getZemExtensions();
		unset($iniParser);
	}
	
	/**
	 * Sets the directive with new or updated value
	 *
	 * @param string $dirName
	 * @param string $dirValue
	 */
	public function setDirective($dirName, $dirValue) {
		$updatedLine = ($dirName . '=' . $this->escapeDirectiveInput($dirValue));
		
		if (isset($this->directives[$dirName])) {
			$this->updateContentLine($this->directives[$dirName], $updatedLine);
		} else {
			// update the directives array to contain the new line
			$this->directives[$dirName] = $this->addNewContentLine($updatedLine);
		}
	}
	
	/**
	 * Gets the extension value and flag if this is enabled or not
	 *
	 * @param string $extValue
	 * @param boolean $enabled
	 */
	public function setExtension($extValue, $enabled=true) {
		$extValue = $this->escapeDirectiveInput($extValue);
		$updatedLine = (Configuration_IniParser::EXTENSION_DIRECTIVE_NAME . '=' . $extValue);
		
		if (isset($this->extensions[$extValue])){
			if($enabled) {
				$this->updateContentLine($this->extensions[$extValue], $updatedLine);
			} else {
				// delete the extension
				unset($this->content[$this->extensions[$extValue]]);
				unset($this->extensions[$extValue]);
			}
		} else {
			if($enabled) {
				// update the extensions array to contain the new line
				$this->extensions[$extValue] = $this->addNewContentLine($updatedLine);
			}
		}
	}
	
	/**
	 * Delete a directive line completely
	 *
	 * @param string $directiveName
	 */
	public function deleteDirective($directiveName) {
		if (isset($this->directives[$directiveName])) {
			$lineNumber = $this->directives[$directiveName];
			unset($this->content[$lineNumber]);
			unset($this->directives[$directiveName]);
		}
	}

	/**
	 * Enable a zem extension
	 *
	 * @param string $identifier e.g. zend_extension_manager.zend_cache --> zend_cache
	 * @param string $folder e.g. /usr/local/zend/lib/caching --> caching
	 */
	public function setZemExtension($identifier, $folder) {
		$directive = Configuration_IniParser::ZEM_DIRECTIVE_PREFIX . '.' . $identifier;
		$path = $this->escapeDirectiveInput(Zwas_Path::create(ZEND_SERVER_INSTALL_PATH, 'lib', $folder));
		
		// in order to avoid multiple lines, update existing zend_extension_manager.whatever if exists
		if (array_key_exists($identifier, $this->zemExtensions)) {
			$this->updateContentLine($this->zemExtensions[$identifier], $directive . '=' . $path);
		} else {
			$this->zemExtensions[$identifier] = $this->addNewContentLine($directive . '=' . $path);
		}
	}
	
	/**
	 * Remove a zem extension
	 * @param string $identifier e.g. zend_extension_manager.zend_cache --> zend_cache
	 */
	public function deleteZemExtension($identifier) {
		if (isset($this->zemExtensions[$identifier])) {
			$lineNumber = $this->zemExtensions[$identifier];
			unset($this->content[$lineNumber]);
			unset($this->zemExtensions[$identifier]);
		}
	}
	
	/**
	 * Saves the content into the stream
	 * @throws Zwas_Exception
	 */
	public function save() {
		// save the backup file before saving changed, for undo option
		try {
			$bakDestination = Zwas_Path::create(ZEND_SERVER_TEMP_PATH, $this->stream->getFilename() . self::BACKUP_EXTENSION);
			Context_Filesystem::getInstance()->copy($this->stream->getPathname(), $bakDestination);
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException("Failed backing up the ini file to {$bakDestination}", $e);
		}
		
		try {
			$stream = $this->stream->openFile('w');
			if (count($this->content) == 0) {
				$contentStr = '';
			} else {
				$contentStr = implode(PHP_EOL, $this->content);
				$contentStr .= PHP_EOL;
			}
			$stream->fwrite($contentStr);
		} catch (Exception $e) {
			// restore the backup
			Context_Filesystem::getInstance()->copy($bakDestination, $this->stream->getPathname());
			Zwas_Log::logException('Failed saving the new content to ' . $bakDestination, $e);
			throw new Zwas_Exception('Failed updating ' . $this->stream->getPathname());
		}
		
		if (is_file($bakDestination)) {
			Context_Filesystem::getInstance()->unlink($bakDestination);
		}
	}

	/**
	 * Creates and returns Configuration_IniExtendedParser object
	 *
	 * @return Configuration_IniExtendedParser
	 */
	protected function createIniParser() {
		return new Configuration_IniExtendedParser($this->stream);
	}
	
	/**
	 * Adds the new directive line to the content
	 *
	 * @param string $updateLine
	 * @return integer line number of new line
	 */
	private function addNewContentLine($updatedLine) {
		array_push($this->content, $updatedLine);
		// put the pointer to the end to get the index of the last element
		end($this->content);
		return key($this->content);
	}
	
	/**
	 * Update the content line
	 *
	 * @param integer $line
	 * @param string $updateLine
	 */
	private function updateContentLine($line, $updatedLine) {
		$this->content[$line] = $updatedLine;
	}
	
	/**
	 * Add safety to the input about to be written to the ini file.  If the input has any non alphnumeric
	 * character, enclose it with quotes.  If the input has quotes in it (excl. surrounding it), throw and exception
	 *
	 * @param string $input
	 * @throws Zwas_Exception
	 * @return string
	 */
	private function escapeDirectiveInput($input) {
		$input = trim($input, '"');
		
		if (false !== strpos($input, '"')) {
			throw new Zwas_Exception(Zwas_Translate::_('Quotes are not an accepted parameter for writing to the ini file'));
		}

		// check for characters that force us to put the input in quotes
		$needQuoteChars = ';/\\';
		if (false !== strpbrk($input, $needQuoteChars)) {
			$input = '"' . $input . '"';
		}
		
		return $input;
	}
}
