<?php

/*
 * What we do/don't parse:
 * 1. We do not need to parse sections from INI file (can be added in the future - if needed)
 * 2. Only lines containing directives should be parsed (we don't care for commented lines, so we don't parse them)
 * 3. Currently we parse only non commented extensions, so the array returned by the getExtensions() method has the
 * 		the following structure: array(extension_filename => true).
 */


/**
 * Configuration_IniParser
 */
class Configuration_IniParser {

	const ZEM_DIRECTIVE_PREFIX 			= 'zend_extension_manager.dir';	
	const EXTENSION_DIRECTIVE_NAME		= 'extension';
	const ZEND_EXTENSION_DIRECTIVE_NAME	= 'zend_extension';
	const COMMENT_INDICATOR				= ';';
	const QUOTE_INDICATOR				= '"';
		
	/**
	 * @var Zwas_FileObject
	 */
	protected $stream;

	protected $extensionList	= null;
	protected $directiveList	= null;
	protected $zemComponents	= null;
	protected $zendExtensions	= null;
	
	public function __construct(SplFileObject $stream) {
		$this->stream = $stream;
		$this->stream->openFile();
	}
	
	
	/**
	 * Returns an array of extensions ( extension_name => true format)
	 * @return array
	 */
	public function getExtensions() {
		if (!is_array($this->extensionList)) {
			$this->parseIni();
		}
		return $this->extensionList;
	}
	
	/**
	 * Returns an array of zend_extension items (zend_extension name => true format)
	 * @return array
	 */
	public function getZendExtensions() {
		if (!is_array($this->zendExtensions)) {
			$this->parseIni();
		}
		return $this->zendExtensions;
	}
	
	/**
	 * Returns an array of extensions (key is extension file name, value is boolean: false for commented extension)
	 * @return array
	 */
	public function getDirectives() {
		if (!is_array($this->directiveList)) {
			$this->parseIni();
		}
		return $this->directiveList;
	}
	
	/**
	 * Return an array of the zwas components defined in the ini file, key is the component identifier, 
	 * value is the line number where it is defined
	 * e.g. zend_cache => 234
	 * @return array
	 */
	public function getZemExtensions() {
		if (! is_array($this->zemComponents)) {
			$this->parseIni();
		}
		return $this->zemComponents;
	}
	
	/**
	 * Goes over the entire INI file, and parse from it a list of directives and extensions
	 */
	protected function parseIni() {
		$this->initLists();
		
		foreach ($this->stream as $line) {
			$directiveLine = $this->parseLine($line);
			if (count($directiveLine)) {
				if (array_key_exists(self::EXTENSION_DIRECTIVE_NAME, $directiveLine)) {
					$this->extensionList[ $directiveLine[self::EXTENSION_DIRECTIVE_NAME] ] = true;
				} elseif (array_key_exists(self::ZEND_EXTENSION_DIRECTIVE_NAME, $directiveLine)) {
					$this->zendExtensions[ $directiveLine[self::ZEND_EXTENSION_DIRECTIVE_NAME] ] = true;
				} else {
					$directiveName = key($directiveLine);
					// separate extensions from zend components
					if (false !== strpos($directiveName, Configuration_IniParser::ZEM_DIRECTIVE_PREFIX)) {
						
						$identifier = substr($directiveName, (1 + strrpos($directiveName, '.')));
						$this->zemComponents[$identifier] = $directiveLine[$directiveName];
					} else {
						$this->directiveList[$directiveName] = $directiveLine[$directiveName];
					}
				}
			}
		}
		// currently there is no way to close the stream, no support in the SPLFileObject
		//$this->stream->close(); or destruct
	}

	/**
	 * Parse a single line, and 'extract' the directive and value from it
	 * @param string $line
	 * @return array directive => value
	 */
	protected function parseLine($line) {
		// Checking if line is a directive declaration (should be in name=value format, and not a comment)
		if ((strpos($line, self::COMMENT_INDICATOR) === 0) ||
			(strpos($line, '=') < 1)) {
			return array();
		}
		
		// Seperate name and value according to the first '=' only
		list($directive, $value) = explode('=', $line, 2);
		$directive = trim($directive);
		$value = $this->trimValue($value);
		
		return array($directive => $value);
	}
	
	/**
	 * Trims a given string (representing the value of the directive) and trim excesive data from it
	 * @param string $value
	 * @return string Trimmed string
	 */
	protected function trimValue($value) {
		$commentPos = strpos($value, self::COMMENT_INDICATOR);
		
		if ($commentPos !== false) {
			$firstQuotePos = strpos($value, self::QUOTE_INDICATOR);
			$secondQuotePos = strpos($value, self::QUOTE_INDICATOR, ($firstQuotePos+1));
			
			// Checking if comment indicator is indeed a comment.
			if (($commentPos < $firstQuotePos) || ($secondQuotePos < $commentPos)) {
				// We need to remove the comment.
				$value = substr($value, 0, ($commentPos));
			}
		}
		
		// We want to trim the remaining value 
		return trim($value, " \t\n\r\0\x0B".self::QUOTE_INDICATOR);		
	}

	/**
	 * Initiates both directiveList and extensionList to arrays
	 */
	protected function initLists() {
		$this->extensionList	= array();
		$this->directiveList	= array();
		$this->zemComponents	= array();
		$this->zendExtensions	= array();
	}
}