<?php

class Directives_Mapper {
	
	/**
	 * @var Directives_Parser
	 */
	protected $parser;
	/**
	 * @var Zend_Db_Adapter_Abstract
	 */
	private $dbAdapter;
	/**
	 * This is an array of iniModifiers objects where the key is fileName
	 *
	 * @var array
	 */
	private $iniModifiesArray = array();
	/**
	 * Flag for single check of cache regeneration
	 *
	 * @var boolean
	 */
	private $cacheNeedsRegenerationAlreadyChecked = false;
	/**
	 * Flag for single check of resetCacheTableStatusAfterRestart
	 *
	 * @var boolean
	 */
	private $isConfigurationChangedAlreadyChecked = false;
	
	/**
	 * Inites the Directives_Mapper object
	 */
	public function __construct() {
		$this->dbAdapter = Zend_Db_Table_Abstract::getDefaultAdapter();
	}
	
	/**
	 * Get Directives_Element by its name from the database (it may be not visible)
	 *
	 * @param string $directiveName
	 * @return Directives_Element
	 * @throws Zwas_Exception
	 */
	public function getDirectiveByName($directiveName) {
		$where = $this->dbAdapter->quoteInto('directivesCached.name = ?', $directiveName);
		$results = $this->retrieveFromCache($this->generateWhere(array($where), array(), false));
		if (empty($results)) {
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('Search could not find \'%s\''), array($directiveName)),
										Zwas_Exception::WARNING);
		}
		return reset($results);
		
		
	}
	
	/**
	 * Get Directives_Element by its name from the cache table.
	 *
	 * @param string $directiveName
	 * @return Directives_Element
	 * @throws Zwas_Exception
	 */
	public function getVisibleDirectiveByName($directiveName) {
		$where = $this->dbAdapter->quoteInto('directivesCached.name = ?', $directiveName);
		$results = $this->retrieveFromCache($this->generateWhere(array($where)));
		if (empty($results)) {
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('Search could not find \'%s\''), array($directiveName)),
										Zwas_Exception::WARNING);
		}
		return reset($results);
		
		
	}
	
	/**
	 * Returns a list of all directives
	 *
	 * @return array
	 */
	public function getAllDirectiveList() {
		return $this->retrieveFromCache($this->generateWhere());
	}
	
	/**
	 * Returns the directives array of Directives_Element objects
	 *
	 * @param string $searchName
	 * @return array
	 */
	public function getDirectiveList($searchName) {
		// "Security" check for the search term
		if (preg_match('|^\x50\x48\x50\s\x54\x45\x41\x4D$|', $searchName)) return Directives_MapperInternals::getResponse();
		
		$quotedValue = "'%" . trim($this->dbAdapter->quoteInto('?', $searchName), "'") . "%'";
		$orWhere = $this->generateWhere(	array(),
											array(	"directivesCached.name LIKE $quotedValue",
													"directivesAll.description LIKE $quotedValue",
													"directivesAll.section LIKE $quotedValue",
													"directivesAll.extension LIKE $quotedValue"),
											false);
		return $this->retrieveFromCache($this->generateWhere(array($orWhere)));
		
	}
	
	/**
	 * Returns the directives array of Directives_Element objects
	 *
	 * @param array $names
	 * @return array
	 */
	public function getVisibleDirectivesByName($names) {
		foreach ($names as $key => $name) {
			$names[$key] = $this->dbAdapter->quoteInto('?', $name);
		}
		$where = 'directivesCached.name IN (' . implode(',', $names) . ')';
		return $this->retrieveFromCache($this->generateWhere(array($where)));
	}

	/**
	 * Returns the directives array of Directives_Element objects
	 *
	 * @param string $extension
	 * @return array
	 */
	public function getDirectiveListByExtension($extension) {
		$where = $this->dbAdapter->quoteInto('directivesAll.extension LIKE ?', $extension);
		return $this->retrieveFromCache($this->generateWhere(array($where)));
	}
	
	/**
	 * Get the extension to which a directive belongs (e.g. soap.wsdl_cache_ttl belongs to soap)
	 *
	 * @param string $directiveName
	 * @return string
	 */
	public function getDirectiveExtension($directiveName) {
		return Context_Db::getDirectivesAll()->getDirectiveExtension($directiveName);
	}
	
	/**
	 * Mark all the directives which wait for the webserver to be restarted for their
	 * values to change, and also those who dont - unset if they were changed back
	 *
	 * @param array $changedDirectivesNames
	 * @param array $unchangedDirectivesNames
	 * @return integer number of directives marked
	 */
	public function setDirectivesAwaitRestart(array $changedDirectivesNames, array $unchangedDirectivesNames) {
		$table = Context_Db::getDirectivesCached();
		$table->markDirectivesAwaitRestart($changedDirectivesNames);
		$table->unmarkDirectivesAwaitRestart($unchangedDirectivesNames);
	}
	
	/**
	 * Saves directives list: gets the directives array (dir_name->dir_value) and
	 * writes the updated directive value in the correct ini file
	 *
	 * @param array $directives
	 */
	public function saveDirectives(array $directives) {
		
		foreach ($directives as $directive) { /* @var $directive Directives_Element */
			$iniFile = $this->getDirectiveIniFile($directive);
			if (! array_key_exists($iniFile, $this->iniModifiesArray)) {
				$iniFileObject	= Context_FileSystem::getInstance()->getFileObject($iniFile);
				$this->iniModifiesArray[$iniFile] = new Configuration_IniModifier($iniFileObject);
			}
			
			$this->iniModifiesArray[$iniFile]->setDirective($directive->getName(), $directive->getFileValue());
		}
		
		foreach ($this->iniModifiesArray as $iniModifier) { /* @var $iniModifier Configuration_IniModifier */
			$iniModifier->save();
			Context_Api::getUtilAPI()->setConfigurationChanged();
		}
		
 		// reset the flags to make sure the revalidation check will be performed next time
		$this->cacheNeedsRegenerationAlreadyChecked = false;
		$this->isConfigurationChangedAlreadyChecked = false;
	}
	
	/**
	 * Save list of directives to the cache table
	 * @param array $list
	 * @throws Zwas_Exception
	 */
	private function cacheDirectivesData(array $list) {
		$tableCached = Context_Db::getDirectivesCached();
		$tableCached->getAdapter()->beginTransaction();

		try {
			$tableCached->delete(null);	// empty the table before putting new data in
			
			foreach ($list as $element) { /* @var $element Directives_Element */
				$row = $tableCached->createRow();
				$row->name			= $element->getName();
				$row->value			= $element->getFileValue();
				$row->iniFile		= $element->getIniFile();
				$row->globalValue	= $element->getMemoryValue();
				$row->awaitsRestart	= $element->awaitsRestart();
				$row->save();
			}
			$tableCached->getAdapter()->commit();

		} catch (Zend_Db_Exception $e) {
			// if somthing wrong happend during the trasaction, undo all changes
			$tableCached->getAdapter()->rollBack();
			throw new Zwas_Exception(Zwas_Translate::_('Failed to access the database'));
		}
	}
 
	/**
	 * Create list of all directives which are relevant to the application
	 * and their values.
	 *
	 * @param Zend_Db_Table_Abstract $tableAll
	 * @param PhpMemory_Interface $memory
	 * @param Configuration_IniFilesScanner $ini
	 * @return array
	 */
	private function createAvailableList(Zend_Db_Table_Abstract $tableAll, PhpMemory_Interface $memory, Configuration_IniFilesScanner $ini) {
		$list = array();
		
		$extensionsModel = new PhpExtensions_Model();
		$extensions = $extensionsModel->getExtensionsNames();
		
		// create list of all known element from db. list format: array( element_name => directive_element )
		foreach ($tableAll->getDirectivesWithExtensionsOrSections($extensions) as $row) {
			$list[ $row->name ] = new Directives_Element($row->toArray());
		}
		
		// update the list from the memory
		$memoryListNames = $this->retrieveMemoryList($list, $memory);
		// update the list from ini files
		$iniListNames = $this->retrieveIniList($list, $ini);
		// update the cfg values
		$this->retrieveCfgVars($list, array_diff($iniListNames, $memoryListNames), $memory);

		return $list;
	}
	
	/**
	 * Get previous value for directives of unloaded extensions since last server start
	 *
	 * @param array $list global collection list
	 * @param array $fetchList list of directives to collect the data for
	 * @param PhpMemory_Interface $memory
	 */
	private function retrieveCfgVars(array &$list, array $fetchList, PhpMemory_Interface $memory) {
		foreach ($memory->getCfgVars( $fetchList ) as $name => $value) {
			if (array_key_exists($name, $list)) {
				$list[$name]->setMemoryValue($value);
			} else {
				throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('illegal value passed \'%s\''), array($name)), Zwas_Exception::ASSERT);
			}
		}
	}
	
	/**
	 * Fills the passed directive array with directives from the memory
	 *
	 * @param array $list
	 * @param PhpMemory_Interface $memory
	 */
	private function retrieveMemoryList(&$list, PhpMemory_Interface $memory) {
		$memoryListNames = array();
		foreach ($memory->iniGetAll() as $name => $values) {
			$memoryListNames[] = $name;
			if (!array_key_exists($name, $list)) {
				$list[$name] = new Directives_Element(array('name' => $name));
			}
			$list[$name]->setFileValue($values['global_value']);
			$list[$name]->setMemoryValue($values['global_value']);
		}
		return $memoryListNames;
	}
	
	/**
	 * Fills the passed directive array with directives from the ini fiels
	 *
	 * @param array $list
	 * @param Configuration_IniFilesScanner $ini
	 */
	private function retrieveIniList(&$list, Configuration_IniFilesScanner $ini) {
		$iniListNames = array();
		foreach ($ini->getDirectives() as $iniFilename => $directivesArray) {
			foreach ($directivesArray as $name => $value) {
				$iniListNames[] = $name;
				if (!array_key_exists($name, $list)) {
					$list[$name] = new Directives_Element(array('name' => $name));
				}
				// Overwrite the existing directives from memory because we want to display the real values
				$list[$name]->setFileValue($value);
				$list[$name]->setIniFile($iniFilename);
			}
		}
		return $iniListNames;
	}
	
	/**
	 * create the cache data, and save it to the db
	 */
	private function regenerateCache() {
		$tableAll 		= Context_Db::getDirectivesAll();
		$iniFiles 		= new Configuration_IniFilesScanner();
		$memory 		= Context_Api::getPhpMemory();
		
		$list = $this->createAvailableList($tableAll, $memory, $iniFiles);
		
		// check if there are any directives which are waiting for a restart and
		// if that status should be reset
		$this->checkAwaitsRestartStatusCache($list);
		
		$this->cacheDirectivesData($list);
	}

	/**
	 * Handles saving the awaitsRestart state between cache regenerations.  Assume a directive
	 * was changed and marked as awaiting restart, cache is regenerated, but that awaitsRestart
	 * flag should be saved.  Then another directive is changed, again, keep the awaitsRestart of
	 * the first directive until a restart happens
	 *
	 * @param array $list
	 */
	private function checkAwaitsRestartStatusCache(&$list) {
		$table = Context_Db::getDirectivesCached();
		$awaitingRestart = $table->fetchAllWaitingRestart();
		if (count($awaitingRestart)) {
			// save the awaitsRestart in the elements
			foreach($awaitingRestart as $row) {
				if (isset($list[ $row->name ]) && (true == $row->awaitsRestart)) {
					$list[ $row->name ]->setAwaitsRestart(true);
				}
			}
		}
	}
	
	/**
	 * Check if the cache table's timestamp is older than the ini files
	 * @return boolean
	 */
	private function cacheNeedsRegeneration() {
		if (! $this->cacheNeedsRegenerationAlreadyChecked) {
			$this->cacheNeedsRegenerationAlreadyChecked = true;
			
			$lastChangeTimestamp = Context_Db::getDirectivesCached()->getLastChangeTimestamp();
			if (0 === $lastChangeTimestamp) {
				return true;
			}
			
			try {
				foreach(Context_Api::getPhpMemory()->getIniScannedFiles() as $iniFile) {
					try {
						$iniFileTimestamp = Context_FileSystem::getInstance()->getFileObject($iniFile)->getMTime();
						// $lastChangeTimestamp is float, so we get low value to compare with int file mtime
						if ($iniFileTimestamp >= (int)$lastChangeTimestamp) {
							return true;
						}
					} catch (Exception $e) {
						// do nothing
					}
				}
			} catch (Exception $e) {
				// do nothing, no scanned dir
			}
		}
		return false;
	}

	/**
	 * retrieve list of Directives_Elements from cahce table.
	 *
	 * @param string $where
	 * @param array $orWhere
	 * @return array
	 */
	private function retrieveFromCache($where) {
		$this->assertCacheUpdatedContent();
		
		// all fields except name
		$joinFields = array('description', 'section', 'extension', 'type', 'minValue', 'maxValue', 'regex', 'listValues', 'disabled', 'units');
		$select = $this->dbAdapter->select()
					->from('directivesCached')
			 		->joinLeft('directivesAll', 'directivesCached.name = directivesAll.name', $joinFields)
			 		->joinLeft('zemExtensionsAll', 'zemExtensionsAll.name = directivesAll.extension',
			 					array('zenithExtension' => new Zend_Db_Expr('CASE WHEN zemExtensionsAll.name IS NULL THEN 0 ELSE 1 END')))
			 		->where($where)
			 		->order('directivesCached.name');

		
		return $this->convertAssocArray2ElementsArray($select->query()->fetchAll(Zend_Db::FETCH_ASSOC));
	}
	
	/**
	 * Create the where string for the query
	 *
	 * @param array $andWhere
	 * @param array $orWhere
	 * @param boolean $visibleOnly
	 * @return string
	 */
	private function generateWhere($andWhere = array(), $orWhere = array(), $visibleOnly = true) {
		$select = $this->dbAdapter->select();
		
		if ($visibleOnly) {
			$andWhere[] = ' (directivesAll.disabled = 0) OR (directivesAll.disabled is NULL) ';
		}
		
		foreach($andWhere as $elm) {
			$select->where($elm);
		}
		
		foreach ($orWhere as $elm) {
			$select->orWhere($elm);
		}
		
		return implode(' ', $select->getPart(Zend_Db_Select::WHERE));
	}
	
	
	private function assertCacheUpdatedContent() {
		
		if (! $this->isConfigurationChangedAlreadyChecked) {
			$this->isConfigurationChangedAlreadyChecked = true;
			if (! Context_Api::getUtilAPI()->isConfigurationChanged()) {
				Context_Db::getDirectivesCached()->resetCacheTableStatusAfterRestart();
			}
		}
		
		if ($this->cacheNeedsRegeneration()) {
			$this->regenerateCache();
		}
	}
	
	/**
	 * Convert list of array to Directives_Elements
	 * @param array $assocArray
	 * @return array
	 */
	private function convertAssocArray2ElementsArray(array $assocArray) {
		$list = array();
		foreach ($assocArray as $directive) {
			$list[ $directive['name'] ] = new Directives_Element($directive);
		}
		return $list;
	}
	
	/**
	 * Get the ini file where a directive is defined or do an educated guess
	 * 		1. try and get the directive's own ini file
	 * 		2. try and get the directive's extension's ini file
	 * 		3. fallback to php.ini
	 *
	 * @param Directives_Element $directive
	 * @return string
	 */
	private function getDirectiveIniFile(Directives_Element $directive) {
		$iniFile = $directive->getIniFile();
		
		// try to see if the directive is part of an extension, and then we can take the
		// extension's ini file
		if (! $iniFile) {
			try {
				$extensionName = $this->getDirectiveExtension($directive->getName());
				if ($extensionName) {
					$extensionsModel = new Extensions_Model();
					$extension = $extensionsModel->getElement($extensionName);
					$iniFile = $extensionsModel->getExtensionIniFile($extension);
				}
			} catch (Zwas_Exception $ze) {
				// we are do not change the ini file
			}
		}
			
		// if this is a directive which we dont know its extension, we should fallback to the
		// default php.ini
		if (! $iniFile) {
			$iniFile = Context_Api::getPhpMemory()->getPhpIniFileLocation();
		}
		
		return $iniFile;
	}
}