<?php

class Directives_Model {
	
	/**
	 * @var Directives_Mapper
	 */
	protected $mapper;
	/**
	 * Contains the list of static list of popular directive names
	 * @var array
	 */
	private $popularList;
	/**
	 * Contains the array of loaded extensions or null at beginning
	 *
	 * @var array | null
	 */
	private $loadedExtensions = null;
	
	/**
	 * Initiates the directive model object
	 */
	public function __construct() {
		$this->mapper = new Directives_Mapper();
		$this->popularList = array ('error_reporting',
									'display_errors',
									'include_path',
									'memory_limit',
									'max_execution_time',
									'short_open_tags',
									'output_buffering',
									'expose_php',
									'variables_order',
									'date.timezone',
									'error_log',
									'upload_max_filesize');
	}
	
	/**
	 * Returns the most popular directives
	 *
	 * @return array
	 */
	public function getPopularDirectives() {
		$directivesList = $this->mapper->getVisibleDirectivesByName($this->popularList);
		return $this->getSectionsViewList($directivesList);
	}
	
	/**
	 * Returns a list of the available directives Views grouped by sections/extensions
	 *
	 * @param string $searchName
	 * @return array
	 */
	public function getAvailableDirectivesList($searchName) {
		$directivesList = $this->getDirectiveList($searchName);
		return $this->getSectionsViewList($directivesList);
	}

	/**
	 * Returns a list of the specific extension directives Views grouped by sections/extensions
	 *
	 * @param string $extension
	 * @return array
	 */
	public function getDirectivesListByExtension($extension) {
		$directivesList = $this->mapper->getDirectiveListByExtension($extension);
		return $this->getSectionsViewList($directivesList);
	}
	
	/**
	 * Saves directives list: gets the directives array (dir_name->dir_value)
	 *
	 * @param array $directives
	 * @return integer number of directives which value is different than their globalValue
	 * @throws Zwas_Exception
	 */
	public function saveDirectives(array $directives) {

		$validationErrorsMessages	= array();
		$directivesToSave			= array();
		
		// arrays for setting the awaitsRestart flag properly
		$changedDirectives			= array();
		$unchangedDirectives		= array();
		
		foreach ($directives as $directive => $value) {
			
			$currentDirective = $this->mapper->getDirectiveByName($directive);
			
			// Seperate changed directives from unchanged directives
			if (! Directives_Compare::isValueEqual($currentDirective, $value)) {
				
				$currentDirective->setFileValue($value);
				$validate = new Directives_Validate($currentDirective);
				// if directive's value is valid, add him to the save list
				if ($validate->isValid()) {
					$directivesToSave[] = $currentDirective;
					
					$this->collectChangedDirectives($currentDirective, $value, $changedDirectives, $unchangedDirectives);
				// if directive's value is not valid, add the errors msg to the errors list
				} else {
					// Throw the first validation error
					$validationErrorsMessages[] = $validate->getError();
					throw new Zwas_Exception(new Zwas_Text('%s. Details: %s', array($directive, $validate->getError())));
				}
			}
			
		}
		
		$this->mapper->saveDirectives($directivesToSave);
		$this->mapper->setDirectivesAwaitRestart($changedDirectives, $unchangedDirectives);
		
		return count($changedDirectives);
	}
	
	/**
	 * Get a directive's details by its name
	 *
	 * @param string $directiveName
	 * @return Directives_Element
	 */
	public function getDirectiveByName($directiveName) {
		return $this->mapper->getDirectiveByName($directiveName);
	}
	
	/**
	 * Get a visible directive's details by its name
	 *
	 * @param string $directiveName
	 * @return Directives_Element
	 */
	public function getVisibleDirectiveByName($directiveName) {
		return $this->mapper->getVisibleDirectiveByName($directiveName);
	}

	/**
	 * Returns a list of all directives
	 *
	 * @return array
	 */
	public function getAllDirectiveList() {
		$directivesList = $this->getAllDirectiveElementsList();
		return $this->getSectionsViewList($directivesList);
	}
		
	/**
	 * Returns a list of all directive elements
	 *
	 * @return array
	 */
	public function getAllDirectiveElementsList(){
		return $this->mapper->getAllDirectiveList();
	}
		
	/**
	 * Creates a sections view list from directive array
	 *
	 * @param array $directivesList
	 * @return array
	 */
	private function getSectionsViewList($directivesList) {
		$sectionsList = array();	// array of section display objects, key is lower cased section name
		
		foreach ($directivesList as $directive) {	/* @var $directive Directives_Element */
			$sectionName = Directives_Model::getParentName($directive);
			$lowerSectionName = strtolower($sectionName);
			$extensionIsLoaded = true;
			
			// get the relevant section object
			if (array_key_exists($lowerSectionName, $sectionsList)) {
				// here we avoid creating the same section twice
				$sectionObj = $sectionsList[ $lowerSectionName ];
			} else {
				
				if ($directive->getExtension() == '') {
					$sectionType = Directives_DisplayData_Section::TYPE_SECTION;
				} else {
					$sectionType = Directives_DisplayData_Section::TYPE_EXTENSION;
				}
				
				$sectionObj = new Directives_DisplayData_Section($sectionType, $sectionName);

				$sectionsList[ $lowerSectionName ] = $sectionObj;
				
				if (Directives_DisplayData_Section::TYPE_EXTENSION === $sectionType) {

					if ($this->isExtensionNotLoaded($sectionName)) {
				
						$action = new DisplayWidget_ExtensionIconButton(	DisplayWidget_ExtensionIconButton::ICON_NOT_LOADED_NO_ACTION,
																			Zwas_Translate::_('The extension is not loaded.  Click for more info'));
																			
						if ($directive->belongsToZendServerExtension()) {
							$action->setController('Configuration');
							$action->setAction('Zend-Server-Components');
							$action->setSidbarIdentifier(MenuItem::generateId('Configuration', 'Zend-Server-Components'));
							$sectionObj->setActionLink($action);
						} else {
							$action->setController('Configuration');
							$action->setAction('Extensions');
							$action->setSidbarIdentifier(MenuItem::generateId('Configuration', 'Extensions'));
							$sectionObj->setActionLink($action);
						}
						
						$extensionIsLoaded = false;
					}
				}
				
			}

			// create a Directive_DisplayData for the directive and add it to the section
			$directiveDisplay = $this->getDirectiveDisplayData($directive);
			$sectionObj->addDirective($directiveDisplay);
			
			// for loaded extensions, display that the extension awaits restart after a directive change
			if ($extensionIsLoaded && $this->isDirectiveChanged($directive)) {
				$sectionObj->setMessage(Zwas_Translate::_('Restart PHP to apply changes'));
			}
		}
		ksort($sectionsList);
		return $sectionsList;
	}
	
	/**
	 * Returns the directive list
	 *
	 * @param string $search
	 * @return array of Directives_Element objects
	 */
	private function getDirectiveList($search) {
		return $this->mapper->getDirectiveList($search);
	}
	
	/**
	 * Returns the directive parent name: section or extension (extension is stronger)
	 *
	 * @param Directives_Element $directive
	 * @return string
	 */
	private static function getParentName(Directives_Element $directive) {
		$extension = $directive->getExtension();
		$section = $directive->getSection();
		return (($extension) ? $extension : $section);
	}
	
	/**
	 * A boolean directive needs special handling when checking if it was changed due to get_cfg_var behavior
	 * which returns an empty string in case the value is Off / No / 0
	 *
	 * @param Directives_Element $element
	 * @return boolean
	 */
	private function isDirectiveChanged(Directives_Element $element) {
		// a disabled element is by default not changed because the user can't change it from the GUI
		if ($element->isDisabled()) {
			return false;
		}
		
		return Directives_Compare::isDirectiveChanged($element);
	}
	
	private function getDirectiveDisplayData(Directives_Element $element) {
		
		switch ($element->getType()) {
			case Directives_Element::TYPE_SHORTHAND :
			case Directives_Element::TYPE_INT :
			case Directives_Element::TYPE_STRING :
			case Directives_Element::TYPE_INT_BOOLEAN :
				$display = new Directives_DisplayData_String(	$element->getName(),
																$element->getFileValue(),
																$element->getLength(),
																$element->getDescription(),
																$element->isDisabled(),
																$element->getUnits());
				break;
			case Directives_Element::TYPE_BOOLEAN :
				$display = new Directives_DisplayData_Boolean(	$element->getName(),
																Directives_Translator::getRealFileValue($element),
																$element->getDescription(),
																$element->isDisabled());
				break;
			case Directives_Element::TYPE_SELECT :
				$display = new Directives_DisplayData_Select(	$element->getName(),
																$element->getFileValue(),
																$element->getDescription(),
																$element->getListValues(),
																$element->isDisabled());
				break;
			default:
				//inmpossible that the directive element exists without type
				throw new Zwas_Exception(new Zwas_Text('The directive "%s" doesn\'t have a type', array($element->getName())), Zwas_Exception::ASSERT);
		}

		if ($this->isDirectiveChanged($element)) {
			$prevValue = Directives_Translator::getStringMemoryValue($element);
			if ('' === $prevValue) {
				$prevValue = Zwas_Translate::_('(empty)');
			}
			$msg = new Zwas_Text(Zwas_Translate::_('Previous value was: %s'), array($prevValue));
			$changedIcon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_NOTICE, $msg);
			$display->setChangedIcon($changedIcon);
		}
		
		return $display;
	}
	
	/**
	 * Put this directive element's name in either the list of directives that should be marked as changed
	 * or in the list of those who are marked unchanged.  This enables the user to change a directive's
	 * value back to what it is in the memory and avoid getting a 'please restart' msg
	 *
	 * @param Directives_Element $directive
	 * @param mixed $value
	 * @param array $changedDirectives
	 * @param array $unchangedDirectives
	 */
	private function collectChangedDirectives(Directives_Element $directive, $value, array &$changedDirectives, array &$unchangedDirectives) {
		if (! Directives_Compare::isGlobalValueEqual($directive, $value)) {
			$changedDirectives[] = $directive->getName();
		} else {
			$unchangedDirectives[] = $directive->getName();
		}
	}
	
	/**
	 * Check if a given extension is not loaded
	 *
	 * @param string $extensionName
	 * @return boolean
	 */
	private function isExtensionNotLoaded($extensionName) {
		if (is_null($this->loadedExtensions)) {
			$this->loadedExtensions = array_flip(Context_Api::getPhpMemory()->getLoadedExtensions());
		}
		if (array_key_exists($extensionName, $this->loadedExtensions)) {
			return false;
		}
		if ($extensionName == 'Zend Extension Manager') {
			// The Zend Extension Manager never appears as a loaded extension (even if it is loaded)
			// We rather think it's loaded when it's not than think it's not loaded when it is (and give an error msg to the user)
			return false;
		}
		return true;
	}
	
}