<?php

class Directives_Validate {
	
	/**
	 * @var Directives_Element
	 */
	private $directive;
	
	/**
	 * @var Zend_Validate;
	 */
	private $validChain;
	
	public function __construct(Directives_Element $el) {
		$this->directive	= $el;
		$this->validChain	= new Zend_Validate();		
		$this->createValidationChain();
	}
	
	/**
	 * Get error validation messages.
	 *
	 * @return string
	 */
	public function getError() {
		// we always have only one error (we stop on first failure
		$errors = $this->validChain->getMessages();
		if (!empty($errors)) { 
			return array_shift($errors);
		}
		return null;
	}
	
	/**
	 * Check if the value sets for the directive is valid.
	 *
	 * @return bool
	 */
	public function isValid() {
		return $this->validChain->isValid($this->directive->getFileValue());
	}
	
	/**
	 * Build the validation chain for specific Directive element.
	 */
	protected function createValidationChain() {
		
		switch ($this->directive->getType()) {
			
			case Directives_Element::TYPE_INT:
				$this->validChain->addValidator(new Zend_Validate_Digits(), true);
				$this->addMaxValueValidator();
				$this->addMinValueValidator();
				break;

			case Directives_Element::TYPE_BOOLEAN:
				$this->validChain->addValidator(new Zend_Validate_Regex('/true|on|yes|1|false|off|no|none|null|0/i'), true);
				break;
			case Directives_Element::TYPE_INT_BOOLEAN:
				$this->validChain->addValidator(new Zend_Validate_Regex('/on|off|[0-9]+/i'), true);
				// If the directive value is not numeric, no need to validate it using the min,max validators
				if (is_numeric($this->directive->getFileValue())) {
					$this->addMaxValueValidator();
					$this->addMinValueValidator();
				}
				break;
			case Directives_Element::TYPE_SHORTHAND:
				/**
				 * From php.net: The available options are K (for Kilobytes), M (for Megabytes) 
				 * and G (for Gigabytes; available since PHP 5.1.0), these are case insensitive. 
				 * Anything else assumes bytes.
				 */				
				$this->validChain->addValidator(new Zend_Validate_Regex('/^[0-9]+[K|M|G]?$/is'), true);
				// The value will be casted to int in the min,max validators
				$this->addMaxValueValidator();
				$this->addMinValueValidator();
				break;
				
			case Directives_Element::TYPE_SELECT:
				$listValues = $this->directive->getListValues();
				if (is_array($listValues)) {
					$this->validChain->addValidator(new Zend_Validate_InArray(array_keys($listValues)), true);
				}
				break;
			case Directives_Element::TYPE_STRING:
				$regex = $this->directive->getRegex();
				if ($regex) {
					$this->validChain->addValidator(new Zend_Validate_Regex($regex), true);
				}
				break;
		}
	}
	
	/**
	 * If a maximum value is defined in the directive, add a less than validator to the chain
	 */
	private function addMaxValueValidator() {
		$maxValue = $this->directive->getMaxValue();
		if (! (is_null($maxValue) || ('' === $maxValue))) {
			// TODO: replace the max value fix with better solution (may be fix directive value in db)
			// becasue Zend_Validate_LessThan checks if the value <= and not <
			$maxValue = $maxValue + 1;
			$this->validChain->addValidator(new Zend_Validate_LessThan($maxValue), true);
		}
	}
	
	/**
	 * If a minimum value is defined in the directive, add a greater than validator to the chain
	 */
	private function addMinValueValidator() {
		$minValue = $this->directive->getMinValue();
		if (! (is_null($minValue) || ('' === $minValue))) {
			// TODO: replace the miv value fix with better solution (may be fix directive value in db)
			// becasue Zend_Validate_GreaterThan checks if the value >= and not >
			$minValue = $minValue - 1;
			$this->validChain->addValidator(new Zend_Validate_GreaterThan($minValue), true);
		}
	}
	
}