<?php
class Extensions_DisplayGenerator {
	
	/**
	 * Get the data from the extension element and create a display data container
	 *
	 * @param Extensions_ElementAbstract $extension
	 * @return Extensions_Display
	 */
	public static function getExtensionDisplay(Extensions_ElementAbstract $extensionElement) {
		$display = new Extensions_Display($extensionElement->getName());
		
		$display->setConfigurationLinkAction(self::getConfigurationLink($extensionElement));
		$display->setShortDescription($extensionElement->getShortDescription());
		$display->setStatusIcon(self::getExtensionStatusIcon($extensionElement));
		$display->setStatusActionLink(self::getExtensionActionLink($extensionElement));
		$display->setStatusMessage(self::getExtensionStatusMessage($extensionElement));
		
		return $display;
	}

	/**
	 * @param Extensions_ElementAbstract $extensionElement
	 * @return DisplayWidget_Abstract
	 */
	private static function getConfigurationLink(Extensions_ElementAbstract $extensionElement) {
		if ($extensionElement->isInstalled() &&
			$extensionElement->getNumberOfAssociactedDirectives() > 0) {
				
			$configurationLinkAction = new DisplayWidget_Link(Zwas_Translate::_('Directives'), 'Directives', 'Search-Extension');
			$configurationLinkAction->setParams(array('extension' => base64_encode($extensionElement->getName())));
			$configurationLinkAction->setSidbarIdentifier(MenuItem::generateId('Directives', 'Index'));
			return $configurationLinkAction;
		}
		return new DisplayWidget_Info('');
	}
		
	/**
	 * @param Extensions_ElementAbstract $extensionElement
	 * @return DisplayWidget_IconTooltip
	 */
	private static function getExtensionStatusIcon(Extensions_ElementAbstract $extensionElement) {
		$iniStatus 		= $extensionElement->getIniStatus();
		$fileExists 	= $extensionElement->fileExists();
		$loaded 		= $extensionElement->isLoaded();
		$awaitRestart	= $extensionElement->awaitsRestart();
		
		if (Extensions_ElementAbstract::INI_ENABLED === $iniStatus) {
			if ($fileExists) {
				if ($loaded) {
					// Loaded and unloadable
					$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_LED_GREEN);
					$icon->setTooltipContent(self::getTooltipContentByType(DisplayWidget_ExtensionActionLink::LINK_TYPE_LOADED_UNLOADABLE));
				} else {	// else, not Loaded
					if ($awaitRestart) {
						// Will be loaded after restart (file exists and was enabled)
						$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_LED_GREY);
						$icon->setTooltipContent(self::getTooltipContentByType(DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_NO_ACTION));
					} else {
						// Enabled and file exists but extension is not loaded - Failed to load the extension
						$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_LED_RED);
						$icon->setTooltipContent(self::getTooltipContentByType(DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_ERROR));
					}
				}
			} else {	// else, file not exists
				if ($loaded) {
					// Built-in and also enabled (without a reason) - should be disabled
					// TODO: G thought: I think a better error should come here (ask debbieLady), something like: "click to fix duplication?"
					$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_LED_GREEN);
					$icon->setTooltipContent(self::getTooltipContentByType(DisplayWidget_ExtensionActionLink::LINK_TYPE_ERROR_FIXABLE));
				} else {	// else, not loaded
					// Extension in not loaded, file not exists but extension is enabled - should be disabled
					$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_LED_RED);
					$icon->setTooltipContent(self::getTooltipContentByType(DisplayWidget_ExtensionActionLink::LINK_TYPE_ERROR_FIXABLE));
				}
			}
		} else {	// else, extension is not enabled in the ini file
			if ($fileExists) {
				if ($loaded) {
					if ($awaitRestart) {
						// Extension is built-in , extension file also exists
						$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_LED_GREEN);
						$icon->setTooltipContent(self::getTooltipContentByType(DisplayWidget_ExtensionActionLink::LINK_TYPE_LOADED_LOADABLE));
					} else {
						$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_LED_GREEN);
						$icon->setTooltipContent(self::getTooltipContentByType(DisplayWidget_ExtensionActionLink::LINK_TYPE_BUILT_IN));
					}
				} else {	// else, not loaded
					// Not loaded and loadable
					$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_LED_GREY);
					$icon->setTooltipContent(self::getTooltipContentByType(DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_LOADABLE));
				}
			} else {	// else, file not exists
				if ($loaded) {
					// Extension is built-in
					$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_LED_GREEN);
					$icon->setTooltipContent(self::getTooltipContentByType(DisplayWidget_ExtensionActionLink::LINK_TYPE_BUILT_IN));
				} else {
					// Should never reach here - extension is not loaded, file doesn't exist and nothing in the ini file
					$icon = new DisplayWidget_IconTooltip(DisplayWidget_IconTooltip::ICON_NULL);
				}
			}
		}

		return $icon;
	}
	
	/**
	 * @param Extensions_ElementAbstract $extensionElement
	 * @return DisplayWidget_Abstract
	 */
	private static function getExtensionActionLink(Extensions_ElementAbstract $extensionElement) {
		$iniStatus 		= $extensionElement->getIniStatus();
		$fileExists 	= $extensionElement->fileExists();
		$loaded 		= $extensionElement->isLoaded();
		$awaitRestart	= $extensionElement->awaitsRestart();
		
		$builtInString = Zwas_Translate::_('Built-in');
		
		if (Extensions_ElementAbstract::INI_ENABLED === $iniStatus) {
			if ($fileExists) {
				if ($loaded) {
					// Loaded and unloadable
					$link = new DisplayWidget_ExtensionActionLink(DisplayWidget_ExtensionActionLink::LINK_TYPE_LOADED_UNLOADABLE);
					$link->setParams(array('name' => $extensionElement->getName()));
				} else {	// else, not Loaded
					if ($awaitRestart) {
						$link = new DisplayWidget_ExtensionActionLink(DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_UNLOADABLE);
						$link->setParams(array('name' => $extensionElement->getName()));
					} else {
						// Enabled and file exists but extension is not loaded - Failed to load the extension
						$link = new DisplayWidget_Info('');
					}
				}
			} else {	// else, file not exists
				if ($loaded) {
					// Built-in and also enabled (without a reason) - should be disabled
					$link = new DisplayWidget_ExtensionActionLink(DisplayWidget_ExtensionActionLink::LINK_TYPE_ERROR_FIXABLE);
					$link->setParams(array('name' => $extensionElement->getName()));
				} else {	// else, not loaded
					// Extension in not loaded, file not exists but extension is enabled - should be disabled
					$link = new DisplayWidget_ExtensionActionLink(DisplayWidget_ExtensionActionLink::LINK_TYPE_ERROR_FIXABLE);
					$link->setParams(array('name' => $extensionElement->getName()));
				}
			}
		} else {	// else, extension is not enabled in the ini file
			if ($fileExists) {
				if ($loaded) {
					if ($awaitRestart) {
						$link = new DisplayWidget_ExtensionActionLink(DisplayWidget_ExtensionActionLink::LINK_TYPE_LOADED_LOADABLE);
						$link->setParams(array('name' => $extensionElement->getName()));
					} else {
						// Extension is built-in , extension file also exists
						$link = new DisplayWidget_Info($builtInString);
					}
				} else {	// else, not loaded
					// Not loaded and loadable
					$link = new DisplayWidget_ExtensionActionLink(DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_LOADABLE);
					$link->setParams(array('name' => $extensionElement->getName()));
				}
			} else {	// else, file not exists
				if ($loaded) {
					// Extension is built-in
					$link = new DisplayWidget_Info($builtInString);
				} else {
					// Should never reach here - extension is not loaded, file doesn't exist and nothing in the ini file
					$link = new DisplayWidget_Info('');
				}
			}
		}
		return $link;
	}
	
	/**
	 * @param integer $extStatusType
	 */
	private static function getTooltipContentByType($extStatusType) {
		switch ($extStatusType) {
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_NULL:
				return '';
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_INSTALLED:
				return Zwas_Translate::_('Not installed');
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_LOADED_UNLOADABLE:
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_LOADED_LOADABLE:
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_BUILT_IN:
				return Zwas_Translate::_('Currently turned on');
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_LOADABLE:
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_NOT_LOADABLE:
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_UNLOADABLE:
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_NO_ACTION:
				return Zwas_Translate::_('Currently turned off');
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_ERROR_FIXABLE:
				return Zwas_Translate::_('There was an error');
			case DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_ERROR:
				return Zwas_Translate::_('There was an error');
			default:
				throw new Zwas_Exception('Undefined type: ' . $extStatusType, Zwas_Exception::ASSERT);
		}
	}
	
	/**
	 * @param Extensions_ElementAbstract $extensionElement
	 * @return DisplayWidget_Info
	 */
	private static function getExtensionStatusMessage(Extensions_ElementAbstract $extensionElement) {
		return new DisplayWidget_Info($extensionElement->getErrorString());
	}
		
}