<?php

class LogTail_FileParser {
	
	const MAX_LINE_LENGTH = 128;	// estimated line length
	 
	/**
	 * @var SplFileObject
	 */
	private $fileObj;
	/**
	 * @var Zend_Session_Namespace
	 */
	private $savedState;
	/**
	 * @var array
	 */
	private $resultLines;
	/**
	 * @var integer
	 */
	private $currentRequestBeginPoint;
	/**
	 * @var integer
	 */
	private $currentRequestEndPoint;
	/**
	 * @var string
	 */
	private $lastSearchString;
	/**
	 * @var integer
	 */
	private $lastRequestBeginPoint;
	/**
	 * @var integer
	 */
	private $lastRequestEndPoint;
	/**
	 * @var integer
	 */
	private $fileEndPoint;
	
	
	public function __construct(SplFileObject $fileObj) {
		if (! $fileObj->isReadable()) {
			throw new Zwas_Exception('You do not have permission to access this log file', Zwas_Exception::ERROR);
		}
		
		$this->fileObj = $fileObj;
		$this->initState();
	}

	/**
	 * Get the first lines in the file
	 *
	 * @param integer $numberOfLines
	 * @param string $searchString
	 * @return array
	 */
	public function getFirstLines($numberOfLines, $searchString = '') {
		$this->lastRequestEndPoint = 0;
		return $this->getNextLines($numberOfLines, $searchString);
	}
	
	/**
	 * Get the first lines in the file
	 *
	 * @param integer $numberOfLines
	 * @param string $searchString
	 * @return array
	 */
	public function getLastLines($numberOfLines, $searchString = '') {
		$this->lastRequestBeginPoint = $this->fileEndPoint;
		return $this->getPrevLines($numberOfLines, $searchString);
	}
	
	/**
	 * Get the next lines
	 *
	 * @param integer $numberOfLines
	 * @param string $searchString
	 * @return array
	 */
	public function getNextLines($numberOfLines, $searchString = '') {
		$this->searchResetCheck($searchString);
		
		$this->currentRequestBeginPoint = $this->lastRequestEndPoint;
		// When clicking too many Nexts, make sure the parser doesn't go over the end of the file
		if ($this->isTheEnding($this->currentRequestBeginPoint)) {
			$this->currentRequestBeginPoint = $this->lastRequestBeginPoint;
		}
		
		$this->fileObj->fseek($this->currentRequestBeginPoint);
		$lineCounter = 0;
		$this->resultLines = array();
		
		while (($lineCounter < $numberOfLines) && (! $this->fileObj->eof())) {
			$line = $this->fileObj->fgets();
			if ((empty($searchString)) || (false !== stripos($line, $searchString))) {
				$this->resultLines[] = $line;
				++$lineCounter;
			}
		}
		
		$this->currentRequestEndPoint = $this->fileObj->ftell();
		$this->saveState();
		
		return $this->resultLines;
	}
	
	/**
	 * Get the previous lines
	 *
	 * @param integer $numberOfLines
	 * @param string $searchString
	 * @return array
	 */
	public function getPrevLines($numberOfLines, $searchString = '') {
		$this->searchResetCheck($searchString);
		
		$bufferSize = (int)(self::MAX_LINE_LENGTH * $numberOfLines);
		$this->currentRequestEndPoint = $this->lastRequestBeginPoint; 

		// if this is a request for the previous lines when the parser is already in the begining of the
		// file, read the previous buffer again		
		if ($this->isTheBegining($this->currentRequestEndPoint)) {
			$collectedLines = $this->collectLinesBetweenPointers(	$this->lastRequestBeginPoint, 
																	$this->lastRequestEndPoint);

			foreach ($collectedLines as $key => $line) {
				if (('' !== $searchString) && (false === stripos($line, $searchString))) {
					unset($collectedLines[$key]);
				}
			}
			
			$this->resultLines = array_values($collectedLines);
			
		} else {
			
			$actualLinesBrought = 0;
			$this->resultLines = array();
			
			$endPoint = $this->currentRequestEndPoint;
			$beginPoint = $endPoint - $bufferSize;
			if ($this->isTheBegining($beginPoint)) {
				$beginPoint = 0;
			}
			
			while (($actualLinesBrought < $numberOfLines) && ($beginPoint >= 0)) {
				$collectedLines = $this->collectLinesBetweenPointers($beginPoint, $endPoint);
				$collectedLinesCount = count($collectedLines);
				
				$checkedLines = array();
				
				while ((count($this->resultLines) < $numberOfLines) && ($collectedLinesCount > 0)) {
					$line = $collectedLines[ --$collectedLinesCount ];
					$checkedLines[] = $line;
					if (('' == $searchString) || (false !== stripos($line, $searchString))) {
						array_unshift($this->resultLines, $line);
					}
				}
				 
				$actualLinesBrought = count($this->resultLines);
				$endPoint -= strlen(implode('', $checkedLines));
				$beginPoint = $endPoint - $bufferSize;
				$this->currentRequestBeginPoint = ($endPoint > 0) ? $endPoint : 0;
			}
		}

		// update the file pointers if the data was collected from the file, i.e. this is not a 
		// request when the parser is already at the begining of the file
		if ($this->currentRequestBeginPoint != $this->lastRequestBeginPoint) {
			$this->saveState();
		}
		
		return $this->resultLines;
	}
	
	public function resetState() {
		$this->savedState->unsetAll();
	}
	
	/**
	 * Save pointers of the begining and end of the current result for the 
	 * next request
	 */
	private function saveState() {
		$this->savedState->lastRequestBeginPoint	= $this->currentRequestBeginPoint;
		$this->savedState->lastRequestEndPoint		= $this->currentRequestEndPoint;
		$this->savedState->lastSearchString			= $this->lastSearchString;
	}
	
	/**
	 * If the search string has changed, reset the parser
	 * @param string $searchString
	 */
	private function searchResetCheck($searchString) {
		if ($searchString != $this->lastSearchString) {
			$this->resetState();
			$this->initState();
		}
		$this->lastSearchString = $searchString;
	}
	
	/**
	 * Initializes the current state - either by collecting its existing data, or creating new data for it
	 * if it's the first run
	 */
	private function initState() {
		$this->savedState = new Zend_Session_Namespace($this->getFileNamespace());
		
		$this->fileObj->fseek(0, SEEK_END);
		$this->fileEndPoint = $this->fileObj->ftell();
		
		if ((! isset($this->savedState->lastRequestBeginPoint)) ||
			(! isset($this->savedState->lastRequestEndPoint)) ||
			(! isset($this->savedState->lastSearchString))) {
				
			$this->savedState->lastSearchString = '';
			$this->savedState->lastRequestEndPoint = 0;
			$this->savedState->lastRequestBeginPoint = $this->fileEndPoint;
		}
		
		$this->lastSearchString			= $this->savedState->lastSearchString;
		$this->lastRequestEndPoint		= $this->savedState->lastRequestEndPoint;  
		$this->lastRequestBeginPoint	= $this->savedState->lastRequestBeginPoint;
	}
	
	/**
	 * Get the complete lines in the block of the file within the begin and end points given
	 * @param integer $beginPoint
	 * @param integer $endPoint
	 * @return array
	 */
	private function collectLinesBetweenPointers($beginPoint, $endPoint) {
		if ($this->isTheBegining($beginPoint)) {
			$beginPoint = 0;
		}
		
		$this->fileObj->fseek($beginPoint, SEEK_SET);
		
		// get all lines in the current buffer range
		$tempLines = array();
		while ((! $this->fileObj->eof()) && ($this->fileObj->ftell() < $endPoint)) {
			$tempLines[] = $this->fileObj->fgets();
		}
		
		// throw away the extra line collected except for the case when the collection
		// started from the begining of the file (which is only when collecting from
		// the begining)
		if ($beginPoint != 0) {
			array_shift($tempLines);
		}
		
		return $tempLines;
	}
	
	/**
	 * Return a unique ID for this file
	 * @return string
	 */
	private function getFileNamespace() {
		return 'logParser_' . md5($this->fileObj->getFilename());
	}

	/**
	 * Is the given  position at the begining of the file
	 * @param integer $position
	 * @return boolean
	 */
	private function isTheBegining($position) {
		return ($position <= 0);
	}
	
	/**
	 * Is the given position at the end of the file
	 * @param integer $position
	 * @return boolean
	 */
	private function isTheEnding($position) {
		return ($position >= $this->fileEndPoint);
	}
	
}