<?php
class LogTail_Model {
	
	const DEFAULT_NUM_OF_LINES			= 20;
	const MIN_NUM_OF_LINES				= 5;
	const MAX_NUM_OF_LINES				= 200;
	const FOLLOW_MODE_NUM_OF_LINES		= 100;
	const DEFAULT_FOLLOW_MODE_INTERVAL	= 5;
	const MINIMAL_FOLLOW_MODE_INTERVAL	= 1;
	
	/**
	 * Get an array of available log files - depends on the system
	 * @return array
	 */
	public function getAvailableLogFiles() {
		
		$logsFile = Zwas_Path::create(ZEND_SERVER_DATA_PATH, ApplicationModel::getLogsFile());
		try {
			$logsFileContent = Context_FileSystem::getFileObject($logsFile)->readAll();
		} catch (Exception $e) {
			Zwas_Log::logMessage('Failed getting the content of the log files definitions XML file', Zend_Log::ERR);
			throw new Zwas_Exception(	new Zwas_Text(Zwas_Translate::_('An internal error has occurred, retry your last action and if that fails reinstall %s'), array(ZEND_SERVER_APPLICATION_NAME)),
										Zwas_Exception::ERROR);
		}

		$xmlValidator = new Zwas_Validate_XmlStructure();
		$xmlValidator->loadXmlString($logsFileContent)->setXsdFile(Zwas_Path::create(dirname(__FILE__), 'logfiles.xsd'));
		if (0 < $xmlValidator->validate()) {
			Zwas_Log::logMessage('The content of the log files definitions XML file is invalid', Zend_Log::ERR);
			throw new Zwas_Exception(	new Zwas_Text(Zwas_Translate::_('An internal error has occurred, retry your last action and if that fails reinstall %s'), array(ZEND_SERVER_APPLICATION_NAME)),
										Zwas_Exception::ERROR);
		}
		$xmlData = simplexml_load_string($logsFileContent);

		foreach($xmlData->logfile as $fileData) {	/* @var $fileData SimpleXMLElement */
			if (empty($fileData->name)) {
				Zwas_Log::logMessage('The content of the log files definition is missing the mandatory property "name"', Zend_Log::ERR);
				throw new Zwas_Exception(	new Zwas_Text(Zwas_Translate::_('An internal error has occurred, retry your last action and if that fails reinstall %s'), array(ZEND_SERVER_APPLICATION_NAME)),
											Zwas_Exception::ERROR);
			}
			$name		= trim($fileData->name);
			$directive	= trim($fileData->directive);
			$value		= trim($fileData->value);
			
			if ('' !== $directive) {
				$files["{$name}"] = addslashes(Context_Api::getPhpMemory()->iniGet("{$directive}"));
			} else if ('' !== $value) {
				$files["{$name}"] = addslashes("{$value}");
			}
		}
		
		return $files;
	}

    public static function getRefreshIntervalValue() {
    	
    	try {
    		$interval = ApplicationModel::getLogfileRefreshInterval();
    	} catch (Zwas_Exception $e) {
    		// Ignore the excpetion, set the default value
    		$interval = self::DEFAULT_FOLLOW_MODE_INTERVAL;
    	}
    	
    	if ($interval < self::MINIMAL_FOLLOW_MODE_INTERVAL) {
    		$interval = self::MINIMAL_FOLLOW_MODE_INTERVAL;
    	}
    	
    	return $interval;
    }
    	
	/**
	 * @param string $fileId
	 * @return array
	 */
	public function getFollowModeLines($fileId) {
		return $this->getLastLines($fileId, self::FOLLOW_MODE_NUM_OF_LINES);
	}
	
	/**
	 * @param string $fileId
	 * @param integer $numOfLines
	 * @return array
	 */
	public function getFirstLines($fileId, $numOfLines, $searchString='') {
		return $this->getParser($fileId)->getFirstLines(self::normalizeNumberOfLines($numOfLines), $searchString);
	}
	
	/**
	 * @param string $fileId
	 * @param integer $numOfLines
	 * @return array
	 */
	public function getLastLines($fileId, $numOfLines, $searchString='') {
		return $this->getParser($fileId)->getLastLines(self::normalizeNumberOfLines($numOfLines), $searchString);
	}
	
	/**
	 * @param string $fileId
	 * @param integer $numOfLines
	 * @return array
	 */
	public function getNextLines($fileId, $numOfLines, $searchString='') {
		return $this->getParser($fileId)->getNextLines(self::normalizeNumberOfLines($numOfLines), $searchString);
	}
	
	/**
	 * @param string $fileId
	 * @param integer $numOfLines
	 * @return array
	 */
	public function getPrevLines($fileId, $numOfLines, $searchString='') {
		return $this->getParser($fileId)->getPrevLines(self::normalizeNumberOfLines($numOfLines), $searchString);
	}
	
	/**
	 * Get the right parser based on the fileId given
	 *
	 * @param string $fileId
	 * @return LogTail_FileParser
	 */
	private function getParser($fileId) {
		if (is_null($fileId)) {
			throw new Zwas_Exception(Zwas_Translate::_('Please select a log file from the View Log drop-down'), Zwas_Exception::ERROR);
		}
		
		$files = $this->getAvailableLogFiles();
		
		if (! isset($files[$fileId])) {
			throw new Zwas_Exception(Zwas_Translate::_('Unrecognized log file. Please select a log file from the View Log drop-down'), Zwas_Exception::ERROR);
		}
		
		try {
			$file = Context_FileSystem::getFileObject($files[$fileId]);
			if (! $file->isFile()) {
				throw new Zwas_Exception('The file defined by the ID is not a real file', Zwas_Exception::ERROR);
			}
		} catch (Exception $e) {
			throw new Zwas_Exception(new Zwas_Text('Log file %s does not exist or missing read permissions', array($files[$fileId])), Zwas_Exception::WARNING);
		}
		if (0 == $file->getSize()) {
			throw new Zwas_Exception(new Zwas_Text('Log file %s is empty', array($files[$fileId])), Zwas_Exception::WARNING);
		}
		
		return new LogTail_FileParser($file);
	}
	
	private static function normalizeNumberOfLines($numOfLines) {
		$numOfLines = (int)$numOfLines;
		if (0 === $numOfLines) {
			$numOfLines = (int)self::DEFAULT_NUM_OF_LINES;
		}
		return $numOfLines;
	}

}