<?php

class PhpExtensions_Element extends Extensions_ElementAbstract {
	
	/**
	 * The extension's filename, e.g. extension.so or php_extesnion.dll
	 * @var string
	 */
	private $filename = '';
	
	/**
	 * Extension element constructor
	 *
	 * @param string | array $data either just the element name or an array with the following 
	 * 						possible keys - name, filename, iniFile, iniStatus, loaded, shortDescription
	 * @throws Zwas_Exception
	 */
	public function __construct($data) {
		
		if (is_string($data)) {	// we got just the filename
			$this->extensionName = $data;
			
		} else if (is_array($data)) { 
		
			if (isset($data['name'])) {
				$this->extensionName = $data['name'];
			} else {
				throw new Zwas_Exception('An ' . __CLASS__ . ' must have a name defined', Zwas_Exception::ASSERT);
			}
			
			$this->populate($data);
		} else {
			throw new Zwas_Exception('Wrong data type supplied to ' . __CLASS__ . ' constructor', Zwas_Exception::ASSERT);
		}
	}
	
	public function isInstalled() {
		return true;
	}
	
	/**
	 * Return if the element has an extension filename specified. 
	 * It doesn't check actual existance on the filesystem
	 *
	 * @return boolean
	 */
	public function hasFilename() {
		return (! empty($this->filename));
	}
	
	/**
	 * @return string
	 */
	public function getFilename() {
		return (string)$this->filename;
	}
	
	/**
	 * @param string $filename
	 */
	public function setFilename($filename) {
		$this->filename = (string)$filename;
	}

	/**
	 * @return string
	 */
	public function getErrorString() {
		$iniStatus 		= $this->getIniStatus();
		$fileExists 	= $this->fileExists();
		$loaded 		= $this->isLoaded();
		$awaitRestart	= $this->awaitsRestart();
		
		$message = '';
		
		// Default empty message
		if (Extensions_ElementAbstract::INI_ENABLED === $iniStatus) {
			if ($fileExists) {
				if (! $loaded) {
					if ($awaitRestart) {
						// Will be loaded after restart (file exists and was enabled)
						return Zwas_Translate::_('This extension\'s status will change to "On" after the PHP is restarted');
					} else {
						// Enabled and file exists but extension is not loaded - Failed to load the extension
						return Zwas_Translate::_('The system could not load this extension');
					}
				}
			} else {	// else, file not exists
				if ($loaded) {
					// Built-in and also enabled (without a reason) - should be disabled
					return Zwas_Translate::_('The extension is enabled while built-in');
				} else {	// else, not loaded
					// Extension in not loaded, file not exists but extension is enabled - should be disabled
					return Zwas_Translate::_('Extension file does not exist');
				}
			}
		} else {	// else, extension is not enabled in the ini file
			if ($fileExists && $loaded) {
				if ($awaitRestart) {
					// Will be unloaded after restart (loaded and was disabled)
					return Zwas_Translate::_('This extension\'s status will change to "Off" after the PHP is restarted');
				}
			}
		}
		
		return $message;		
	}	
	
	/**
	 * Populate the object with data from an array
	 * Available array keys: filename, iniFile, iniStatus, loaded, shortDescription
	 * @param array $data
	 */
	private function populate(array $data) {
		if (isset($data['fileExists'])) {
			$this->setFileExists((boolean)$data['fileExists']);
		}
		if (isset($data['filename'])) {
			$this->setFilename((string)$data['filename']);
		}
		if (isset($data['iniFile'])) {
			$this->setIniFile((string)$data['iniFile']);
		}
		if (isset($data['iniStatus'])) {
			$this->setIniStatus((int)$data['iniStatus']);
		}
		if (isset($data['loaded'])) {
			$this->setLoaded((boolean)$data['loaded']);
		}
		if (isset($data['shortDescription'])) {
			$this->setShortDescription((string)$data['shortDescription']);
		}
		if (isset($data['numOfDirectives'])) {
			$this->setNumberOfAssociactedDirectives((int)$data['numOfDirectives']);
		}
		if (isset($data['awaitsRestart'])) {
			$this->setAwaitsRestart((boolean)$data['awaitsRestart']);
		}
	}
	
}