<?php

class PhpExtensions_Mapper {
	
	/**
	 * @var PhpExtensions_Parser
	 */
	private $phpExtensionsParser;
	
	/**
	 * @var PhpMemory_Interface
	 */
	private $phpMemory;
	/**
	 * The value of extension_dir
	 * @var string
	 */
	private $extensionDir = '';
	
	/**
	 * array of PhpExtension_Element objects, keyed by the extension's name lowercased (e.g. 'sqlite3' [not SQLite3)
	 * @var array
	 */
	private $elementsList = array();
	/**
	 * An array with how many directives each extension has - applies only to known extensions
	 * format is lowercase(extname) => counter
	 * @var array
	 */
	private $extensionsDirectiveCount = array();
	
	/**
	 * An array with description for php extensions
	 *
	 * @var array
	 */
	private $extensionsDescription = array();
	
	/**
	 * @param string $extensionName
	 * @return Extensions_ElementAbstract
	 */
	public function getExtensionElementByExtensionName($extensionName) {
		$this->assertCacheTableNotEmpty();
		
		$row = Context_Db::getPhpExtensionsCached()->fetchExtension($extensionName);

		if ($row->count() == 1) {
			return new PhpExtensions_Element($row->current()->toArray());
		}
		
		return new Extensions_ElementNull;	
	}
	
	/**
	 * Get a list of available extensions
	 * If the list is regenerated (i.e. $revalidate==true) then
	 *   1. First collect the complete list from all known extensions in the database
	 *   2. Start creating the list of elements by finding all extensions' files (e.g. extension.so)
	 *   3. Add to that list the extensions found in the memory (loaded extensions)
	 *   4. Add to that list the extensions which have a directive set for them (e.g. extension=myextension.so)
	 * 
	 * @return array
	 */
	public function getExtensionList() {
		
		$this->elementsList = array();
		
		if(! $this->assertCacheTableNotEmpty()) {
			$this->getAvailableExtensions();
		}		
		
		$this->lowerCaseKeys();	// normalize the result array
		return $this->elementsList;
	}

	/**
	 * Mark an extension as waiting for webserver restart
	 *
	 * @param string $extensionName
	 */
	public function markExtensionsAwaitsWebserverRestart($extensionName) {
		Context_Db::getPhpExtensionsCached()->markExtensionsAwaitsWebserverRestart($extensionName);
	}
	
	/**
	 * Mark an extension as not waiting for webserver restart
	 *
	 * @param string $extensionName
	 */
	public function unmarkExtensionsAwaitsWebserverRestart($extensionName) {
		Context_Db::getPhpExtensionsCached()->unmarkExtensionsAwaitsWebserverRestart($extensionName);
	}
	
	/**
	 * Get an array of all the cached extensions' names
	 * @return array - formatted integer key => string value
	 */
	public function getExtensionsNames() {
		$this->assertCacheTableNotEmpty();
		
		$select = Context_Db::getDbAdapter()->select()->from('extensionsCached', 'name');
		return $select->query()->fetchAll(Zend_Db::FETCH_COLUMN);
	}
	
	/**
	 * @return boolean true if cache was filled, false if not
	 */
	private function assertCacheTableNotEmpty() {
		$this->extensionDir = Context_Api::getPhpMemory()->iniGet('extension_dir');
		
		// if the webserver was restarted nullify the 'awaitsRestart' field, as it may affect
		// the need of regeneration of the cache data
		if (! Context_Api::getUtilAPI()->isConfigurationChanged()) {
			$this->resetCacheTableStatusAfterRestart();
		}
		
		if ($this->cacheNeedsRegeneration()) {
			$this->phpExtensionsParser = Context_Os::getPhpExtensionsParser();
			$this->phpMemory = Context_Api::getPhpMemory();
			
			// known extensions array: the key is a <lowercase extension file basename> => <extension name>
			$listByBasename = $this->getAllKnownExtensions();
			
			$this->collectExtensionsFromExtensionsDirectory($listByBasename);
			$this->collectExtensionsFromMemory();
			$this->collectExtensionsFromIniFiles($listByBasename);
			$this->collectExtensionsNumberOfDirectives();
			$this->collectExtensionsWhichAreWaitingServerRestart();				
			$this->collectExtensionsShortDescription($listByBasename);
			
			$this->saveElementsToCacheTable();
			return true; 
		}		
		return false;
	}
	
	/**
	 * Get an array of all the extensions known in the database
	 * key is the extension's filename (e.g. accelerator)
	 * value is the extension's name (e.g. 'Zend Accelerator')
	 *
	 * @return array
	 */
	private function getAllKnownExtensions() {
		$listByBasename = array();
		$table = Context_Db::getPhpExtensionsAll();
		
		foreach($table->fetchAll() as $extension) {
			$listByBasename[ strtolower($extension->filename) ] = $extension->name; 
			$this->extensionsDirectiveCount[ $extension->name ] = $extension->numOfDirectives;
			$this->extensionsDescription[ $extension->name ] = $extension->shortDescription;
		}
		
		return $listByBasename;
	}
	
	/**
	 * Add PhpExtensions_Elements to the list of elements by finding extension files (e.g. extension.so or php_extension.dll)
	 * @param array $listByBasename
	 */
	private function collectExtensionsFromExtensionsDirectory(array &$listByBasename) {
		foreach($this->phpExtensionsParser->getDirectoryExtensions($this->extensionDir) as $filename => $nullValue) {
			$basename = $this->phpExtensionsParser->getExtensionNameFromFileName($filename);
			if (array_key_exists(strtolower($basename), $listByBasename)) {
				$extensionName = $listByBasename[strtolower($basename)];
			} else {
				// if the extension is now a known extension, we guess its name by its filename
				$extensionName = $basename;
			}
			
			$element = new PhpExtensions_Element(array(
														'name'		=> $extensionName,
														'filename'	=> $filename
													));
			$element->setFileExists(true);
			$this->elementsList[ $extensionName ] = $element;											
		}
	}
	
	/**
	 * PhpExtensions_Elements to the list of elements by scanning the loaded extensions from the memory
	 */
	private function collectExtensionsFromMemory() {
		$loadedExtensions = $this->phpExtensionsParser->getLoadedExtensions();
		
		foreach($loadedExtensions as $extensionName => $nullValue) {
			
			if (array_key_exists($extensionName, $this->elementsList)) {
				$this->elementsList[ $extensionName ]->setLoaded(true);
			} else {
				$this->elementsList[ $extensionName ] = new PhpExtensions_Element(array(
																					'name'		=> $extensionName,
																					'loaded'	=> true,
																				));
			}
		}
	}
	
	/**
	 * PhpExtensions_Elements to the list of elements by finding directives in ini files (e.g. extension=extension.so)
	 * @param array $listByBasename
	 */
	private function collectExtensionsFromIniFiles(array $listByBasename) {
		$scanner = new Configuration_IniFilesScanner();
		
		$extensionsIniFiles = $this->phpExtensionsParser->getIniExtensions($scanner);
		
		foreach($extensionsIniFiles as $filename => $iniFile) {
			$basename = $this->phpExtensionsParser->getExtensionNameFromFileName($filename);
			if (isset($listByBasename[strtolower($basename)])) {
				$extensionName = $listByBasename[strtolower($basename)];
			} else {
				// if the extension is now a known extension, we guess its name by its filename				
				$extensionName = $basename;
			}
			
			if (! array_key_exists($extensionName, $this->elementsList)) {
				$this->elementsList[ $extensionName ] = new PhpExtensions_Element(array(
																					'name'		=> $extensionName,
																				));
			}
			
			// assume the extension file (so / dll) is as mentioned in the ini file if there isnt one 
			// already (which was found through scanning the actual extensions directory
			if (! $this->elementsList[ $extensionName ]->hasFilename()) {
				$this->elementsList[ $extensionName ]->setFilename($filename);
			}
			
			$this->elementsList[ $extensionName ]->setIniFile($iniFile);
			$this->elementsList[ $extensionName ]->setIniStatus(Extensions_ElementAbstract::INI_ENABLED);
		}
	}
	
	/**
	 * For each extension add the number of directives associated with it
	 */
	private function collectExtensionsNumberOfDirectives() {
		foreach ($this->elementsList as $extension) {
			$extName = $extension->getName();
			if (isset($this->extensionsDirectiveCount[$extName])) {
				$this->elementsList[$extName]->setNumberOfAssociactedDirectives( $this->extensionsDirectiveCount[$extName] );
			}
		}
	}

	/**
	 * Keep old 'need restart' values if they are still relevant in order to make
	 * it stateful
	 */
	private function collectExtensionsWhichAreWaitingServerRestart() {
		foreach (Context_Db::getPhpExtensionsCached()->fetchAllWaitingRestart() as $waitingExtension) { /* @var $waitingExtension Zend_Db_Table_Row_Abstract */
			$extensionElement = $this->elementsList[ $waitingExtension->name ]; /* @var $extensionElement Extensions_ElementAbstract */
			// check if the current state and saved state are the same, which means the extension is still waiting for the restart
			if ($extensionElement->isLoaded() == $waitingExtension->loaded) {
				$extensionElement->setAwaitsRestart(true);
			}
		}
		
	}
	
	/**
	 * For each php extension add the short description associated with it
	 */
	private function collectExtensionsShortDescription() {
		foreach ($this->elementsList as $extension) { /* @var $extension Zem_Element */
			$extName = $extension->getName();		
			if (isset($this->extensionsDescription[$extName])) {
				$this->elementsList[$extName]->setShortDescription($this->extensionsDescription[$extName]);
			}
		}
	}
		
	/**
	 * Save the list of PhpExtensions_Elements to the database cache table
	 */
	private function saveElementsToCacheTable() {
		$tableCached = Context_Db::getPhpExtensionsCached();
		$dbAdapter = $tableCached->getAdapter();
		
		// insert data to the cache table
		$dbAdapter->beginTransaction();
		try {
			$tableCached->delete('1=1');	// empty the table
			foreach ($this->elementsList as $element) { /* @var $element Extensions_ElementAbstract */
				$row = $tableCached->createRow();
				$row->name				= $element->getName();
				$row->filename			= $element->getFilename();
				$row->fileExists		= $element->fileExists();
				$row->loaded			= $element->isLoaded();
				$row->iniStatus			= $element->getIniStatus();
				$row->iniFile			= $element->getIniFile();
				$row->awaitsRestart		= $element->awaitsRestart();
				$row->save();
			}			
		} catch (Zend_Db_Exception $e) {
			// something went wrong, rollback 
			$dbAdapter->rollBack();
			throw new Zwas_Exception(Zwas_Translate::_('Failed to access the internal database'));
		}
		$dbAdapter->commit();
	}
	
	/**
	 * Get the list of PhpExtensions_Elements from the database cache table
	 */
	private function getAvailableExtensions() {
		$select = Context_Db::getDbAdapter()->select();
		$select->from('extensionsCached')
			   ->joinLeft('extensionsAll', 'extensionsCached.name = extensionsAll.name', 
			   				array('shortDescription', 'numOfDirectives'));
			   				
		$availableExtensions = $select->query(Zend_Db::FETCH_ASSOC)->fetchAll();
		$this->elementsList = array();
		foreach ($availableExtensions as $extension) {
			$this->elementsList[ $extension['name'] ] = new PhpExtensions_Element($extension);
		}
	}

	/**
	 * Normalize the result array to have only lowercased keys
	 */
	private function lowerCaseKeys() {
		foreach($this->elementsList as $key => $object) {
			// unsetting MUST be done before putting in the lowercased key for cases like extname12
			unset($this->elementsList[$key]);
			$this->elementsList[ strtolower($key) ] = $object;
		}
	}
	
	/**
	 * Check if the cache table's timestamp is older than the ini files
	 * Once a timestamp like that is found, return true
	 * @return boolean
	 */
	private function cacheNeedsRegeneration() {
		$lastChangeTimestamp = Context_Db::getPhpExtensionsCached()->getLastChangeTimestamp();
		if (0 === $lastChangeTimestamp) {
			return true;
		}
		
		try {
			foreach(Context_Api::getPhpMemory()->getIniScannedFiles() as $iniFile) {
				try {
					$iniFileTimestamp = Context_FileSystem::getFileObject($iniFile)->getMTime();
					
					if ($iniFileTimestamp > $lastChangeTimestamp) {
						return true;
					}
				} catch (Exception $e) {
					// do nothing
				}
			}
		} catch (Exception $e) {
			// do noting, no ini scanned dir
		}
		
		try {
			$extensionsDirTimestamp = Context_FileSystem::getDirectoryObject($this->extensionDir)->getMTime();
			if ($extensionsDirTimestamp > $lastChangeTimestamp) {
				return true;
			}
		} catch (Exception $e) {
			// do nothing, no extension dir
		}
		
		return false;
	}
	
	/**
	 * Reset all 'awaitsRestart' flags in the cache table.  If any row was changed
	 * there's also a need to regenerate the cache table, so set its timestamp to 0
	 */
	private function resetCacheTableStatusAfterRestart() {
		Context_Db::getPhpExtensionsCached()->resetCacheTableStatusAfterRestart(); 
	}
}
