<?php

class PhpExtensions_Model implements Extensions_ModelInterface {
	
	/**
	 * Return list (array) of PhpExtensions_Elements display data
	 * @return array
	 */
	public function getAvailableExtensionList() {
		$mapper = new PhpExtensions_Mapper();
		return $mapper->getExtensionList();
	}
	
	/**
	 * @param string $extensionName
	 * @return Extensions_ElementAbstract|false
	 * @throws Zwas_Exception
	 */
	public function getElement($extensionName) {
		$mapper = new PhpExtensions_Mapper();
		return $mapper->getExtensionElementByExtensionName($extensionName);
	}
	
	/**
	 * Load an extension
	 * @param PhpExtensions_Element $extensionElement
	 * @throws Zwas_Exception
	 */
	public function loadExtension(Extensions_ElementAbstract $extensionElement) {
		if (! ($extensionElement instanceof PhpExtensions_Element)) {
			throw new Zwas_Exception(__FUNCTION__ . ' expects to get an instance of PhpExtensions_Element', Zwas_Exception::ASSERT);
		}
		
		if (! $extensionElement->hasFilename()) {
			throw new Zwas_Exception(Zwas_Translate::_('Could not locate the extension\'s filename'));
		}
			
		$iniFile = $this->getExtensionIniFile($extensionElement);
		
		$iniFileObject = Context_FileSystem::getFileObject($iniFile);
		$iniModifier = new Configuration_IniModifier($iniFileObject);
		$iniModifier->setExtension($extensionElement->getFilename(), true);
		$iniModifier->save();
		Context_Api::getUtilAPI()->setConfigurationChanged();
		
		// set the extension's "waiting for restart" state
		$mapper = new PhpExtensions_Mapper();
		if (! $extensionElement->isLoaded()) {		
			$mapper->markExtensionsAwaitsWebserverRestart($extensionElement->getName());
			$extensionElement->setAwaitsRestart(true);
		} else {
			$mapper->unmarkExtensionsAwaitsWebserverRestart($extensionElement->getName());
			$extensionElement->setAwaitsRestart(false);
		}
	}
	
	/**
	 * Unload an extension
	 * @param PhpExtensions_Element $extensionElement
	 * @throws Zwas_Exception
	 */
	public function unloadExtension(Extensions_ElementAbstract $extensionElement) {
		if (! ($extensionElement instanceof PhpExtensions_Element)) {
			throw new Zwas_Exception(__FUNCTION__ . ' expects to get an instance of PhpExtensions_Element', Zwas_Exception::ASSERT);
		}
		
		// there's a need to make sure there is an extension filename associated
		// with this extension in order to find the right extension to unload
		if (! $extensionElement->hasFilename()) {
			throw new Zwas_Exception(Zwas_Translate::_('Could not locate the extension\'s filename'));
		}
		
		$iniFile = $this->getExtensionIniFile($extensionElement);
		
		$iniFileObject = Context_FileSystem::getFileObject($iniFile);
		$iniModifier = new Configuration_IniModifier($iniFileObject);
		$iniModifier->setExtension($extensionElement->getFilename(), false);
		$iniModifier->save();
		Context_Api::getUtilAPI()->setConfigurationChanged();
		
		// set the extension's "waiting for restart" state
		$mapper = new PhpExtensions_Mapper();
		if ($extensionElement->isLoaded()) {
			$mapper->markExtensionsAwaitsWebserverRestart($extensionElement->getName());
			$extensionElement->setAwaitsRestart(true);
		} else {
			$mapper->unmarkExtensionsAwaitsWebserverRestart($extensionElement->getName());
			$extensionElement->setAwaitsRestart(false);
		}
	}

	/**
	 * Get the ini file where the extensions is defined
	 * Cascade from an extension's defined ini file, to an ini file with the same name as the extension, 
	 * to the fallback of php.ini
	 *
	 * @param PhpExtensions_Element $extensionElement
	 * @return string
	 */
	public function getExtensionIniFile(PhpExtensions_Element $extensionElement) {
		$iniFile = $extensionElement->getIniFile();

		if (empty($iniFile)) {
			$phpMemory = Context_Api::getPhpMemory();
			
			$extensionFileName = $extensionElement->getFilename();
			$extensionName = Context_Os::getPhpExtensionsParser()->getExtensionNameFromFileName($extensionFileName);
			
			
			if ($extensionName) {
				$guessedIniFile = Zwas_Path::create($phpMemory->getIniScannedDirectory(), $extensionName . '.ini');
				try {
					Context_FileSystem::getFileObject($guessedIniFile);
					$iniFile = $guessedIniFile;
				} catch (Exception $e) {
					// do nothing, iniFile stays null
				}
			}
			
			// fallback to default php.ini
			if (empty($iniFile)) {
				$iniFile = $phpMemory->getPhpIniFileLocation();
			}		
		}
		return $iniFile;
	}

	public function getExtensionsNames() {
		$mapper = new PhpExtensions_Mapper();
		return $mapper->getExtensionsNames();
	}
}