<?php

class Zem_Mapper {
	
	/**
	 * array of Zem_Element objects, keyed by the extension's name lowercased (e.g. 'sqlite3' [not SQLite3)
	 * @var array
	 */
	private $elementsList = array();
	
	/**
	 * Return a list (array) of Zend extensions names (e.g. array('zend utils' => 'Zend Utils') )
	 * @return array
	 */
	public function getZemExtensionNames() {
		$list = array();
		
		foreach(Context_Db::getZemExtensionsAll()->fetchAll() as $zemExtRow) { /* @var $zemExtRow Zend_Db_Table_Row */
			$name = $zemExtRow->name;
			$list[ strtolower($name) ] = $name;
		}
		
		return $list;
	}
	
	/**
	 * Return array of Zem_Elements.
	 *	- Itarate all known Zem Elements and find if its loaded to memory:
	 * 		- create the element from memory,
	 * 		- or from db if not loaded.
	 *	- Add to every element it's ini file (if exist).
	 *	- Save all data to cache.
	 *
	 * @return array
	 */
	public function getZemExtensionList() {
		$this->elementsList = array();
		
		if (! $this->assertCacheTableNotEmpty()) {
			$this->getAvailableExtensions();
		}
		
 		return $this->elementsList;
	}
	
	/**
	 * @param string $name
	 * @return Extensions_ElementAbstract
	 */
	public function getZemExtensionByName($extensionName) {
		$this->assertCacheTableNotEmpty();
		
		$where = Context_Db::getDbAdapter()->quoteInto('zemExtensionsCached.name = ?', $extensionName);
		
		$select = Context_Db::getDbAdapter()->select();
		$select->from('zemExtensionsCached')
				->joinLeft('zemExtensionsAll', 'zemExtensionsCached.name=zemExtensionsAll.name', array('numOfDirectives', 'shortDescription' ))
				->where($where)
				->limit(1);
		
		$result = $select->query(Zend_Db::FETCH_ASSOC)->fetchAll();

		if (isset($result[0]) && is_array($result[0])) {
			return Zem_Element::getElementFromDbDetails($result[0]);
		}
		return new Extensions_ElementNull();
	}

	/**
	 * Return if the extension is part of the installed package
	 * @param string $name
	 * @return boolean
	 */
	public function isComponentPartOfPackage($extensionName) {
		return Context_Db::getZemExtensionsAll()->isComponentPartOfPackage($extensionName);
	}
	
	/**
	 * Mark an extension as waiting for webserver restart
	 *
	 * @param string $extensionName
	 */
	public function markExtensionsAwaitsWebserverRestart($extensionName) {
		Context_Db::getZemExtensionsCached()->markExtensionsAwaitsWebserverRestart($extensionName);
	}
	
	/**
	 * Mark an extension as not waiting for webserver restart
	 *
	 * @param string $extensionName
	 */
	public function unmarkExtensionsAwaitsWebserverRestart($extensionName) {
		Context_Db::getZemExtensionsCached()->unmarkExtensionsAwaitsWebserverRestart($extensionName);
	}

	/**
	 * Get the default ini of the extension which was defined in the DD file, if it exists in the filesystem
	 * @param string $extensionName
	 * @return string
	 */
	public function getExtensionDefaultIniFile($extensionName) {
		$row = Context_Db::getZemExtensionsAll()->find($extensionName)->current();
		if (! is_null($row)) {
			$iniFile = $row->defaultIni;
			$scanDir = Context_Api::getPhpMemory()->getZendIniScandir();
			
			try {
				$iniFilePath = Zwas_Path::create($scanDir, $iniFile);
				$file = Context_FileSystem::getFileObject($iniFilePath);
				return $iniFilePath;
			} catch (Exception $e) {
				// do nothing, the default ini file doesnt exist
			}
		}
		return '';
	}
	
	/**
	 * Make sure the cache is not empty, if it is, fill it
	 * @return boolean true if the cache was filled; false if wasn't
	 */
	private function assertCacheTableNotEmpty() {
		if (! Context_Api::getUtilAPI()->isConfigurationChanged()) {
			$this->resetCacheTableStatusAfterRestart();
		}
		
		if ($this->cacheNeedsRegeneration()) {
			$listByBasename = $this->getAllKnownZemExtensions();
			$this->collectAllKnownZemExtensions($listByBasename);
			$this->collectExtensionsFromMemory($listByBasename);
			$this->collectExtensionsFromIni($listByBasename);
			$this->collectExtensionsFromFolders($listByBasename);
			$this->collectExtensionsWhichAreWaitingServerRestart();
			$this->saveElementsToCacheTable();
			
			return true;
		}
		return false;
	}
	
	/**
	 * Get the list of Zem_Elements from the database cache table
	 */
	private function getAvailableExtensions() {
		$select = Context_Db::getDbAdapter()->select();
		$select->from('zemExtensionsCached')
				->joinLeft('zemExtensionsAll', 'zemExtensionsCached.name=zemExtensionsAll.name');

		$availableZemExtensions = $select->query(Zend_Db::FETCH_ASSOC)->fetchAll();
		foreach ($availableZemExtensions as $rowArray) {
			$this->elementsList[ strtolower($rowArray['name']) ] = Zem_Element::getElementFromDbDetails($rowArray);
		}
	}
	
	/**
	 * Fill the elements list with all known Zend Server components
	 * @param array $listByBasename
	 */
	private function collectAllKnownZemExtensions(array $listByBasename) {
		foreach ($listByBasename as $basename => $extensionDetails) {
			$element = new Zem_Element(	$extensionDetails['name'],
										$extensionDetails['identifier'],
										$extensionDetails['subPath']);
										
			$element->setNumberOfAssociactedDirectives($extensionDetails['numOfDirectives']);
			$element->setShortDescription($extensionDetails['shortDescription']);
			$element->setInstalled(false);
			
			$this->elementsList[ $basename ] = $element;
		}
	}
	
	private function collectExtensionsFromIni(array $listByBasename) {
		$iniFileList = $this->getIniFilesList();
		
		foreach($listByBasename as $basename => $extensionDetails) {
			$identifier = $extensionDetails['identifier'];

			if (isset($iniFileList[$identifier])) {
				$iniFile = $iniFileList[$identifier];
				$iniStatus = Extensions_ElementAbstract::INI_ENABLED;
				$installed = true;
				
				// if there is no zem extension for this, create a new one based on the
				// fact that it is mentioned in an ini file
				if (! isset($this->elementsList[$basename])) {
					$this->elementsList[$basename] = new Zem_Element(	$extensionDetails['name'],
																		$identifier,
																		$extensionDetails['subPath']);
				}
				$this->elementsList[$basename]->setInstalled(true);
			} else {
				$iniFile = '';
				$iniStatus = Extensions_ElementAbstract::INI_NOT_EXIST;
			}
			
			// it is possible to still not have a Zem_element at this point (for example
			// an element which appears in the All table but is not installed
			if (isset($this->elementsList[$basename])) {
				$this->elementsList[$basename]->setIniFile($iniFile);
				$this->elementsList[$basename]->setIniStatus($iniStatus);
			}
		}
	}
	
	private function collectExtensionsFromMemory($listByBasename) {
		$memoryZemExtensionsList = Context_Api::getUtilAPI()->getExtensionsInfo();
		
		foreach($memoryZemExtensionsList as $memoryDetails) {
			if ($memoryDetails['extension_name'] == '') {
				// Memory details exists for the extension but doesn't contain the element name (usualy due to a problem during extension loading)
				// Find the extension name from the data list according to the extension id
				$extensionName = $this->findExtensionNameFromList($listByBasename, $memoryDetails['extension_id']);
				if (! $extensionName) {
					// TODO: Extension failed to load (probably) and is not identified, we may want to notify/log this event ot the user
					continue;
				}
				$memoryDetails['extension_name'] = $extensionName;
			}
			$this->createZemElementFromMemory($memoryDetails);
		}
	}
	
	/**
	 * Find and return an extension name from the list of all extensions data according to the extension identifier
	 *
	 * @param array $listByBasename
	 * @param string $extensionId Extension identifier
	 * @return string Extension name or null if identifier is not found in list
	 */
	private function findExtensionNameFromList($listByBasename, $extensionId) {
		foreach($listByBasename as $extensionDetails) {
			if ($extensionDetails['identifier'] == $extensionId) {
				return $extensionDetails['name'];
			}
		}
		return null;
	}

	/**
	 * Create Zem_Elements which appear in the All table and for which it
	 * is possible to identify the default folder's existance
	 *
	 * @param array $listByBasename
	 */
	private function collectExtensionsFromFolders(array $listByBasename) {
		$libPath = Zwas_Path::create(ZEND_SERVER_INSTALL_PATH, 'lib');
		
		foreach($listByBasename as $basename => $extensionDetails) {
			if (! isset($this->elementsList[$basename])) {
				$folder = Zwas_Path::create($libPath, $extensionDetails['subPath']);
				
				try {
					$folderObject = Context_FileSystem::getDirectoryObject($folder);
					if ($folderObject->isDir()) {
						$element = new Zem_Element(	$extensionDetails['name'],
													$extensionDetails['identifier'],
													$extensionDetails['subPath']);
						$element->setLoaded(false);
						$element->setIniStatus(Extensions_ElementAbstract::INI_NOT_EXIST);
						$element->setFileExists(true);
						$element->setInstalled(true);
						
						$this->elementsList[$basename] = $element;
					}
					
				} catch (Zwas_Exception $e) {
					// do nothing, the folder was not located
				}
			} else {
				$element = $this->elementsList[$basename]; /* @var $element Zem_Element */
				$folder = Zwas_Path::create($libPath, $element->getFolder());
				
				try {
					$folderObject = Context_FileSystem::getDirectoryObject($folder);
					if ($folderObject->isDir()) {
						$element->setFileExists(true);
						$element->setInstalled(true);
					}
				} catch (Zwas_Exception $e) {
					// do nothing, the folder was not located
				}
			}
		}
	}
	
	/**
	 * Keep old 'need restart' values if they are still relevant in order to make
	 * it stateful
	 */
	private function collectExtensionsWhichAreWaitingServerRestart() {
		foreach (Context_Db::getZemExtensionsCached()->fetchAllWaitingRestart() as $waitingExtension) { /* @var $waitingExtension Zend_Db_Table_Row_Abstract */
			$extensionElement = $this->elementsList[ strtolower($waitingExtension->name) ]; /* @var $extensionElement Extensions_ElementAbstract */
			// check if the current state and saved state are the same, which means the extension is still waiting for the restart
			if ($extensionElement->isLoaded() == $waitingExtension->loaded) {
				$extensionElement->setAwaitsRestart(true);
			}
		}
		
	}
		
	/**
	 * Save the list of Zem_Elements to the database cache table
	 */
	private function saveElementsToCacheTable() {
		$tableCached = Context_Db::getZemExtensionsCached();
		$dbAdapter = Context_Db::getDbAdapter();
		
		$dbAdapter->beginTransaction();
		try {
			$tableCached->delete('1=1');	// empty the table
			foreach ($this->elementsList as $element) { /* @var $element Zem_Element */
				$row = $tableCached->createRow();
				
				$row->name				= $element->getName();
				$row->identifier		= $element->getIdentifier();
				$row->installed			= $element->isInstalled();
				$row->loaded			= $element->isLoaded();
				$row->folder			= $element->getFolder();
				$row->folderExists		= $element->fileExists();
				$row->iniFile			= $element->getIniFile();
				$row->iniStatus			= $element->getIniStatus();
				$row->version			= $element->getVersion();
				$row->errorString		= $element->getErrorString();
				$row->awaitsRestart		= $element->awaitsRestart();
				$row->save();
			}
		} catch (Zend_Db_Exception $e) {
			// something went wrong, rollback
			$dbAdapter->rollBack();
			throw new Zwas_Exception(Zwas_Translate::_('Failed to access the internal database'));
		}
		$dbAdapter->commit();
	}
	
	/**
	 * Create list of zem directives from the scanned *.ini files. the format of the array:
	 *   array( element_identifier => ini_file_path, ...)
	 *
	 * @return array
	 */
	private function getIniFilesList() {
		$iniList	= array();
		$scanner 	= new Configuration_IniFilesScanner();
		$scanedList = $scanner->getZemExtensions();
		
		if (is_array($scanedList)) {
			foreach ($scanedList as $iniFile => $zemValues) {
				foreach ($zemValues as $identifier => $path) {
					$iniList[ $identifier ] = $iniFile;
				}
			}
		}
		return $iniList;
	}
	
	private function getAllKnownZemExtensions() {
		$listByBasename = array();
		$table = Context_Db::getZemExtensionsAll();
		
		foreach($table->fetchAll() as $extension) { /* @var $extension Zend_Db_Table_Row */
			$listByBasename[ strtolower($extension->name) ] = $extension->toArray();
		}
		return $listByBasename;
	}
	
	/**
	 * @param array $memoryDetails
	 */
	private function createZemElementFromMemory($memoryDetails) {
		$subPath = basename(dirname(dirname($memoryDetails['full_path'] )));
		$lowerCasedName = strtolower($memoryDetails['extension_name']);
		
		if (isset($this->elementsList[ $lowerCasedName ])) {
			$element = $this->elementsList[ $lowerCasedName ]; /* @var $element Zem_Element */
			$element->setIdentifier($memoryDetails['extension_id']);
			$element->setFolder($subPath);
		} else {
			$element = new Zem_Element($memoryDetails['extension_name'], $memoryDetails['extension_id'], $subPath);
		}
		$element->setVersion($memoryDetails['version']);
		$element->setLoaded(true);	// If it's created from memoty, it means the extension is loaded
		
		if ($memoryDetails['loaded_error_message']) {
			$errorString = $memoryDetails['loaded_error_message'];
			
		} elseif ($memoryDetails['licensed_error_message']) {
			$errorString = $memoryDetails['licensed_error_message'];
			
		} else {
			$errorString = $memoryDetails['enabled_error_message'];
		}
		$element->setErrorString($errorString);
		$element->setInstalled(true);
		
		$this->elementsList[ $lowerCasedName ] = $element;
	}
	
	/**
	 * Check if the cache table's timestamp is older than the ini files
	 * @return boolean
	 */
	private function cacheNeedsRegeneration() {
		$lastChangeTimestamp = Context_Db::getZemExtensionsCached()->getLastChangeTimestamp();
		if (0 === $lastChangeTimestamp) {
			return true;
		}
		
		try {
			foreach(Context_Api::getPhpMemory()->getIniScannedFiles() as $iniFile) {
				try {
					$iniFileTimestamp = Context_FileSystem::getFileObject($iniFile)->getMTime();
					if ($iniFileTimestamp > $lastChangeTimestamp) {
						return true;
					}
				} catch (Exception $e) {
					// do nothing
				}
			}
		} catch (Exception $e) {
			// do nothing, no scandir
		}
		
		return false;
	}

	/**
	 * Normalize the result array to have only lowercased keys
	 */
	private function lowerCaseKeys() {
		foreach($this->elementsList as $key => $object) {
			// unsetting MUST be done before putting in the lowercased key for cases like extname12
			unset($this->elementsList[$key]);
			$this->elementsList[ strtolower($key) ] = $object;
		}
	}

	/**
	 * Reset all 'awaitsRestart' flags in the cache table.  If any row was changed
	 * there's also a need to regenerate the cache table, so set its timestamp to 0
	 */
	private function resetCacheTableStatusAfterRestart() {
		Context_Db::getZemExtensionsCached()->resetCacheTableStatusAfterRestart();
	}
	
}