<?php

class Zem_Model implements Extensions_ModelInterface {

	/**
	 * @var Zem_Mapper
	 */
	protected $mapper;
	
	/**
	 * array of Zem_Element objects, keyed by the extension's name lowercased (e.g. 'Zend Accelerator')
	 * @var array
	 */
	private $elementsList;
		
	public function __construct() {
		$this->mapper = new Zem_Mapper();
	}
		
	/**
	 * Return a list (array) of Zend extensions names (e.g. array('zend utils' => 'Zend Utils') )
	 * @return array
	 */
	public function getAvailableExtensionNames() {
		return $this->mapper->getZemExtensionNames();
	}
	
	/**
	 * Return list (array) of Zem_Elements display data
	 * @return array
	 */
	public function getAvailableExtensionList() {
		return $this->mapper->getZemExtensionList();
	}
	
	/**
	 * @param string $extensionName
	 * @return Extensions_ElementAbstract
	 * @throws Zwas_Exception
	 */
	public function getElement($extensionName) {
		return $this->mapper->getZemExtensionByName($extensionName);
	}
	
	/**
	 * Return if the extension is part of the installed package
	 * @param string $extensionName
	 * @return boolean
	 */	
	public function isComponentPartOfPackage($extensionName) {
		return $this->mapper->isComponentPartOfPackage($extensionName);
	}
	
	/**
	 * Load an extension
	 * @param Zem_Element $element
	 * @return boolean does this action requires a server restart 
	 * @throws Zwas_Exception
	 */
	public function loadExtension(Extensions_ElementAbstract $extensionElement) {
		if (! ($extensionElement instanceof Zem_Element)) {
			throw new Zwas_Exception(__FUNCTION__ . ' expects to get an instance of Zem_Element', Zwas_Exception::ASSERT);
		}
		
		$iniFileObject = Context_FileSystem::getFileObject($this->getExtensionIniFile($extensionElement));
		$iniModifier = new Configuration_IniModifier($iniFileObject);
		$iniModifier->setZemExtension($extensionElement->getIdentifier(), $extensionElement->getFolder());
		$iniModifier->save();
		Context_Api::getUtilAPI()->setConfigurationChanged();
		
		// set the element's "waiting for a restart" state
		$mapper = new Zem_Mapper();
		if (! $extensionElement->isLoaded()) {		
			$mapper->markExtensionsAwaitsWebserverRestart($extensionElement->getName());
			$extensionElement->setAwaitsRestart(true);
			return true;
		} else {
			$mapper->unmarkExtensionsAwaitsWebserverRestart($extensionElement->getName());
			$extensionElement->setAwaitsRestart(false);
			return false;
		}
	}
	
	/**
	 * Unload an extension
	 * @param Zem_Element $element
	 * @throws Zwas_Exception
	 */
	public function unloadExtension(Extensions_ElementAbstract $extensionElement) {
		if (! ($extensionElement instanceof Zem_Element)) {
			throw new Zwas_Exception(__FUNCTION__ . ' expects to get an instance of Zem_Element', Zwas_Exception::ASSERT);
		}
		
		$iniFileObject = Context_FileSystem::getFileObject($this->getExtensionIniFile($extensionElement));
		$iniModifier = new Configuration_IniModifier($iniFileObject);
		$iniModifier->deleteZemExtension($extensionElement->getIdentifier());
		$iniModifier->save();
		Context_Api::getUtilAPI()->setConfigurationChanged();
		
		// set the element's "waiting for a restart" state
		$mapper = new Zem_Mapper();
		if ($extensionElement->isLoaded()) {		
			$mapper->markExtensionsAwaitsWebserverRestart($extensionElement->getName());
			$extensionElement->setAwaitsRestart(true);
			return true;
		} else {
			$mapper->unmarkExtensionsAwaitsWebserverRestart($extensionElement->getName());
			$extensionElement->setAwaitsRestart(false);
			return false;
		}
	}

	/**
	 * Get the ini file where the ZEM extension is defined
	 * Cascade from where the extension is define (if loaded) to the default extension ini file saved
	 * in the all list, to the default php.ini
	 *
	 * @param Zem_Element $extensionElement
	 * @return string
	 */
	public function getExtensionIniFile(Zem_Element $extensionElement) {
		$iniFile = $extensionElement->getIniFile();
		
		if (empty($iniFile)) {
			$iniFile = $this->mapper->getExtensionDefaultIniFile($extensionElement->getName());
			
			// fallback to default php.ini
			if (empty($iniFile)) {
				$iniFile = Context_Api::getPhpMemory()->getPhpIniFileLocation();
			}		
		}
		return $iniFile;
	}

}