<?php

abstract class ZwasComponents_Abstract implements ZwasComponents_Interface, ZwasComponents_ProbeInterface {
	
	/**
	 * @var Zem_Model
	 */
	protected $zemModel;
	/**
	 * @var Zem_Element
	 */
	protected $zemElement;

	public function __construct() {
		$extensionName = $this->getExtensionName();
		if (! $extensionName) {
			throw new Zwas_Exception('A Zwas component must have an extension name', Zwas_Exception::ASSERT);
		}
		$this->zemModel = new Zem_Model();
		$this->zemElement = $this->zemModel->getElement($extensionName);
			
		if ($this->zemElement instanceof Extensions_ElementNull) {
			// Feature 23677 - differentiate between a component which wasn't created because it's not a part of the package, and an error
			if ($this->zemModel->isComponentPartOfPackage($extensionName)) {
				$error = new Zwas_Text(Zwas_Translate::_('Could not create the proper Zend Server component named %s'), $extensionName);
				Zwas_Log::logMessage($error);
				throw new ZwasComponents_Exception(
					$error,
					ZwasComponents_Exception::TYPE_GENERAL_ERROR
				);
			}
			
			throw new ZwasComponents_Exception(
				new Zwas_Text(Zwas_Translate::_('Creating %s was aborted as it is not a part of the package'), $extensionName),
				ZwasComponents_Exception::TYPE_NOT_PART_OF_PACKAGE
			);
		}
	}
	
	/**
	 * Activate the component - make sure it is loaded
	 * Should be overwritten if any other/additional action should be performed in order to activate this component
	 *
	 * @return boolean does this action requires a server restart
	 * @throws Zwas_Exception on failure
	 */
	public function activateComponent() {
		return (boolean)$this->zemModel->loadExtension($this->zemElement);
	}
	
	/**
	 * Deactivate the component - make sure it is not loaded
	 * Should be overwritten if any other/additional action should be performed in order to activate this component
	 *
	 * @return boolean does this action requires a server restart
	 * @throws Zwas_Exception on failure
	 */
	public function deactivateComponent() {
		if ($this->isLoadedInFile()) {
			return (boolean)$this->zemModel->unloadExtension($this->zemElement);
		}
		return false;
	}

	/**
	 * Deactivate the component completely
	 * @return boolean does this action requires a server restart
	 * @throws Zwas_Exception on failure
	 */
	public function forceDeactivateComponent() {
		
		try {
			$needRestart = $this->zemModel->unloadExtension($this->zemElement);
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException('Failed unloading ' . $this->getExtensionName(), $e);
			throw new Zwas_Exception(new Zwas_Text('Could not turn off the \'%s\' component', array($this->getExtensionName())));
		}
		
		return (boolean)$needRestart;
	}
	
	/**
	 * @return Zem_Element
	 */
	public function getZemElement() {
		return $this->zemElement;
	}
	
	/**
	 * @param ZwasComponents_Abstract $component
	 * @param string $controller
	 * @param string $target
	 *
	 * @return ZwasComponents_Display
	 */
	public static function getZwasComponentDisplayObject(ZwasComponents_Abstract $component, $controller = null, $target = null) {
		
		$zemElement = $component->getZemElement();
		
		$display = new ZwasComponents_Display();
		$display->setName($zemElement->getName());
		$display->setVersion($zemElement->getVersion());
		$display->setStatus($component->getStatusInMemory());
		$display->setExtensionAction($component->getComponentActionLink());
		$display->setErrorString($component->getErrorString());
		$display->setConfigurationLink(Extensions_DisplayGenerator::getExtensionDisplay($zemElement)->getConfigurationLinkAction());
		$display->setActionContainer($component->getDisplayActionContainer($controller, $target));
		$display->setConfigureContainer($component->getDisplayConfigureContainer($controller, $target));
		$display->setMessage($component->getComponentMessage());
		$display->setShortDescription($zemElement->getShortDescription());
		
		return $display;
	}
	
	/**
	 * @return boolean
	 */
	public function isInstalled() {
		return (boolean)$this->zemElement->isInstalled();
	}

	/**
	 * @return boolean
	 */
	public function isLoadedInMemory() {
		return (boolean)$this->zemElement->isLoadedInMemory();
	}
	
	/**
	 * @return boolean
	 */
	public function isLoadedInFile() {
		return (boolean)$this->zemElement->isLoadedInFile();
	}
	
	/**
	 * Returns whether or not the component is loaded with an error (not licensed, not enabled or loading error)
	 *
	 * @return boolean
	 */
	public function isLoadedWithError() {
		if ('' != $this->zemElement->getErrorString()) {
			return true;
		}
		return false;
	}
	
	/**
	 * Returns the error that caused the component to load with error
	 *
	 * @see isLoadedWithError()
	 * @return string
	 */
	public function getLoadedErrorString() {
		return $this->zemElement->getErrorString();
	}
	
	/**
	 * @return string
	 */
	public function getComponentMessageString() {
		return $this->getComponentMessage()->getLabel();
	}
	
	/**
	 * Return whether the whole _component_ awaits a server restart (in some, this
	 * also depends on the enable directive)
	 *
	 * @return boolean
	 */
	public function awaitsRestart() {
		return $this->zemElement->awaitsRestart();
	}
	/**
	 * Returns the component error string (if exists)
	 *
	 * @return string
	 */
	protected function getErrorString() {
		return $this->zemElement->getErrorString();
	}
	
	/**
	 * @return integer
	 */
	protected function getStatusInMemory() {
		if (! $this->isInstalled()) {
			return ZwasComponents_Abstract::STATUS_NOT_INSTALLED;
		}
		if ($this->isLoadedWithError()) {
			return ZwasComponents_Abstract::STATUS_ERROR;
		} elseif ($this->isLoadedInMemory()) {
			return ZwasComponents_Abstract::STATUS_OK;
		}
		
		return ZwasComponents_Abstract::STATUS_NOT_LOADED;
	}
	
	/**
	 * Should return array object with appropriate actions
	 *
	 * @param string $controller
	 * @param string $target
	 * @return array
	 */
	protected function getDisplayActionContainer($controller = null, $target = null) {
		return array();
	}

	/**
	 * Should return array object with appropriate actions
	 *
	 * @param string $controller
	 * @param string $target
	 * @return array
	 */
	protected function getDisplayConfigureContainer($controller = null, $target = null) {
		return array();
	}
	
	/**
	 * @return DisplayWidget_ExtensionActionLink|DisplayWidget_Info
	 */
	protected function getComponentActionLink() {
		if (! $this->isInstalled()) {
			$link = $this->getNotInstalledLink();
		} elseif ($this->isLoadedWithError()) {
			$link = $this->getErrorLink();
		} elseif ($this->isActivated()) {
			$link = $this->getDeactivateLink();
		} else {
			$link = $this->getActivateLink();
		}
		return $link;
	}
	
	/**
	 * @return DisplayWidget_Info
	 */
	protected function getComponentMessage() {
		if (! $this->isInstalled()) {
			return new DisplayWidget_Info(Zwas_Translate::_('This component is not installed, for instructions see the Installation Guide'));
		}
		
		if ($this->awaitsRestart()) {
			if ($this->isLoadedInFile()) {
				return new DisplayWidget_Info(Zwas_Translate::_('This component\'s status will change to "On" after the PHP is restarted'));
			} else {
				return new DisplayWidget_Info(Zwas_Translate::_('This component\'s status will change to "Off" after the PHP is restarted'));
			}
		}
		
		if ($this->isLoadedWithError()) {
			return new DisplayWidget_ErrorInfo(new Zwas_Text(Zwas_Translate::_('Error message: %s'), array($this->zemElement->getErrorString())));
		}
		
		return new DisplayWidget_Info('');
	}
			
	/**
	 * Service function for reusing this code
	 * @return DisplayWidget_ExtensionActionLink
	 */
	protected function getActivateLink() {
		$link = new DisplayWidget_ExtensionActionLink(DisplayWidget_ExtensionActionLink::LINK_TYPE_NOT_LOADED_LOADABLE);
		$link->setController($this->getController());
		$link->setAction('Activate');
		$link->setTarget('Configuration', 'Zend-Server-Components');
		
		return $link;
	}
	
	/**
	 * Service function for reusing this code
	 * @return DisplayWidget_ExtensionActionLink
	 */
	protected function getDeactivateLink() {
		$link = new DisplayWidget_ExtensionActionLink(DisplayWidget_ExtensionActionLink::LINK_TYPE_LOADED_UNLOADABLE);
		$link->setController($this->getController());
		$link->setAction('Deactivate');
		$link->setTarget('Configuration', 'Zend-Server-Components');
		
		return $link;
	}
	
	/**
	 * Service function for reusing this code
	 * @return DisplayWidget_Abstract
	 */
	protected function getErrorLink() {
		$link = new DisplayWidget_ExtensionActionLink(DisplayWidget_ExtensionActionLink::LINK_TYPE_ERROR_FIXABLE);
		$link->setController($this->getController());
		$link->setAction('Force-Deactivate');
		$link->setTarget('Configuration', 'Zend-Server-Components');
		
		return $link;
	}
	
	/**
	 * Service function for reusing this code
	 *
	 * @return DisplayWidget_Info
	 */
	protected function getNotInstalledLink() {
		return new DisplayWidget_Info('');
	}

	/**
	 * Return the controller associated with this component
	 * @return string
	 */
	protected abstract function getController();
}
