<?php

class ZwasComponents_Debugger_Model extends ZwasComponents_Abstract {

	/**
	 * @var string
	 */
	private $directiveAllowedHosts		= 'zend_debugger.allow_hosts';
	private $directiveDenyHosts			= 'zend_debugger.deny_hosts';
	
	/**
	 * Cache
	 */
	private $currentAllowHost			= null;
	private $currentDenyHost			= null;
	
	/**
	 * @return string
	 */
	public function getExtensionName() {
		return 'Zend Debugger';
	}
	
	/**
	 * Returns whether the component's status is supposed to be on or off
	 * taking into consideration possible future restart action
	 *
	 * @return boolean
	 */
	public function isActivated() {
		if ((! $this->isLoadedWithError()) && $this->isLoadedInFile()) {
			return true;
		}
		
		return false;
	}
	
	/**
	 * Check if IP given, is include in the allow host list (include in mask)
	 *
	 * @param string $ip
	 * @return boolean
	 */
	public function isInAllowHosts($ip) {
		return $this->isIpIncludedInList($ip, $this->getAllowedHosts());
	}
	
	/**
	 * Check if IP given, is include in the deny host list (include in mask)
	 *
	 * @param string $ip
	 * @return boolean
	 */
	public function isInDenyHosts($ip) {
		return $this->isIpIncludedInList($ip, $this->getDeniedHosts());
	}
	
	
	/**
	 * Returns array of allowed host from the ini directive
	 * @return array
	 */
	public function getAllowedHosts() {
		if (is_null($this->currentAllowHost)) {
			$this->currentAllowHost = $this->getHostList($this->directiveAllowedHosts);
		}
		return $this->currentAllowHost;
	}
	
	/**
	 * Returns array of deny host from the ini directive
	 * @return array
	 */
	public function getDeniedHosts() {
		if (is_null($this->currentDenyHost)) {
			$this->currentDenyHost = $this->getHostList($this->directiveDenyHosts);
		}
		return $this->currentDenyHost;
	}
	
	/**
	 * Add a new allowed host to the directive string and saves in the ini file
	 * @param string $new
	 * @return string Normalized host
	 */
	public function addAllowedHosts($new) {
		$host = $this->normalizeIpMask($new);
		// if the host exist in the deny list, throws an exception.
		if ($this->isInDenyHosts($host)) {
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('\'%s\' is already in the denied hosts list and cannot be added to the allowed hosts list'), array($host)));
		}
		if ($this->isInAllowHosts($host)) {
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('\'%s\' is already in the allowed hosts list'), array($host)));
		}
		$this->currentAllowHost[] = $host;
		$this->saveHosts($this->currentAllowHost, $this->directiveAllowedHosts);
		
		return $host;
	}

	/**
	 * Add a new deny host to the directive string and saves in the ini file
	 * @param string $new
	 * @return string Normalized host
	 */
	public function addDenyHosts($new) {
		$host = $this->normalizeIpMask($new);
		if ($this->isInDenyHosts($host)) {
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('\'%s\' is already in the denied hosts list'), array($host)));
		}
		$this->currentDenyHost[] = $host;
		$this->saveHosts($this->currentDenyHost, $this->directiveDenyHosts);
		if ($this->isInAllowHosts($host)) {
			$this->removeAllowedHosts(array($host));
		}
		
		return $host;
	}
	
	/**
	 * Removed an array of hosts from the directive value and saves the change in the ini file
	 * @param array $hosts
	 */
	public function removeAllowedHosts(array $hosts) {
		$this->currentAllowHost = array_diff($this->getAllowedHosts(), $this->normalizeHostList($hosts));
		$this->saveHosts($this->currentAllowHost, $this->directiveAllowedHosts);
	}

	/**
	 * Removed an array of hosts from the directive value and saves the change in the ini file
	 * @param array $hosts
	 */
	public function removeDenyHosts(array $hosts) {
		$this->currentDenyHost = array_diff($this->getDeniedHosts(), $hosts);
		$this->saveHosts($this->currentDenyHost, $this->directiveDenyHosts);
	}
	
	public function removeAllAllowHosts() {
		$this->saveHosts(array(), $this->directiveAllowedHosts);
	}
	
	public function removeAllDenyHosts() {
		$this->saveHosts(array(), $this->directiveDenyHosts);
	}
	
	/**
	 * Return the controller associated with this component
	 * @return string
	 */
	protected function getController() {
		return 'Debugger';
	}
	
	/**
	* Returns an array with appropriate actions inside
	*
	* @param string $targetController
	* @param string $targetAction
	* @return array
	*/
	protected function getDisplayConfigureContainer($targetController = null, $targetAction = null) {
		$actions = array();

		if ($this->isInstalled()) {
			$manage = new DisplayWidget_Link(Zwas_Translate::_('Allowed Clients'), 'Debugger', 'Index', $targetController, $targetAction);
			$manage->setSidbarIdentifier(MenuItem::generateId('Debugger', 'Index'));
			$actions[] = $manage;
		}
		
		return $actions;
	}
	
	
	/**
	 * return hosts list from specific directive
	 *
	 * @param string $directive
	 * @return array
	 */
	private function getHostList($directive) {
		$directivesModel = new Directives_Model();
		
		$element = $directivesModel->getDirectiveByName($directive);
		$value = $element->getFileValue();
		
		if (!empty($value)) {
			$ipList = explode(',', $value);
			return $this->normalizeHostList($ipList);
		} else {
			return array();
		}
	}
	
	private function saveHosts(array $hosts, $directive) {
		$directivesModel = new Directives_Model();
		$directivesModel->saveDirectives(array($directive => implode(',', $hosts)));
	}

	private function addHost($currentList, $host, $directive) {
		if (! in_array($host, $currentList)) {
			$currentList[] = $host;
			$this->saveHosts($currentList, $directive);
		}
	}
	
	/**
	 * Normalize the IP and the mask:
	 * 10.1.2.3		=> 10.1.2.3/32
	 * 10.01.02.03	=> 10.1.2.3/32
	 * 10.1.2.3/8	=> 10.0.0.0/8
	 * 10.1.2.3/16	=> 10.1.0.0/16
	 *
	 * @param string $host
	 * @return string
	 */
	private function normalizeIpMask($host) {
		
		if (false === strpos($host, '/')) {		// no mask define, we set the mask to 32
			$host = "$host/32";
		}
		list($fullIp, $mask) = explode('/', $host);
		
		$validateIp		= new Zwas_Validate_Ip();
		$validateMask	= new Zend_Validate_Between(1, 32);
		
		if (!$validateIp->isValid($fullIp)) {
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('\'%s\' is not a valid IP address'), array($fullIp)));
		}
		if (!$validateMask->isValid($mask)) {
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('\'%s\' is not a valid network mask'), array($mask)));
		}
		
		$ip = explode('.', $fullIp);
		$ip = array_map('intval', $ip);	// normalize IPs like 01.01.01.01 to 1.1.1.1
		
		switch ($mask) {
			case 32:
				return "{$ip[0]}.{$ip[1]}.{$ip[2]}.{$ip[3]}/32";
				break;
				
			case 24:
				return "{$ip[0]}.{$ip[1]}.{$ip[2]}.0/24";
				break;
				
			case 16:
				return "{$ip[0]}.{$ip[1]}.0.0/16";
				break;
				
			case 8:
				return "{$ip[0]}.0.0.0/8";
				break;
				
			default:	// Any other user-defined masks
				return $host;
				break;
		}
	}

	/**
	 * Normalize the IPs in a host list array
	 *
	 * @param array $hosts
	 * @return array
	 */
	private function normalizeHostList(array $hosts) {
		foreach($hosts as $key => $value) {
			$hosts[$key] = $this->normalizeIpMask($value);
		}
		return $hosts;
	}
	

	/**
	 * check existing of ip in list of hosts
	 *
	 * @param string $ip 	- with/without mask
	 * @param array $list	- arrray('x.x.x.x/mask', 'y.y.y.y/mask', ..)
	 * @return boolean
	 */
	private function isIpIncludedInList($ip, $list) {
		list($nakedIp) = explode('/', $ip);
		foreach ($list as $host) {
			list($hostIp, $hostMask) = explode('/', $host);
			$validator = new Zwas_Validate_IpRange($hostIp, $hostMask);
			if ($validator->isValid($nakedIp)) {
				return true;
			}
		}
		return false;
	}
}