<?php

abstract class ZwasComponents_Enableable extends ZwasComponents_Abstract {

	/**
	 * @var string
	 */
	protected $directiveEnable = null;
	/**
	 * @var Directives_Element
	 */
	protected $directiveEnableElement = null;
	
	public function __construct() {
		parent::__construct();
		
		if (is_null($this->directiveEnable)) {
			throw new Zwas_Exception('A class extending ' . __CLASS__ . ' must set directiveEnable', Zwas_Exception::ASSERT);
		}
	}
	
	/**
	 * Activate the component - make sure it is both loaded and enabled
	 * @return boolean does this action requires a server restart
	 * @throws Zwas_Exception on failure  
	 */
	public function activateComponent() {
		$needRestart = $this->zemModel->loadExtension($this->zemElement);
		
		if (! $this->isEnabledInFile()) {
			try {
				$needRestart |= $this->enable();
			} catch (Zwas_Exception $e) {
				Zwas_Log::logException('Failed enabling ' . $this->getExtensionName(), $e);
				throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('The component %s was partially turned on.  Retry your last action and if this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center'), array($this->getExtensionName())), Zwas_Exception::ERROR);
			}
		}
		return (boolean)$needRestart;
	}
	
	/**
	 * Deactivate the component - only if it is active
	 * @return boolean does this action requires a server restart
	 * @throws Zwas_Exception on failure  
	 */
	public function deactivateComponent() {
		$needRestart = false;
		
		if ($this->isEnabledInFile() && $this->isLoadedInFile()) {
			try {
				$needRestart = $this->disable();
			} catch (Zwas_Exception $e) {
				Zwas_Log::logException('Failed disabling ' . $this->getExtensionName(), $e);
				throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('The component %s was partially turned off.  Retry your last action and if this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center'), array($this->getExtensionName())), Zwas_Exception::ERROR);
			}
			
			$needRestart |= $this->zemModel->unloadExtension($this->zemElement);
		}
		return (boolean)$needRestart;
	}
	
	/**
	 * Deactivate the component completely
	 * @return boolean does this action requires a server restart
	 * @throws Zwas_Exception on failure  
	 */
	public function forceDeactivateComponent() {
		$exception = null;
		
		try {
			$needRestart = $this->disable();
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException('Failed disabling ' . $this->getExtensionName(), $e);
			$exception = new Zwas_Exception(new Zwas_Text('Could not turn off the \'%s\' component', array($this->getExtensionName())));
		}
		
		try {
			$needRestart |= $this->zemModel->unloadExtension($this->zemElement);
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException('Failed unloading ' . $this->getExtensionName(), $e);
			$exception = new Zwas_Exception(new Zwas_Text('Could not turn off the \'%s\' component', array($this->getExtensionName())));
		}
		
		if (! is_null($exception)) {
			throw $exception;
		}
		
		return (boolean)$needRestart;
	}
	
	/**
	 * Returns whether the component's status is supposed to be on or off
	 * taking into consideration possible future restart action and current error status
	 *
	 * @return boolean
	 */	
	public function isActivated() {
		if ((! $this->isLoadedWithError()) && $this->isLoadedInFile() && $this->isEnabledInFile()) {
			return true;
		}
		
		return false;
	}
	
	/**
	 * @return boolean
	 */
	public function isEnabledInMemory() {
		return Directives_Translator::getRealMemoryValue($this->getDirectiveEnableElement());
	}

	/**
	 * @return boolean
	 */
	public function isEnabledInFile() {
		return Directives_Translator::getRealFileValue($this->getDirectiveEnableElement());
	}
	
	/**
	 * Return whether the whole _component_ awaits a server restart 
	 * (depends on the enable directive and the extension)
	 *
	 * @return boolean
	 */
	public function awaitsRestart() {
		return ($this->zemElement->awaitsRestart() || $this->getDirectiveEnableElement()->awaitsRestart());
	}
	
	/**
	 * Check if the related daemon is running
	 * @return boolean
	 * @throws Zwas_Exception
	 */
	public function isRunning() {
		return true;
	}
	
	/**
	 * @return boolean - whether or not the action requires a restart
	 * @throws Zwas_Exception
	 */
	protected function enable() {
		$directivesModel = new Directives_Model();
		return (boolean)$directivesModel->saveDirectives(array($this->directiveEnable => 1));
	}

	/**
	 * @return boolean - whether or not the action requires a restart
	 * @throws Zwas_Exception
	 */
	protected function disable() {
		$directivesModel = new Directives_Model();
		return (boolean)$directivesModel->saveDirectives(array($this->directiveEnable => 0));
	}
	
	/**
	 * @return Directives_Element
	 */
	protected function getDirectiveEnableElement() {
		if (is_null($this->directiveEnableElement)) {
			$model = new Directives_Model();
			$this->directiveEnableElement = $model->getDirectiveByName($this->directiveEnable);
		}
		return $this->directiveEnableElement;
	}	

	/**
	 * @return DisplayWidget_Info
	 */
	protected function getComponentMessage() {
		if (! $this->isInstalled()) {
			return new DisplayWidget_Info(Zwas_Translate::_('This component is not installed, for instructions see the Installation Guide'));
		}

		if ($this->awaitsRestart()) {
			if ($this->isLoadedInFile() && $this->isEnabledInFile()) {
				return new DisplayWidget_Info(Zwas_Translate::_('This component\'s status will change to "On" after the PHP is restarted'));
			} else {
				return new DisplayWidget_Info(Zwas_Translate::_('This component\'s status will change to "Off" after the PHP is restarted')); 
			}
		}
		
		if ($this->isLoadedWithError()) {
			return new DisplayWidget_ErrorInfo(new Zwas_Text(Zwas_Translate::_('Error message: %s'), array($this->zemElement->getErrorString())));
		}

		
		return new DisplayWidget_Info('');
	}
	
	/**
	 * Overwrite the parent function to check if the component is not enabled
	 * @return integer
	 */
	protected function getStatusInMemory() {
		// Special case of Enableble componenets: disabled (loaded in memory but directive is disabled)
		if ($this->isLoadedInMemory() && (! $this->isLoadedWithError()) &&
			(! $this->isEnabledInMemory())) {
			
			return ZwasComponents_Abstract::STATUS_NOT_ENABLED;
		}
		
		return parent::getStatusInMemory();
	}
	
}