<?php

class Nodes_Action {
	
	/**
	 * @var Nodes_Model
	 */
	private $nodesModel;
	
	/**
	 * @var Nodes_Communication_Mapper
	 */
	private $mapper;
	
	/**
	 * @return Nodes_Communication_Mapper
	 */
	public function getCommMapper() {
		if (is_null($this->mapper)) {
			$this->mapper = new Nodes_Communication_Mapper();
		}
		return $this->mapper;
	}
	
	/**
	 * @param Nodes_Communication_Mapper $mapper
	 */
	public function setCommMapper($mapper) {
		$this->mapper = $mapper;
	}
	
	/**
	 * @return Nodes_Model
	 */
	public function getNodesModel() {
		if (is_null($this->nodesModel)) {
			$this->nodesModel = new Nodes_Model();
		}
		return $this->nodesModel;
	}
	
	/**
	 * @param Nodes_Model $nodesModel
	 */
	public function setNodesModel($nodesModel) {
		$this->nodesModel = $nodesModel;
	}

	/**
	 * @param Zwas_Controller_Action $controller
	 * @param Nodes_Element_Mapper $node
	 */
	public static function remoteNodeAction(Zwas_Controller_Action $controller, Nodes_Element_Mapper $node) {
		$nodesAction		= new Nodes_Action();
		$request = $controller->getRequest();
		
		try {
			$response = $nodesAction->getCommMapper()->remoteAction($node, $request->getControllerName(), $request->getActionName(), $request->getParams());
			if ($response->isError()) {
				if (is_string($response->getData())) {
					throw new Zwas_Exception($response->getData());
				}
				throw new Zwas_Exception(new Zwas_Text(
					Zwas_Translate::_('Server failed to execute the request [%s]'),
					$response->getCode()
				));
			}
			$nodesAction->applyAction($controller, $response);
			
		} catch (Zwas_Exception $e) {
			
			if($e instanceof Nodes_Model_Exception) {
				$controller->setMessagePane(new MessagePane_Info($e->getMessageObject()));	
			} else {
				$controller->setMessagePane(new MessagePane_Error($e->getMessageObject()));
			}
			
			// if there is exception, we can't know if there is a template for this action
			// (the exception could thrwon before "forward" action, for example)
			// so we always render a template with message pane
			$controller->renderScript('message.phtml');
		}
	}
	
	/**
	 * @param Zwas_Controller_Action $controller
	 */
	public static function remoteAction(Zwas_Controller_Action $controller) {
		self::remoteActionCommon($controller, false);
	}
	
	/**
	 * @param Zwas_Controller_Action $controller
	 */
	public static function remoteDistributeAction(Zwas_Controller_Action $controller) {
		self::remoteActionCommon($controller, true);
	}
	
	/**
	 * Lock the UI in case the cluster has members of different Zend Server versions
	 * @param Zwas_Controller_Action $controller
	 */
	public static function lockContentOnMixedCluster(Zwas_Controller_Action $controller) {
		$nodesModel = new Nodes_Model();
		if ($nodesModel->hasDifferentEnvironmentId()) {
			$controller->setMessagePane(
				new MessagePane_Info(Zwas_Translate::_('This page\'s functionality has been disabled because the cluster has servers with different Zend Server versions'))
			);
			$controller->getHelper('ActionStack')->direct('Lock-Content', 'Index');
		}
	}
	
	/**
	 * @param string $controllerName
	 * @param string $actionName
	 * @param array $params
	 * @return Nodes_Response
	 */
	public function representativeAction($controllerName, $actionName, array $params = array()) {
		try {
			$representativeNode = $this->getNodesModel()->getRepresentativeForReading();
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException('Representative retrieval failed',$e);
			throw new Zwas_Exception('There are currently no servers connected to the Zend Server Cluster Manager. Go to Cluster Setup | Servers to add servers to the cluster');
		}
		return $this->getCommMapper()->remoteAction($representativeNode, $controllerName, $actionName, $params);
	}
	
	/**
	 * @param string $controllerName
	 * @param string $actionName
	 * @param array $params
	 * @return Nodes_Response
	 * @throws Zwas_Exception
	 */
	public function distributeAction($controllerName, $actionName, array $params = array()) {
		if (! $this->getNodesModel()->hasRespondingNodes()) {
			throw new Zwas_Exception(Zwas_Translate::_('There are no servers connected to the cluster'), Zwas_Exception::ERROR);
		}
		$this->getNodesModel()->getRepresentativeForWriting();
		$nodes = array_values($this->getNodesModel()->getRespondingNodes());
		
		return $this->getCommMapper()->distributeAction(
			$nodes,
			$controllerName,
			$actionName,
			$params
		);
	}
	
	/**
	 * Add the forward, render and renderScript to the Controller and assigns the view
	 * public for tests only
	 * @param Zwas_Controller_Action $controller
	 * @param Node_Response $response
	 */
	public function applyAction(Zwas_Controller_Action $controller, Nodes_Response $response) {
		// apply view data from the response
		$controller->view->assign( $response->getData() );
		
		// apply additional controller actions (forward, render...)
		$cp = $response->getControllerParams();
		if ($cp->hasAction()) {
			
			if ($cp->isForward()) {
				// In case the forward is a result of an MainContent::afterActionForward() call, that data should be added to 
				// the action stack in order to support refresh after restart (bug 29761)
				if ($cp->isAfterActionForward()) {
					$param = $cp->getAfterActionForward();
					$controller->getHelper('ActionStack')->direct('Layout-State', 'Index' , 'EE',
											array(	'module'		=> 'EE',
													'controller' 	=> $param['controller'],
													'action'		=> $param['action'],
											));
				}
				
				$param = $cp->getForward();
				$controller->forward($param['action'], $param['controller'], 'EE', $param['params']);
			}
			elseif ($cp->isRender()) {
				$param = $cp->getRender();
				$controller->render($param['action'], $param['name'], $param['noController']);
			}
			elseif ($cp->isRenderScript()) {
				$param = $cp->getRenderScript();
				$controller->renderScript($param['script'], $param['name']);
			}
		}
		
		if ($response->isDistributed() && $response->hasDistributedMvcErrors()) {
			$controller->setMessagePane(new MessagePane_Notice(
				Zwas_Translate::_('Some cluster members could not complete the requested action. If you are in the process of upgrading your cluster, complete the upgrade before performing any other actions')
			));
		}
	}
		
	
	/**
	 * @param Zwas_Controller_Action $controller
	 * @param boolean $distribute
	 */
	private static function remoteActionCommon(Zwas_Controller_Action $controller, $distribute) {
		try {
			
			$nodesAction		= new Nodes_Action();
			$nodesModel 		= $nodesAction->getNodesModel();
			/// if no nodes are present, exit immediately
			if (!$nodesModel->hasRespondingNodes()) {
				if ($nodesModel->hasNodes()) {
					throw new Nodes_Model_Exception(
						Zwas_Translate::_('No response detected from cluster members. For more details, go to Cluster Setup | Servers')
					);
				} else {
					throw new Nodes_Model_Exception(Zwas_Translate::_('There are currently no servers connected to the Zend Server Cluster Manager. Go to Cluster Setup | Servers to add servers to the cluster'));
				}
				
			}
			
			$request			= $controller->getRequest();
			if ($distribute) {
				$response		= $nodesAction->distributeAction($request->getControllerName(), $request->getActionName(), $request->getParams());
			} else {
				$response		= $nodesAction->representativeAction($request->getControllerName(), $request->getActionName(), $request->getParams());
			}
			if ($response->isError()) {
				if (is_string($response->getData())) {
					throw new Zwas_Exception($response->getData());
				}
				
				/// trap for stale representative information or general communications failure
				if($response->getData() instanceof Communication_Queue_Exception) {
					throw new Zwas_Exception(
						Zwas_Translate::_('Unable to access an online server')
					);
				}
				
				throw new Zwas_Exception(Zwas_Translate::_('Server failed to execute the request'));
			}
			$nodesAction->applyAction($controller, $response);
			
		} catch (Zwas_Exception $e) {
			
			if($e instanceof Nodes_Model_Exception) {
				$controller->setMessagePane(new MessagePane_Info($e->getMessageObject()));	
			} else {
				$controller->setMessagePane(new MessagePane_Error($e->getMessageObject()));
			}
			
			// if there is exception, we can't know if there is a template for this action
			// (the exception could thrwon before "forward" action, for example)
			// so we always render a template with message pane
			$controller->renderScript('message.phtml');
		}
	}
		
}