<?php

class Nodes_Communication_Mapper {

	/**
	 * @var Communication_Queue
	 * @return Nodes_Communication_Mapper
	 */
	private $client;
	
	/**
	 * @param string $uri
	 * @param string $password
	 * @return Node_Environment
	 * @throws Exception
	 */
	public function probeNode($uri, $password) {
		$node = Nodes_Element_Mapper::fabricateNodeElement($uri, $password);
		$responseNode = $this->request($node, 'Node-Public', 'Probe');
		if ($responseNode->isError()) {
			switch ($responseNode->getCode()) {
				case 4001:
					$message = Zwas_Translate::_('The server address or password is incorrect. Please try again');
					break;
				case 6001:
					$message = Zwas_Translate::_('The server is already connected to a cluster');
					break;
				default:
					if (is_string($responseNode->getMessage())) {
						$message = $responseNode->getMessage();
					} else {
						$message = new Zwas_Text(Zwas_Translate::_('Unable to contact node, make sure the server is running and available'));
					}
			}
			// TODO: add error check for more verbose message
			throw new Zwas_Exception($message);
		}
		$env = $responseNode->getData();
		// if a string was returned, attempt to parse it as an XML
		if (is_string($env)) {
			$env = Node_Environment::createFromXML($env);
		}
		
		if ($env instanceof Node_Environment) {
			return $env;
		}
		throw new Zwas_Exception('Unexpected response, make sure the server is running and available');
	}
	
	/**
	 * @param string $uri
	 * @param string $password
	 * @return boolean
	 * @throws Zwas_Exception on failure
	 */
	public function lockNode($uri, $password) {
		$node = Nodes_Element_Mapper::fabricateNodeElement($uri, $password);
		$responseNode = $this->request($node, 'Node-Public', 'Connect');
		if ($responseNode->isError()) {
			
			switch ($responseNode->getCode()) {
				case 4001:
					throw new Nodes_Communication_Mapper_Exception(Zwas_Translate::_('The server address or password is incorrect. Please try again'), $responseNode->getCode());
					break;
				case 6001:
					throw new Nodes_Communication_Mapper_Exception(Zwas_Translate::_('The server is already connected to a cluster'), $responseNode->getCode());
					break;
				case 6002:
					throw new Nodes_Communication_Mapper_Exception(Zwas_Translate::_('Unable to lock the node\'s administration interface. Check that the node is available and that you have the appropriate permissions for this server'), $responseNode->getCode());
					break;
				default:
					Zwas_Log::logMessage('Unable to communicate with the node when trying to lock it', $responseNode->getCode());
					throw new Nodes_Communication_Mapper_Exception(Zwas_Translate::_('Unable to communicate with the server'), $responseNode->getCode());
					break;
			}
		}
		return true;
	}
	
	/**
	 * @param string $uri
	 * @param string $password
	 * @return boolean
	 * @throws Zwas_Exception
	 */
	public function unlockNode($uri, $password) {
		$node = Nodes_Element_Mapper::fabricateNodeElement($uri, $password);
		$responseNode = $this->request($node, 'Node', 'Disconnect');
		if ($responseNode->isError()) {
			
			switch ($responseNode->getCode()) {
				case 4001:
					throw new Nodes_Communication_Mapper_Exception(Zwas_Translate::_('Authentication error. Check your password settings'), $responseNode->getCode());
					break;
				case 6003:
					throw new Nodes_Communication_Mapper_Exception(Zwas_Translate::_('Unable to restore the server to standalone mode'), $responseNode->getCode());
					break;
					
				default:
					Zwas_Log::logMessage('Unable to communicate with the server while trying to unlock it', $responseNode->getCode());
					throw new Nodes_Communication_Mapper_Exception(Zwas_Translate::_('Unable to communicate with the server'), $responseNode->getCode());
					break;
			}
		}
		return true;
	}
	
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @param string $controllerName
	 * @param string $actionName
	 * @param array $params
	 * @return Nodes_Response
	 */
	public function remoteAction(Nodes_Element_Mapper $node, $controllerName, $actionName, array $params = array()) {
		return $this->request($node, $controllerName, $actionName, $params);
	}
	
	/**
	 * @param array $nodes with Nodes_Element_Mapper elements
	 * @param string $controllerName
	 * @param string $actionName
	 * @param array $params
	 * @return Nodes_Response_Collection
	 * @throws Zwas_Exception
	 */
	public function distributeActionGetResponses(array $nodes, $controllerName, $actionName, array $params = array()) {
		if (0 === count($nodes)) {
			throw new Zwas_Exception('The cluster has no nodes', Zwas_Exception::ERROR);
		}
	
		return $this->requests($nodes, $controllerName, $actionName, $params);
	}
	
	/**
	 * Perform a distributed action on the entire cluster and retrieve only the representative response.
	 * 
	 * @param array $nodes with Nodes_Element_Mapper elements
	 * @param string $controllerName
	 * @param string $actionName
	 * @param array $params
	 * @return Nodes_Response
	 * @throws Zwas_Exception
	 */
	public function distributeAction(array $nodes, $controllerName, $actionName, array $params = array()) {
		$nodesResponses = $this->distributeActionGetResponses($nodes, $controllerName, $actionName, $params);
		try {
			return $nodesResponses->getRepresentativeResponse();
		} catch (Zwas_Collection_Exception $e) {
			throw new Zwas_Exception(Zwas_Translate::_('An internal error has occurred, retry your last action and if this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center'));
		}
	}
	
	/**
	 * @param Communication_Queue_Abstract $client
	 * @return Nodes_Communication_Mapper
	 */
	public function setClient(Communication_Queue_Abstract $client) {
		$this->client = $client;
		return $this;
	}

	/**
	 * @return Communication_Queue_Abstract
	 */
	public function getClient() {
		if (null == $this->client) {
			$this->client = Communication_Queue :: getCommunicationClient();
		}
		return $this->client;
	}
	
	/**
	 * @param string $uri
	 * @param string $password
	 * @param string $controller
	 * @param string $action
	 * @param array  $params
	 * @return Nodes_Response
	 * @throws Zwas_Excpetion
	 */
	private function request(Nodes_Element_Mapper $node, $controller, $action, array $params = array()) {
		$client	= $this->getClient();
		
		$params['controller']	= $controller;
		$params['action']		= $action;
		$params['module']		= 'PE';
		$params['nodekey'] 		= $node->getPassword();
		
		if ( empty($params['nodekey']) ){
			return Node_Response::factory(
				Zwas_Translate::_('An internal error has occurred, retry your last action and if this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center'), 
				7000, 
				'Missing a password in order to send the request'
			);
		}

		$requestObject = $client->makeRequest($params, $node->getAddress());
		$requestObject->setNodeId($node->getId());
		$client->attachRequest($requestObject);
		
		try {
			return $client->sendRequest();
		} catch (Communication_Queue_Exception $e) {
			Zwas_Log::logException((string)$e->getMessageObject(), $e);
			// TRANSLATE
			throw new Zwas_Exception('Could not communicate with the server, retry your last action and if this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center');
		}
	}
	
	/**
	 * @param array $nodes
	 * @param array $controllerName
	 * @param $actionName
	 * @param $params
	 * @return Nodes_Response_Collection
	 * @throws Zwas_Excpetion if authentication failed
	 */
	private function requests(array $nodes, $controllerName, $actionName, array $params = array()) {
		$client	= $this->getClient();
		
		$params['controller']	= $controllerName;
		$params['action']		= $actionName;
		$params['module']		= 'PE';
		foreach ($nodes as $node) { /* @var $node Nodes_Element_Mapper */
			
			if (! $node instanceof Nodes_Element_Mapper) {
				throw new Zwas_Exception('Invalid node passed', Zwas_Exception::ASSERT);
			}
			
			$params['nodekey'] = $node->getPassword();
			if ( empty($params['nodekey']) ){
				return Node_Response::factory(
					Zwas_Translate::_('An internal error has occurred, retry your last action and if this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center'), 
					7000, 
					'Missing a password in order to send the request');
			}
			
			$request = $client->makeRequest($params, $node->getAddress(), $node->isRepresentative());
			$request->setNodeId($node->getId());
			$client->attachRequest($request);
		}
		
		return new Nodes_Response_Collection($client->sendRequests());
	}
	
}