<?php
class Nodes_Extension_Table extends Zend_Db_Table_Abstract {
	
	protected $_name		= 'extension';
	protected $_primary		= array('nodeId','name');
	protected $_rowClass	= 'Nodes_Extension_Element';
	
	/**
	 * @param array $extensions array of Node_Extension_Container elements
	 * @param integer $nodeId
	 */
	public function addExtensions(array $extensions, $nodeId) {
		// remove all existing entries
		$this->removeAllNodeExtensions($nodeId);
		
		$sql = <<<SQL
		INSERT INTO extension 
			(nodeId, name , filename , fileExists , loaded , iniStatus , iniFile ,
				awaitsRestart, errorString, extensionHash, numOfDirectives) 
			VALUES
			(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);
SQL;

		$stmt = $this->getAdapter()->prepare($sql);

		// @see mantis bug 31014 - using the transaction and prepared statement is used in order to speed things up
		$this->getAdapter()->beginTransaction();
		foreach ($extensions as $extension) { /* @var $extension Node_Extension_Container */
			if ($extension instanceof Node_Extension_Container) {
				$stmt->execute(array(
					$nodeId,
					$extension->getName(), 
					$extension->getFilename(), 
					$extension->fileExists(), 
					$extension->isLoaded(), 
					$extension->getIniStatus(), 
					$extension->getIniFile(), 
					$extension->awaitsRestart(),
					$extension->getErrorString(),
					Nodes_Extension_Element::generateHash($extension),
					$extension->getNumberOfDirectives(),
				));
			}
		} 
		$this->getAdapter()->commit();
	}
	
	/**
	 * @param integer $nodeId
	 * @return integer the number of rows deleted
	 */
	public function removeAllNodeExtensions($nodeId) {
		return $this->delete(array('nodeId = ?' =>  $nodeId));
	}

	/**
	 * @param string $extensionName
	 * @return Zend_Db_Table_Rowset_Abstract
	 */
	public function getExtension($extensionName) {
		$select = $this->select();
		return $this->fetchAll($select->where('name = ?', $extensionName));
	}
	
	/**
	 * @param string $extensionName
	 * @return Zend_Db_Table_Rowset_Abstract
	 */
	public function getExtensionsWithMessagesByExtensionName($extensionName) {
		$select = $this->select();
		$select->where('name = ?', $extensionName);
		$select->where('errorString != \'\'');
		return $this->fetchAll($select);
	}
	
	/**
	 * @return Zend_Db_Table_Rowset_Abstract
	 */
	public function getExtensionsWithMessages() {
		$select = $this->select();
		$select->where('errorString != \'\'');
		return $this->fetchAll($select);
	}
	
	/**
	 * @param string $extensionName
	 * @return Nodes_Extension_Element
	 */
	public function getNodeExtension($extensionName, $nodeId) {
		$select = $this->select();
		$select->where('nodeId = ?', $nodeId);
		$select->where('name = ?', $extensionName);
		
		$extensionRow = $this->fetchRow($select);
		if (! ($extensionRow instanceof Nodes_Extension_Element)) {
			$extensionRow = $this->createRow();
		}
		return $extensionRow;
	}
	
	/**
	 * @param integer $nodeId
	 * @return Zend_Db_Table_Rowset_Abstract
	 */
	public function getNodeExtensions($nodeId) {
		$select = $this->select();
		$select->where('nodeId = ?', $nodeId);
		
		return $this->fetchAll($select);
	}
	
	/**
	 * @param string $extensionName
	 * @return boolean
	 */
	public function isExtensionUnstable($extensionName) {
		$select = $this->select()
			->from($this->_name, array('counter' => 'COUNT(DISTINCT "extensionHash")', 'name'))
			->where('name = ?', $extensionName)
			->group('extension.name');
		
		return (1 < (int)$select->query()->fetchColumn(0));
	}
}
