<?php

class Nodes_Mapper {

	/**
	 * Create or retrieve an existing environment based on the Node's environment object
	 * @param Node_Environment $nodeEnvironment
	 * @return Nodes_Environment_Element
	 */
	public function addEnvironment(Node_Environment $nodeEnvironment) {
		$nodesEnvironmentElement = Nodes_Environment_Element::fromNodeEnvironment($nodeEnvironment);
		return Context_Db::getNodesEnvironmentTable()->addEnvironment($nodesEnvironmentElement);
	}
	
	/**
	 * @param $environmentId
	 * @return Nodes_Environment_Element
	 */
	public function getEnvironmentById($environmentId) {
		$table = Context_Db::getNodesEnvironmentTable();
		return $table->fetchRow($table->getAdapter()->quoteInto('environmentId = ?', $environmentId));
	}
	
	/**
	 * @return Nodes_Element_Mapper|null
	 */
	public function getLastRespondingServer() {
		$lastServer = Context_Db::getNodesTable()->getLastRespondingServer();
		if (is_null($lastServer)) {
			return null;
		}
		return new Nodes_Element_Mapper($lastServer);
	}
	
	/**
	 * @return boolean
	 */
	public function hasDifferentEnvironmentId() {
		return Context_Db::getNodesTable()->hasDifferentEnvironmentId();
	}
	
	/**
	 * @return boolean
	 */
	public function hasNodesInDisableTransition() {
		return Context_Db::getNodesTable()->hasNodesInDisableTransition();
	}
	
	/**
	 * @return boolean
	 */
	public function hasNodesInRemovalTransition() {
		return Context_Db::getNodesTable()->hasNodesInRemovalTransition();
	}
	
	/**
	 * @return integer
	 */
	public function getRepresentativeEnvironmentId() {
		return Context_Db::getNodesTable()->getRepresentativeEnvironmentId();
	}
	
	/**
	 * Return the a representative node
	 * @return Nodes_Element_Mapper
	 * @throws Zwas_Exception
	 */
	public function getRepresentative() {
		return new Nodes_Element_Mapper(Context_Db::getNodesTable()->getRepresentative());
	}
	
	/**
	 * Return a candidate to standin for the representative
	 * This method attempts to retrieve an online node that has the same signature
	 * as the original representative.
	 * 
	 * Note that if the representative is online, this method will not return the
	 * current representative
	 * @return Nodes_Element_Mapper|null
	 * @throws Zwas_Exception
	 * @throws Zend_Db_Exception
	 */
	public function getAlternativeRepresentative() {
		$nodesModel = new Nodes_Model();
		if (! $nodesModel->countNodes()) {
			throw new Zwas_Exception(Zwas_Translate::_('The cluster has no servers'));
		}
		
		$realRepresentative = new Nodes_Element_Mapper(Context_Db::getNodesTable()->getCurrentRepresentative());

		$signatureNameMaster = Node_HeartbeatGenerator::SIGNATURE_NAME_MASTER;
		// collect Nodes with a signature that matches
		$select = $this->getAdapter()->select();
		$select->from('nodes', 'id')
				->join('nodesSignature', "nodesSignature.nodeId = nodes.id AND nodesSignature.sigName = '$signatureNameMaster'")
				->where('sigValue = ? AND status = ? AND disabled = 0 AND representative = 0')
				->order(array('nodes.metaHBdate ' . Zend_Db_Select::SQL_DESC, 'nodes.id ' . Zend_Db_Select::SQL_DESC))
				->limit(1);
		
		$row = $select->query(null, array($realRepresentative->getSignature(), Nodes_Element::STATUS_ONLINE))->fetch();
		$nodeId = $row['id'];
		
		$candidate = null;
		
		if (is_numeric($nodeId)) {
			try {
				$candidate = $nodesModel->getNode($nodeId);
			} catch (Zwas_Exception $e) {
				$candidate = null;
			}
		}
		
		if (is_null($candidate)) {
			// TRANSLATE
			Zwas_Log::logMessage('No representative candidates found');
			throw new Zwas_Exception(Zwas_Translate::_('No candidate server found'));
		}
		return $candidate;
	}
	
	/**
	 * Get the maximal Zend Server version from responding servers (= not disabled and not offline)
	 * @return string
	 */
	public function getMaxServerVersion() {
		$select = $this->getAdapter()->select();

		$select->from('environment', 'MAX(environment.serverVersion) AS maxVersion')
				->join('nodes', 'nodes.environmentId = environment.environmentId', '')
				->where('nodes.disabled = ?', 0)
				->where('nodes.status != ?', Nodes_Element::STATUS_OFFLINE);
				
		$row = $select->query()->fetch();
		
		return (string)$row['maxVersion'];
	}
	
	public function getAdapter() {
		return Context_Db::getZsmAdapter();
	}
}

