<?php

class Nodes_Model {
	
	const HEARTBEAT_LOCK_UP_TIME_LIMIT = 120;
	
	/**
	 * @var Nodes_Table
	 */
	private $nodesTable;
	/**
	 * @var Nodes_Communication_Mapper
	 */
	private static $comMapper;
	
	/**
	 * @var Nodes_Model_Heartbeat
	 */
	private $modelHeartbeat;
	
	/**
	 * Convenience method to create and fire a cluster-wide heartbeat refresh
	 * @return Nodes_Model
	 */
	public static function refreshNodesState() {
		$model = new Nodes_Model();
		return $model->updateNodesState();
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @throws Zwas_Exception
	 * @return Nodes_Element_Mapper
	 */
	public function enableNode(Nodes_Element_Mapper $node) {
		$node->setDisabled(false)->save();
		
		$scPlugin = new Nodes_Model_Plugin_ConnectNodeToSessionClustering();
		$scPlugin->postAction($this, array($node));
		
		$event = new ZwasComponents_MonitorUi_UiEvent_NodeEnabled();
		$event->setNode($node)->report();
		
		if (Zwas_Synchronize::sync('updateNodesState', self::HEARTBEAT_LOCK_UP_TIME_LIMIT)) {
			try {
				$this->getModelHeartbeat()->updateHeartBeat($node);
			} catch (Zwas_Exception $e) {
				Zwas_Synchronize::release();
				throw $e;
			}
			Zwas_Synchronize::release();
		} else {
			$node->setTransition(Nodes_Element::SERVER_TRANSITION_TO_ENABLE);
			$node->save();
		}
		return $node;
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @throws Zwas_Exception
	 * @return Nodes_Element_Mapper
	 */
	public function disableNode(Nodes_Element_Mapper $node) {
		$delegateRemoveNode = new Nodes_Model_Delegate_DisableNode($this);
		try {
			return $delegateRemoveNode->disable($node);
		} catch(Nodes_Model_Delegate_Exception $e) {
			Zwas_Log::logMessage("Could not disable server {$node->getId()}", Zwas_Exception::ERROR);
			throw new Zwas_Exception(new Zwas_Text(
				Zwas_Translate::_("Failed to disable server #%s, %s"), array($node->getId(), $e->getMessageObject())
			));
		}
	}
	
	/**
	 * @return Nodes_Model_Heartbeat
	 */
	public function getModelHeartbeat() {
		if (is_null($this->modelHeartbeat)) {
			$this->modelHeartbeat = new Nodes_Model_Heartbeat();
			$this->modelHeartbeat->setNodesModel($this);
		}
		return $this->modelHeartbeat;
	}

	/**
	 * @param Nodes_Model_Heartbeat $modelHeartbeat to set
	 * @return Nodes_Model
	 */
	public function setModelHeartbeat(Nodes_Model_Heartbeat $modelHeartbeat) {
		$this->modelHeartbeat = $modelHeartbeat;
		return $this;
	}
	
	/**
	 * @return Nodes_Communication_Mapper
	 */
	public function getCommMapper() {
		if (is_null(self::$comMapper)) {
			self::$comMapper = new Nodes_Communication_Mapper();
		}
		return self::$comMapper;
	}
		
	/**
	 * @return Nodes_Table
	 */
	public function getNodesTable() {
		if (is_null($this->nodesTable)) {
			$this->nodesTable = Context_Db::getNodesTable();
		}
		return $this->nodesTable;
	}
	
	/**
	 * @return array of Nodes_Element_Mapper
	 * @throws Zwas_Exception
	 */
	public function getEnabledNodes() {
		$nodes = array();
		try {
			foreach($this->getNodesTable()->getEnabledNodes() as $node) {
				$nodes[$node->getId()] = new Nodes_Element_Mapper($node);
			}
		
		} catch (Exception $e) {
			Zwas_Log::logMessage("Unable to get nodes list: {$e->getMessage()}", Zwas_Exception::ERROR);
			throw new Zwas_Exception(Zwas_Translate::_('Error occurred fetching nodes from DB'), Zwas_Exception::ERROR);
		}
		return $nodes;
	}
	
	/**
	 * @return boolean
	 */
	public function hasEnabledNodes() {
		return (0 < $this->getNodesTable()->countEnabledNodes());
	}
	
	/**
	 * @return array of Nodes_Element_Mapper
	 * @throws Zwas_Exception
	 */
	public function getNodes() {
		$nodes = array();
		try {
			foreach($this->getNodesTable()->getNodesList() as $node) {
				$nodes[$node->getId()] = new Nodes_Element_Mapper($node);
			}
		
		} catch (Exception $e) {
			Zwas_Log::logMessage("Unable to get nodes list: {$e->getMessage()}", Zwas_Exception::ERROR);
			throw new Zwas_Exception(Zwas_Translate::_('Error occurred fetching nodes from DB'), Zwas_Exception::ERROR);
		}
		return $nodes;
	}
	
	/**
	 * @return boolean
	 */
	public function hasRespondingNodes() {
		return (0 < $this->getNodesTable()->countRespondingNodes());
	}
	
	/**
	 * @return array of Nodes_Element_Mapper
	 */
	public function getRespondingNodes() {
		$nodes = array();
		try {
			foreach($this->getNodesTable()->getRespondingNodes() as $node) {
				$nodes[$node->getId()] = new Nodes_Element_Mapper($node);
			}
		
		} catch (Exception $e) {
			Zwas_Log::logMessage("Unable to get responding nodes list: {$e->getMessage()}", Zwas_Exception::ERROR);
		}
		return $nodes;
	}
	
	/**
	 * @return Nodes_Collection
	 */
	public function getNodesListOrderedByName() {
		try {
			$nodes = array();
			foreach ($this->getNodesTable()->getNodesListOrderedByName() as $node) {
				$nodes[] = new Nodes_Element_Mapper($node);
			}
			return new Nodes_Collection($nodes);
		} catch (Exception $e) {
			Zwas_Log::logMessage("Unable to get nodes list: {$e->getMessage()}", Zwas_Exception::ERROR);
			throw new Zwas_Exception(Zwas_Translate::_('Error occurred fetching nodes from DB'), Zwas_Exception::ERROR);
		}
	}

	/**
	 * @return integer
	 */
	public function getNewestMetaHBdate() {
		return $this->getNodesTable()->getMaxMetaHBdate();
	}
	
	/**
	 * @return array of Ids
	 */
	public function getNodeIds() {
		$nodes = $this->getNodes();
		$nodeIds = array();
		foreach($nodes as $node) { /* @var $node Nodes_Element_Mapper */
			$nodeIds[] = $node->getId();
		}
		return $nodeIds;
	}
	
	/**
	 * @param integer $id
	 * @return Nodes_Element_Mapper
	 * @throws Zwas_Exception
	 */
	public function getNode($id) {
		try {
			$node = $this->getNodesTable()->getNodeById($id);
			
		} catch (Exception $e) {
			Zwas_Log::logMessage("Unable to get node {$id}: {$e->getMessage()}", Zwas_Exception::ERROR);
			throw new Zwas_Exception(Zwas_Translate::_('Error occurred fetching node from DB'), Zwas_Exception::ERROR);
		}
		if (is_null($node)) {
			Zwas_Log::logMessage("Unable to get node {$id}", Zwas_Exception::WARNING);
			throw new Zwas_Exception(Zwas_Translate::_(new Zwas_Text("Node id %s does not exist", array($id))), Zwas_Exception::WARNING);
		}
		return new Nodes_Element_Mapper($node);
	}
	
	/**
	 * @param unknown_type $ipAddress
	 * @return Nodes_Element_Mapper|null
	 */
	public function getNodeByAddress($ipAddress) {
		try {
			$node = $this->getNodesTable()->getNodeByAddress($ipAddress);
		} catch (Exception $e) {
			Zwas_Log::logMessage("Unable to get node by address {$ipAddress}: {$e->getMessage()}", Zwas_Exception::ERROR);
			throw new Zwas_Exception(Zwas_Translate::_('Error occurred fetching node from DB'), Zwas_Exception::ERROR);
		}
		
		if (! is_null($node)) {
			return new Nodes_Element_Mapper($node);
		}
		return $node;
	}
	
	/**
	 * Collecting heartbeat data for all nodes
	 * Scan for offline nodes and register an event for missing nodes
	 * Scan for configuration mismatches and register an event
	 * @return Nodes_Model
	 * @throws Zwas_Exception
	 */
	public function updateNodesState() {
		try {
			if (Zwas_Synchronize::sync('updateNodesState', self::HEARTBEAT_LOCK_UP_TIME_LIMIT)) {
				$heartbeatModel = $this->getModelHeartbeat();
				$nodes = new Nodes_Collection($this->getEnabledNodes());
				
				$candidatesForRemoval = $nodes->scanNodesSessionClusteringInProcess();
				$nodes = $heartbeatModel->updateNodesState($nodes);
			
				if (0 < $candidatesForRemoval->count()) {
					$candidate = $candidatesForRemoval->current();
					try {
						$candidate->refresh(); // refresh the current scstate
						$this->contrastCheckDelete($candidate);
					}  catch (Zwas_Db_Exception $e) {
						/// failed during refresh, the node no longer exists
						// TRANSLATE
						Zwas_Log::warn(new Zwas_Text('Could not refresh node state: %s', array($e->getMessageObject())));
					}
				}
				
				$this->registerEventForAllNodes(
					$nodes->scanNodesOffline(), 
					new ZwasComponents_MonitorUi_UiEvent_NodeNotResponding()
				);
				$this->registerEventForAllNodes(
					$nodes->scanNodesConfigMismatch(), 
					new ZwasComponents_MonitorUi_UiEvent_NodeSignatureMismatch()
				);
				Zwas_Synchronize::release();
			} else {
				Zwas_Log::debug('Heartbeat collection was skipped due to synchronization');
			}
		} catch (Zwas_Exception $e) {
			Zwas_Synchronize::release();
			throw $e;
		}
		
		return $this;
	}
	
	/**
	 * Check if there is nodes in DB
	 * @return boolean
	 */
	public function hasNodes() {
		return (0 < $this->countNodes());
	}
	
	/**
	 * @return integer
	 */
	public function countNodes() {
		return (int)$this->getNodesTable()->countNodes();
	}
	
	/**
	 * @return boolean
	 */
	public function hasMoreNodesThanLicensed() {
		$numOfNodes = $this->countNodes();
		try {
			$licenseDirectives = License_Directives::getSystemLicenseDirectives();
			$licenseInfo = Context_Api::getUtilAPI()->getSerialNumberInfo(
				$licenseDirectives->getLicenseKey(), $licenseDirectives->getOrderNumber()
			);			
			$license = Context_Version::getLicenseModel($licenseInfo);
			
			return ($numOfNodes > $license->getMaxClusterNodes());
		} catch (Zwas_Exception $e) {
			
		}
		return false;
	}
	
	/**
	 * @param string $name
	 * @param string $address
	 * @param string $password
	 * @param bool   $distributeFlag
	 * @return Nodes_Element_Mapper
	 */
	public function addNode($name, $address, $password, $distributeFlag = false) {
		// Verify that adding node is allowed
		if ($this->hasNodes() && (0 === $this->getNodesTable()->countRespondingNodes())) {
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('You can not add a server while no other servers are enabled'));
		}
		
		$delegate = new Nodes_Model_Delegate_AddNode($this);
		try {
			return $delegate->addNode($name, $address, $password, $distributeFlag);
		} catch(Nodes_Model_Delegate_Exception $e) {
			Zwas_Log::logMessage(new Zwas_Text(Zwas_Translate::_('Could not add server %s'), array($name)), Zwas_Exception::ERROR);
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_("Failed while trying to add server %s: %s"), array($name, $e->getMessageObject())));
		}
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Element_Mapper
	 * @throws Zwas_Exception on failure
	 */
	public function removeNode(Nodes_Element_Mapper $node) {
		$delegateRemoveNode = new Nodes_Model_Delegate_RemoveNode($this);
		try {
			return $delegateRemoveNode->remove($node);
		} catch(Nodes_Model_Delegate_Exception $e) {
			Zwas_Log::logMessage("Could not remove server {$node->getId()}", Zwas_Exception::ERROR);
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_("Failed while trying to remove server #%s, %s"), array($node->getId(), $e->getMessageObject())));
		}
	}
	
	/**
	 * Attempt to remove all nodes
	 */
	public function detachAllNodes() {
		$delegateRemoveNode = new Nodes_Model_Delegate_RemoveNode($this);
		foreach ($this->getNodes() as $node) { /** @var Nodes_Element_Mapper $node */
			try {
				$delegateRemoveNode->detachOnly($node);
			} catch(Nodes_Model_Delegate_Exception $e) {
				Zwas_Log::logMessage("Could not detach server {$node->getId()}", Zwas_Exception::WARNING);
			}
		}
		/* 
		 * Since we are essentially emptying the cluster we have to make
		 * sure that CM does not attempt to connect to any node's 
		 * JobQueue daemon
		 */
		ZwasComponents_JobQueue_Binding::saveZwasBindingDirective('');
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @param boolean $silentRun
	 * @return Nodes_Element_Mapper
	 * @throws Zwas_Exception
	 */
	public function ungracefulRemoveNode(Nodes_Element_Mapper $node, $silentRun = false) {
		$delegateRemoveNode = new Nodes_Model_Delegate_RemoveNode($this);
		if ($silentRun) {
			$delegateRemoveNode->resetEventToReport();
		}
		try {
			return $delegateRemoveNode->ungracefulRemove($node);
		} catch(Nodes_Model_Delegate_Exception $e) {
			Zwas_Log::logMessage("Could not remove server {$node->getId()}", Zwas_Exception::ERROR);
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_("Failed while trying to remove server #%s, %s"), array($node->getId(), $e->getMessageObject())));
		}
	}
	
	/**
	 * @param integer $id
	 * @throws Zwas_Exception on failure
	 */
	public function reconfigureNode($id) {
		$pluginBroker = new Nodes_Model_Plugin_Broker();
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectNodeToSessionClustering());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectMonitorToZSM());
		
		$node		= $this->getNode($id);
		try {
			// as the node is already a part of the cluster, no need to check that it matches the cluster
			$this->copyNode($node);
			// update node specific data 
			$pluginBroker->postAction($this, array($node));
		} catch (Zwas_Exception $e) {
			Zwas_Log::logMessage("Could not reconfigure the server by ID {$id}", Zwas_Exception::ERROR);
			throw new Zwas_Exception($e->getMessageObject());
		}
		
		if (Zwas_Synchronize::sync('updateNodesState', self::HEARTBEAT_LOCK_UP_TIME_LIMIT)) {
			try {
				$this->getModelHeartbeat()->updateHeartBeat($node);
			} catch(Zwas_Exception $e) {
				Zwas_Log::logException('Could not update node heartbeat following reconfiguration', $e);
			}
			Zwas_Synchronize::release();
		} else {
			$node->updateTransition(Nodes_Element::SERVER_TRANSITION_TO_REFRESH);
		}
	}
	
	/**
	 * Use the Cluster manager's own configuration files for distributing the desired configuration
	 * to the nodes in the cluster
	 * @param array $fileNames
	 * @return Cluster_ConfigurationData
	 */
	public function distributeConfigurationFiles(array $fileNames = array()) {
		if (count($fileNames)) {
			$model = new Node_Model();
			
			$configurationData = new Cluster_ConfigurationData($this->getClusterEnvironment());
			$configurationData->collectZendServerConfigurations($fileNames);
			$nodeConfig = $configurationData->produceNodeConfiguration();
			$this->distributeConfigurationToNodes($configurationData->produceNodeConfiguration());
		} else {
			throw new Nodes_Model_Exception(Zwas_Translate::_('No configuration data to redistribute'));
		}
		return $configurationData;
	}
	
	/**
	 * @return Nodes_Element_Mapper|null
	 * @throws Zwas_Exception
	 */
	public function getRepresentativeForReading() {
		$nodesMapper = new Nodes_Mapper();
		try {
			$representative = $nodesMapper->getRepresentative();
		} catch (Zwas_Exception $e) {
			try {
				$representative = $nodesMapper->getAlternativeRepresentative();
			} catch (Zwas_Exception $e) {
				$representative = $nodesMapper->getLastRespondingServer();
			}
		}
		
		if (is_null($representative)) {
			Zwas_Log::logMessage('Unable to retrieve a representative');
			throw new Zwas_Exception('Unable to locate a responding server. Make sure the cluster members are online');
		}
		
		return $representative;
		
	}
	
	/**
	 * Retrieves a working representative and promotes it if
	 * the server is only a candidate
	 * @return Nodes_Element_Mapper|null
	 * @throws Zwas_Exception
	 */
	public function getRepresentativeForWriting() {
		$representative = $this->getRepresentativeForReading();
		if (($representative instanceof Nodes_Element_Mapper) && (! $representative->isRepresentative())) {
			$this->getNodesTable()->setRepresentative($representative->getId());
			$representative = new Nodes_Element_Mapper($this->getNodesTable()->getRepresentative());
		}
		return $representative;
	}
	
	/**
	 * @return Nodes_Element_Mapper
	 * @throws Zwas_Exception
	 */
	public function getRepresentativeNode() {
		return $this->getRepresentativeForWriting();
	}
	
	/**
	 * @return boolean 
	 */
	public function hasDifferentEnvironmentId() {
		$nodesMapper = new Nodes_Mapper();
		return $nodesMapper->hasDifferentEnvironmentId();
	}
	
	/**
	 * @return boolean 
	 */
	public function hasNodesInRemovalTransition() {
		$nodesMapper = new Nodes_Mapper();
		return $nodesMapper->hasNodesInRemovalTransition();
	}
	
	/**
	 * @return boolean 
	 */
	public function hasNodesInDisableTransition() {
		$nodesMapper = new Nodes_Mapper();
		return $nodesMapper->hasNodesInDisableTransition();
	}
	
	/**
	 * @return Nodes_Element_Mapper
	 */
	public function replaceRepresentative() {
		try {
			$nodesMapper = new Nodes_Mapper();
			$representative = $nodesMapper->getAlternativeRepresentative();
		} catch (Zwas_Exception $e) {
			$representative = $this->getNodesTable()->getLastRespondingServer();
		}
		
		if (is_null($representative)) {
			throw new Zwas_Exception(Zwas_Translate::_('The cluster has no servers'));
		}
		
		if($representative instanceof Nodes_Element) {
			$representative = new Nodes_Element_Mapper($representative);
		}
		
		if ($representative->isRepresentative()) {
			Zwas_Log::logMessage(Zwas_Translate::_('No eligible candidate servers are available to be used for replacing the representative'));
			throw new Zwas_Exception(Zwas_Translate::_(
				'An internal application error has occurred.  If this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center'
			));
		}
		
		$this->getNodesTable()->setRepresentative($representative->getId());
		$representative->refresh();
		
		return $representative;
	}
	
	/**
	 * check if has node status is in process
	 * @return boolean
	 */
	public function hasNodeInShutdownProcess() {
		return $this->getNodesTable()->hasNodeInShutdownProcess();
	}
	
	/**
	 * Returns true if there are any nodes in startup state
	 * @return boolean
	 */
	public function hasNodeInStartupProcess() {
		return $this->getNodesTable()->hasNodeInStartupProcess();
	}
	
	/**
	 * @return boolean
	 */
	public function hasNodeInvalid() {
		foreach ($this->getRespondingNodes() as $node) { /* @var $node Nodes_Element_Mapper */
			if (Nodes_Element::STATUS_ERROR == $node->getStatus()) {
				return true;
			}
			if (Nodes_Element::STATUS_CONFIG_MISMATCH == $node->getStatus()) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @return boolean
	 */
	public function hasNodeInError() {
		foreach ($this->getRespondingNodes() as $node) { /* @var $node Nodes_Element_Mapper */
			if (Nodes_Element::STATUS_ERROR == $node->getStatus()) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Returns if there is at least one server which is online (and also not currently in Session Clustering Graceful Shutdown process)
	 * @return boolean
	 */
	public function hasUnstableNode() {
		return $this->getNodesTable()->hasUnstableNode();
	}
	
	/**
	 * Returns the Node_Environment which represents the cluster
	 * @return Node_Environment 
	 */
	public function getClusterEnvironment() {
		try { // fetch representative
			$repNode = $this->getRepresentativeForWriting();
			
		} catch(Nodes_Model_Exception $e) {
			throw $e;
		} catch (Zwas_Exception $e) {
			// If there is nodes, but "getRepresentativeNode" throws an exception, there is no
			// responding node to act as representative - therefore we can't add a new node
			throw new Zwas_Exception(Zwas_Translate::_('Unable to add server, check your cluster for errors'));
		}
		$com = $this->getCommMapper();
		return $com->probeNode($repNode->getAddress(), $repNode->getPassword());		
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Node_ConfigurationData
	 * @throws Zwas_Exception
	 */
	public function getNodeConfiguration(Nodes_Element_Mapper $node) {
		$com = $this->getCommMapper();
		
		//get configuration data from new node
		$repResponse = $com->remoteAction(
			$node,
			'Node',
			'Get-Configuration'
		);

		if ($repResponse->isError()) {
			throw new Zwas_Exception('Unable to get node\'s configuration data');
		}
		
		return $repResponse->getData();
	}
	
	/**
	 * @return integer
	 */
	public static function getMaxServerId() {
		$model = new self;
		$model->getNodesTable()->getMaxServerId();
	}

	/**
	 * Distribute the ZS license to all connected nodes
	 */
	public function distributeNodesLicense() {
		try {
			$nodes = $this->getRespondingNodes();
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException(Zwas_Translate::_('Could not fetch the servers list'), $e);
			return false;
		}
		
		try {
			$licenseDirectives = ApplicationModel::getNodesLicense();
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException(Zwas_Translate::_('Could not fetch the ZS license details'), $e);
			return false;
		}
		
		$params = array(
			'user'		=> $licenseDirectives->getOrderNumber(), 
			'serial'	=> $licenseDirectives->getLicenseKey()
		);
		
		try {
			$this->getCommMapper()->distributeAction($nodes, 'Node', 'Import-License', $params);
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException(Zwas_Translate::_('Could not distribute the license details'), $e);
			return false;
		}
		return true;
	}
	
	/**
	 * Perform a constrast check for the scStatus flag:
	 * $node is expected to be an up-to-date node element which was previously inProcess
	 * This method makes no effort to verify this fact other than to check the node's current sCstatus 
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Collection
	 */
	public function contrastCheckDelete(Nodes_Element_Mapper $node) {
		if((ZwasComponents_SessionClustering_Model::SCD_STATUS_OFF == $node->getScStatus())
			|| (ZwasComponents_SessionClustering_Model::SCD_STATUS_STANDBY == $node->getScStatus())) {
				
			try {
				$nodeId = $node->getId();
				if ($node->isNodeInRemovalProcess()) {
					$this->removeNode($node);
				} elseif ($node->isNodeInDisableProcess()) {
					$this->disableNode($node);
				} else {
					Zwas_Log::warn("Server {$nodeId} seems ready to be removed, but no transition is set");
				}
			} catch(Zwas_Exception $e) {
				/// if we've failed in removing, leave the status as-is, record the failure + event?
				Zwas_Log::logException("Failed 2nd step of removal process for node {$node->getId()}", $e);
				throw $e;
			}
		} elseif(ZwasComponents_SessionClustering_Model::SCD_STATUS_SHUTDOWN_FAILED == $node->getScStatus()) {
			Zwas_Log::logMessage("Node {$node->getId()} could not shut down session clustering", Zend_Log::ERR);
			throw new Zwas_Exception(Zwas_Translate::_(new Zwas_Text("Node %s could not shut down session clustering", array($node->getId()))));
		} elseif(ZwasComponents_SessionClustering_Model::SCD_STATUS_SHUTDOWN_IN_PROCESS != $node->getScStatus()) {
			Zwas_Log::logMessage("Node {$node->getId()} could not shut down session clustering. Unexpected state {$node->getScStatus()}", Zend_Log::WARN);
		}
		return $node;
	}
	
	/**
	 * Register an event as described by the parameter for all nodes in the collection
	 * @param ZwasComponents_MonitorUi_UiEvent_Abstract $event
	 * @return Nodes_Collection
	 */
	public function registerEventForAllNodes(Nodes_Collection $nodes, ZwasComponents_MonitorUi_UiEvent_Abstract $event) {
		foreach($nodes as $node) {
			$event->setNode($node)->report();
		}
		return $nodes;
	}
	
	/**
	 * Check if a given server is a node that is part of the cluster group
	 * @param string server ip address
	 * @return boolean 
	 */
	public function isServerPartOfClusterNodesGroup($address) {
		$node = Context_Db::getNodesTable()->getNodeByAddress($address);
		if (is_null($node)) {
			return false;
		}
		return true;
	}
	
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @throws Zwas_Exception
	 */
	private function copyEventRulesToNode(Nodes_Element_Mapper $node) {
		$communicationMapper = $this->getCommMapper();
		$nodeEnviroment = $communicationMapper->probeNode($node->getAddress(), $node->getPassword());
		$nodeConfiguration = new Node_ConfigurationData($nodeEnviroment);
		
		// get the rules file
		$filePath = ZwasComponents_Monitor_Model::getXMLFile();
		$fileObject = Context_FileSystem::getFileObject($filePath, 'r');
		$file = new Node_FileObject(basename($filePath), '', $fileObject->readAll());
		
		$nodeConfiguration->setZendServerConfigurations(array($file));

		$params = array('data' => Zwas_Package::pack($nodeConfiguration));
		
		$response = $communicationMapper->remoteAction($node, 'Node', 'Set-Configuration', $params);
		
		if ($response->isError()) {
			Zwas_Log::logMessage('Got the wrong response from the node when trying to send over the Zend Monitor rules "' . $response->getMessage().'"');
			throw new Zwas_Exception(Zwas_Translate::_('Failed sending the Zend Monitor rules definitions file to the node'));
		}
	}
	
	/**
	 * Copy configuration from repNode to new node
	 *
	 * @param string $address
	 * @param string $password
	 * @throws Zwas_Exception
	 */
	private function copyNode($node) {
		//get configuration data from representative node
		$configurationData = $this->getNodeConfiguration($this->getRepresentativeForWriting());
		 
		//set configuration data to new node
		$response = $this->getCommMapper()->remoteAction(
			$node,
			'Node',
			'Set-Configuration',
			array('data' => Zwas_Package::pack($configurationData))
		);
		
		if ($response->isError()) {
			throw new Zwas_Exception($response->getData());
		}
	}
	
	/**
	 * @param Node_ConfigurationData $configuration
	 * @param array $fileNames
	 * @return Node_ConfigurationData
	 */
	private function filterConfigurationFiles(Node_ConfigurationData $configuration, array $fileNames) {
		$filesToDistribute = array();
		foreach ($configuration->getZendServerConfigurations() as $fileObject) {
			if (in_array($fileObject->getFileName(), $fileNames)) {
				$filesToDistribute[] = $fileObject;
			}
		}
    	
		$configuration->setZendServerConfigurations($filesToDistribute);
		return $configuration;
	}
	
	/**
	 * @param Node_ConfigurationData $configuration Configuration package to distribute
	 * @param boolean $forceRepresentative Send the configuration package to all nodes, including the representative
	 * @throws Zwas_Exception
	 */
	private function distributeConfigurationToNodes(Node_ConfigurationData $configuration) {
		$hasError = false;
		$nodesToDistribute = array();
		foreach ($this->getRespondingNodes() as $node) {
			if ($node->getStatus() != Nodes_Element::STATUS_OFFLINE) {
				//set configuration data to each node
				$nodesToDistribute[]= $node;
			}
		}
		
		$responses = $this->getCommMapper()->distributeActionGetResponses(
			$nodesToDistribute,
			'Node',
			'Set-Configuration',
			array('data' => Zwas_Package::pack($configuration))
		);
		
		$errors = $responses->scanErrorResponses();
		
		if ($errors->count() > 0) {
			foreach ($errors as $errorResponse) { /* @var $errorResponse Nodes_Response */
				Zwas_Log::warn("Could not redistribute file(s) on the server by ID {$errorResponse->getNodeId()}", Zwas_Exception::ERROR);
			}
			throw new Nodes_Model_Exception(Zwas_Translate::_('Could not apply configuration to one or more servers, try rematch the servers that appear in error. If this problem persists, remove the servers and reassociate them to the cluster'));
		}
	}

}