<?php
abstract class Nodes_Model_Delegate_Abstract {
	/**
	 * @var Nodes_Communication_Mapper
	 */
	protected $commMapper = null;
	/**
	 * @var Nodes_Table
	 */
	protected $nodesTable = null;
	/**
	 * @var Nodes_Element_Mapper
	 */
	protected $node = null;
	/**
	 * @var Nodes_Model
	 */
	protected $model = null;
	/**
	 * @var ZwasComponents_MonitorUi_UiEvent_Abstract
	 */
	protected $eventToReport = null;
	
	/**
	 * @param Nodes_Model $model
	 */
	public function __construct(Nodes_Model $model) {
		$this->model = $model;
		$this->commMapper = $model->getCommMapper();
		$this->nodesTable = Context_Db::getNodesTable();
	}
	
	/**
	 * @return Nodes_Model
	 */
	public function getModel() {
		return $this->model;
	}
	
	/**
	 * @return Nodes_Element_Mapper
	 */
	public function getNode() {
		return $this->node;
	}
	
	public function resetEventToReport() {
		$this->eventToReport = null;
	}
	
	/**
	 * @param ZwasComponents_MonitorUi_UiEvent_Abstract $event
	 */
	public function setEventToReport(ZwasComponents_MonitorUi_UiEvent_Abstract $event) {
		$this->eventToReport = $event;
	}
	
	/**
	 * @return boolean
	 */
	protected function hasNodesInShutdownProcess() {
		return $this->nodesTable->hasNodeInShutdownProcess();
	}
	
	/**
	 * @return boolean
	 */
	protected function hasNodesInStartup() {
		return $this->nodesTable->hasNodeInStartupProcess();
	}

	/**
	 * @param Nodes_Element_Mapper $node
	 * @return boolean
	 */
	protected function isNodeReadyForPostGraceful(Nodes_Element_Mapper $node) {
		return (in_array($node->getScStatus(), 
				array(
					ZwasComponents_SessionClustering_Model::SCD_STATUS_UNKNOWN,
					ZwasComponents_SessionClustering_Model::SCD_STATUS_OFF,
					ZwasComponents_SessionClustering_Model::SCD_STATUS_STANDBY
				)));
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Model_Delegate_Abstract
	 * @throws Nodes_Model_Delegate_Exception
	 */
	protected function gracefulShutdownSc(Nodes_Element_Mapper $node) {
		$this->node = $node;
		
		$this->isClusterReadyForGraceful($node);
		
		$plugin = new Nodes_Model_Plugin_GracefullShutdownSessionClustering();
		try {
			$plugin->preAction($this->model, $node);
		} catch (Nodes_Model_Delegate_Exception $e) {
			if (Nodes_Model_Plugin_Exception::ABORT_PLUGIN_CHAIN !== $e->getCode()) {
				Zwas_Log::logException('Node disable action paused to allow graceful shutdown', $e);
			}
		}
		
		return $this;
	}
	
	/**
	 * Check that the current server may be gracefully shut down
	 */
	protected function isNodeReadyForGracefulShutdown($node) {
		// Check that the server's current state is right and that there are other servers in the cluster to which the server may
		// transfer its sessions
		return ((! in_array($node->getScStatus(), 
				array(
					ZwasComponents_SessionClustering_Model::SCD_STATUS_UNKNOWN,
					ZwasComponents_SessionClustering_Model::SCD_STATUS_OFF,
					ZwasComponents_SessionClustering_Model::SCD_STATUS_STANDBY
				)
			)) && (1 < $this->model->countNodes()));
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @throws Nodes_Model_Delegate_Exception
	 * @return boolean
	 */
	protected function isClusterReadyForGraceful(Nodes_Element_Mapper $node) {
		if ($node->getScStatus() == ZwasComponents_SessionClustering_Model::SCD_STATUS_SHUTDOWN_IN_PROCESS) {
			throw new Nodes_Model_Delegate_Exception(Zwas_Translate::_('This server\'s Session Clustering is already being shut down'));
		}
		
		if ($node->getScStatus() == ZwasComponents_SessionClustering_Model::SCD_STATUS_STARTING_UP) {
			throw new Nodes_Model_Delegate_Exception(Zwas_Translate::_('This server\'s Session Clustering is already in the process of starting up'));
		}
		
		if ($this->hasNodesInShutdownProcess()) {
			throw new Nodes_Model_Delegate_Exception(Zwas_Translate::_('Another server is being removed / disabled.  Wait until this process completes and retry'));
		}
		
		if ($this->hasNodesInStartup()) {
			throw new Nodes_Model_Delegate_Exception(Zwas_Translate::_('Another server is being enabled.  Wait until this process completes and retry'));
		}
		return true;
	}
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Model_Delegate_Abstract
	 */
	protected function createEvent(Nodes_Element_Mapper $node) {
		if ($this->eventToReport instanceof ZwasComponents_MonitorUi_UiEvent_Abstract) {
			$this->eventToReport->setNode($node)->report();
		}
		return $this;
	}
}