<?php
class Nodes_Model_Delegate_AddNode extends Nodes_Model_Delegate_Abstract {
	
	public function __construct(Nodes_Model $model) {
		parent::__construct($model);
		$this->setEventToReport(new ZwasComponents_MonitorUi_UiEvent_NodeAdded());
	}
	
	/**
	 * @param string $name
	 * @param string $address
	 * @param string $password
	 * @param bool   $distributeFlag
	 * @return Nodes_Element_Mapper
	 */
	public function addNode($name, $address, $password, $distributeFlag = false) {
		$firstNode = (! $this->model->hasNodes());
		if ($firstNode) {
			$this->node = $this->addFirstNode($name, $address, $password);
		} else if ($distributeFlag) {
			$this->node = $this->addNewRepresentativeNode($name, $address, $password);
		} else {
			$this->node = $this->addAdditionalNode($name, $address, $password);
		}

		$this->node->setStatus(Nodes_Element::STATUS_ONLINE);
		$this->node->save();
		
		/// set all nodes' awaits restart flags
		$restartCollection = new Nodes_Collection($this->model->getRespondingNodes());
		$restartCollection->setCollectionAwaitsRestart();
		$restartCollection->save();
		
		$this->createEvent($this->node);
		// since $restartCollection works on copies of the nodes elements, the data in the new node needs refreshing
		$this->node->refresh();
		return $this->node;
	}
	
	/**
	 * @param string $name
	 * @param string $address
	 * @param string $password
	 * @return Nodes_Element_Mapper
	 */
	protected function addFirstNode($name, $address, $password) {
		if ($this->model->hasNodes()) {
			throw new Nodes_Model_Delegate_Exception(
				Zwas_Translate::_('Cluster already has connected servers; adding a server as the initial one is not allowed'), 
				Zwas_Exception::ASSERT
			);
		}
		
		$pluginBroker = new Nodes_Model_Plugin_Broker();
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_PropagateLicense());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_MashupEventsRules());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectServerToJobQueue());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectMonitorToZSM());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectNodeToSessionClustering());
		
		$nodesEnvironment = $this->getNodeEnvironment($address, $password);
		
		// store the node in the local database
		$addedNode = $this->saveNode($name, $address, $password, $nodesEnvironment, true);
		
		try {
			$pluginBroker->preAction($this->model, $addedNode);

			//add node actions
			$affectedNodes = array($addedNode);
			
			//post add node actions
			$pluginBroker->postAction($this->model, $affectedNodes);
			
			
			// Update local ZSCM jobqueue.ini
			$daemonBinding	= "tcp://{$addedNode->getHostname()}:10085";
			ZwasComponents_ZSM_JobQueue_Model::saveBindingDirective($daemonBinding, null);
			
			if (Zwas_Synchronize::sync('updateNodesState', Nodes_Model::HEARTBEAT_LOCK_UP_TIME_LIMIT)) {
				try {
					$this->model->getModelHeartbeat()->updateHeartBeat($addedNode);
				} catch (Zwas_Exception $e) {
					Zwas_Synchronize::release();
					throw $e;
				}
				Zwas_Synchronize::release();
			} else {
				$addedNode->setTransition(Nodes_Element::SERVER_TRANSITION_TO_ADD);
				$addedNode->save();
			}
			
		} catch (Zwas_Exception $e) {
			/// we've either failed to add the node during monitor connect, which is handled in removeNode
			/// or we've failed to connect to session cluster - in which case we don't need to shutdown the node's daemon anyway
			Zwas_Log::logException('Failed to add a node due to error on node. Attempting to release node.', $e);
			$this->model->ungracefulRemoveNode($addedNode, true);
			throw $e;
		}
		return $addedNode;
	}
	
	/**
	 * @param string $name
	 * @param string $address
	 * @param string $password
	 * @return Nodes_Element_Mapper
	 */
	protected function addNewRepresentativeNode($name, $address, $password) {
		if (! $this->model->hasNodes()) {
			throw new Nodes_Model_Delegate_Exception(
				Zwas_Translate::_('Cluster already has no connected servers; adding a server as an additional server is not allowed'), 
				Zwas_Exception::ASSERT
			);
		}
		
		$pluginBroker = new Nodes_Model_Plugin_Broker();
		/// configuration distribution must take place before all postAction plug-ins
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_DistributeConfiguration());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_PropagateLicense());
		$pluginBroker->registerPlugin(New Nodes_Model_Plugin_MaintainClusterJobQueueSettings());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_MashupEventsRules());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectMonitorToZSM());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectNodeToSessionClustering());
		
		$nodesEnvironment = $this->getNodeEnvironment($address, $password);
		
		$this->checkNodeMatchesCluster($nodesEnvironment);
		
		// store the node in the local database
		$addedNode = $this->saveNode($name, $address, $password, $nodesEnvironment, true);
		
		try {
			/*
			 * Note: Ticket #27889, comment #45598
			 */
			$pluginBroker->preAction($this->model, $addedNode);

			// Set the newly added node as a representative, retrieve an updated
			$affectedNodes = $this->model->getRespondingNodes();
			
			//post add node actions
			$pluginBroker->postAction($this->model, $affectedNodes);
			
		} catch (Zwas_Exception $e) {
			/// we've either failed to add the node during monitor connect, which is handled in removeNode
			/// or we've failed to connect to session cluster - in which case we don't need to shutdown the node's daemon anyway
			Zwas_Log::logException('Failed to add a node due to error on node. Attempting to release node.', $e);
			$this->model->ungracefulRemoveNode($addedNode, true);
			throw $e;
		}
		return $addedNode;
	}
		
	/**
	 * @param string $name
	 * @param string $address
	 * @param string $password
	 * @return Nodes_Element_Mapper
	 */
	protected function addAdditionalNode($name, $address, $password) {
		if (! $this->model->hasNodes()) {
			throw new Nodes_Model_Delegate_Exception(
				Zwas_Translate::_('Cluster already has connected servers; adding a server as the initial one is not allowed'), 
				Zwas_Exception::ASSERT
			);
		}
		
		$pluginBroker = new Nodes_Model_Plugin_Broker();
		/// configuration distribution must take place before all postAction plug-ins
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_DistributeConfiguration());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectMonitorToZSM());
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectNodeToSessionClustering());
		
		$nodesEnvironment = $this->getNodeEnvironment($address, $password);
		$this->checkNodeMatchesCluster($nodesEnvironment);
		
		/// store the node in the local database
		$addedNode = $this->saveNode($name, $address, $password, $nodesEnvironment);
		
		try {
			$pluginBroker->preAction($this->model, $addedNode);

			//add node actions
			$affectedNodes = array($addedNode);
			
			//post add node actions
			$pluginBroker->postAction($this->model, $affectedNodes);
			
			// Update the node's lighttpd's jobqueue.ini manually with the previously set job queue details
			// Done directly as this is a single-server action done after all is done
			$jobQueueUpdater = new Nodes_Model_Plugin_MaintainClusterJobQueueSettings();
			$jobQueueUpdater->preAction($this->model, $addedNode);
		} catch (Zwas_Exception $e) {
			/// we've either failed to add the node during monitor connect, which is handled in removeNode
			/// or we've failed to connect to session cluster - in which case we don't need to shutdown the node's daemon anyway
			Zwas_Log::logException('Failed to add a node due to error on node. Attempting to release node.', $e);
			$this->model->ungracefulRemoveNode($addedNode, true);
			throw $e;
		}
		return $addedNode;
	}	
	
	/**
	 * @param string $address
	 * @param string $password
	 * @throws Zwas_Exception
	 */
	private function checkNodeMatchesCluster(Nodes_Environment_Element $nodeEnvironment) {
		$representative = $this->model->getRepresentativeForWriting();
		if ($representative->getEnvironmentId() !== $nodeEnvironment->getEnvironmentId()) {
			$nodesMapper = new Nodes_Mapper();
			$repEnvironment = $nodesMapper->getEnvironmentById($representative->getEnvironmentId());
			Nodes_Environment_Element::compare($repEnvironment, $nodeEnvironment);
		}
		
		return true;
	}	
	
	/**
	 * @param string $name
	 * @param string $address
	 * @param string $password
	 * @param boolean $representative
	 * @return Nodes_Element_Mapper
	 */
	private function saveNode($name, $address, $password, Nodes_Environment_Element $environment, $representative = false) {
		// locking the node gui
		$this->commMapper->lockNode($address, $password);
		try {
			// add gui details to database
			$id = $this->nodesTable->addNode($name, $address, $password, $environment->getEnvironmentId(), $representative);
			$node = $this->model->getNode($id);
		} catch (Zwas_Exception $e) {
			$this->commMapper->unlockNode($address, $password);
			throw $e;
		} catch (Exception $e) {
			// if adding node to db failes, unlocking the node gui
			$this->commMapper->unlockNode($address, $password);
			Zwas_Log::logException('Could not add the requested server to the list', $e);
			throw new Zwas_Exception(Zwas_Translate::_('Could not add the server to the list, if this problem persists go to the Zend Support Center at: http://www.zend.com/support-center'), Zwas_Exception::ERROR);
		}
		
		// commit the node's new attributes into its row
		$node->save();
		$this->node = $node;
		return $node;
	}
	
	/**
	 * @param string $address
	 * @param string $password
	 * @return Nodes_Environment_Element
	 */
	private function getNodeEnvironment($address, $password) {
		$nodeEnvironment = $this->model->getCommMapper()->probeNode($address, $password);
		$nodesMapper = new Nodes_Mapper();
		return $nodesMapper->addEnvironment($nodeEnvironment);
	}

}