<?php
class Nodes_Model_Delegate_RemoveNode extends Nodes_Model_Delegate_Abstract {
	
	public function __construct(Nodes_Model $model) {
		parent::__construct($model);
		$this->setEventToReport(new ZwasComponents_MonitorUi_UiEvent_NodeRemoved());
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Element_Mapper
	 * @throws Nodes_Model_Delegate_Exception
	 */
	public function remove(Nodes_Element_Mapper $node) {
		if ((Nodes_Element::STATUS_OFFLINE == $node->getStatus()) || $node->isDisabled()) {
			/// update node state, maybe it's back and can be removed normally
			try {
				$this->model->getModelHeartbeat()->updateHeartBeat($node);
			} catch (Exception $e) {
				// TRANSLATE
				Zwas_Log::logException('Server revival encountered a problem', $e);
			}
		}

		if (Nodes_Element::STATUS_OFFLINE == $node->getStatus()) {
			$this->removeOfflineNode($node);
		} elseif (1 === $this->model->countNodes()) {
			$this->removeLastNode($node);
		} elseif ($this->isNodeReadyForGracefulShutdown($node)) {
			$this->gracefulShutdownSc($node);
		} else {
			/// SC is off and we may proceed regardless of the cluster's state
			if ($node->isRepresentative()) {
				$this->removeRepresentativeNode($node);
			} else {
				$this->removeNode($node);
			}
			GracefulShutdown::destroy();
		}
		return $this->getNode();
	}
		
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Model_Delegate
	 * @throws Nodes_Model_Delegate_Exception only for database failure
	 */
	public function detachOnly(Nodes_Element_Mapper $node) {
		$this->node = $node;
		$pluginBroker = new Nodes_Model_Plugin_Broker();
		
		// disconnect monitor daemon services
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_DetachMonitorFromZSM());
		
		try {
			$pluginBroker->preAction($this->model, $this->node);
		} catch(Zwas_Exception $e) {
			Zwas_Log::logException(Zwas_Translate::_("Could not detach monitor for server {$this->node->getAddress()} ({$e->getMessageObject()})"), $e);
		}
		
		try {
			// disconnect node
			$this->unlockNode();
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException(Zwas_Translate::_("Could not unlock server {$this->node->getAddress()} ({$e->getMessageObject()})"), $e);
		}
		
		$copyOfNode = clone $this->node;
		$copyOfNode->setReadOnly(true);
		
		$this->removeNodeElement();
		
		// attempt to create event, may fail with no event reported
		$this->createEvent($copyOfNode);
		try {
			$pluginBroker->postAction($this->model, $this->model->getRespondingNodes());
		} catch(Zwas_Exception $e) {
			/// Silently ignore the error - we are past the point of no return
			Zwas_Log::logException(Zwas_Translate::_('Could not perform cleanup after detachment'), $e);
		}
		return $this;
	}
	
	/**
	 * Bypass graceful shutdown and remove a node normally
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Element_Mapper
	 */
	public function ungracefulRemove(Nodes_Element_Mapper $node) {
		$node->setScStatus(ZwasComponents_SessionClustering_Model::SCD_STATUS_OFF); // override the sc service status
		$mapper = $this->remove($node);
		GracefulShutdown::destroy();
		return $mapper;
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Model_Delegate
	 * @throws Nodes_Model_Delegate_Exception
	 */
	protected function removeLastNode(Nodes_Element_Mapper $node) {
		$this->node = $node;
		$pluginBroker = new Nodes_Model_Plugin_Broker();
		
		// session clustering handler
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_DetachNodeFromSessionClustering());
		// disconnect monitor daemon services
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_DetachMonitorFromZSM());
		
		try {
			$pluginBroker->preAction($this->model, $this->node);
		} catch(Zwas_Exception $e) {
			throw new Nodes_Model_Delegate_Exception($e->getMessageObject());
		}
		// disconnect node
		$this->unlockNode();
		
		$copyOfNode = clone $this->node;
		$copyOfNode->setReadOnly(true);
		
		$this->removeNodeElement();
		
		/* 
		 * Since we are essentially emptying the cluster we have to make
		 * sure that CM does not attempt to connect to any node's 
		 * JobQueue daemon
		 */
		ZwasComponents_JobQueue_Binding::saveZwasBindingDirective('');
		// create event
		$this->createEvent($copyOfNode);
		// No post actions - there are no servers connected to work on
		return $this;
	}
	
	
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return boolean
	 * @throws Nodes_Model_Delegate_Exception
	 */
	protected function gracefulShutdownSc(Nodes_Element_Mapper $node) {
		parent::gracefulShutdownSc($node);
		
		if ($this->isNodeReadyForPostGraceful($node)) {
			return $this->remove($node);
		}
		$node->setTransition(Nodes_Element::SERVER_TRANSITION_TO_REMOVE)->save();
		GracefulShutdown::init();
		return $this;
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Model_Delegate
	 * @throws Nodes_Model_Delegate_Exception
	 */
	protected function removeNode(Nodes_Element_Mapper $node) {
		$this->node = $node;
		$pluginBroker = new Nodes_Model_Plugin_Broker();
		
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_DetachNodeFromSessionClustering());
		// disconnect monitor daemon services
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_DetachMonitorFromZSM());
		
		try {
			$pluginBroker->preAction($this->model, $this->node);
		} catch(Nodes_Model_Delegate_Exception $e) {
			if ($e->getCode() == Nodes_Model_Plugin_Exception::ABORT_PLUGIN_CHAIN) {
				Zwas_Log::logException('Node remove preAction chain was aborted', $e);
				return $this;
			}
		} catch(Nodes_Model_Plugin_Exception $e) {
			throw new Nodes_Model_Delegate_Exception($e->getMessageObject());
		}
		// disconnect node
		$this->unlockNode();
		
		$copyOfNode = clone $this->node;
		$copyOfNode->setReadOnly(true);
		
		$this->removeNodeElement();
		
		// create event
		$this->createEvent($copyOfNode);
		if ($this->model->hasNodes()) {
			$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectNodeToSessionClustering());
			try {
				$pluginBroker->postAction($this->model, $this->model->getRespondingNodes());
			} catch(Zwas_Exception $e) {
				/// Silently ignore the error - we are past the point of no return
				Zwas_Log::logException(Zwas_Translate::_('Could not perform cleanup after removing a representative'), $e);
			}
		}
		return $this;
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Model_Delegate
	 * @throws Nodes_Model_Delegate_Exception
	 */
	protected function removeRepresentativeNode(Nodes_Element_Mapper $node) {
		$this->node = $node;
		$pluginBroker = new Nodes_Model_Plugin_Broker();
		
		// session clustering handler
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_DetachNodeFromSessionClustering());
		// disconnect monitor daemon services
		$pluginBroker->registerPlugin(new Nodes_Model_Plugin_DetachMonitorFromZSM());
		
		try {
			$pluginBroker->preAction($this->model, $this->node);
		} catch(Nodes_Model_Delegate_Exception $e) {
			if ($e->getCode() == Nodes_Model_Plugin_Exception::ABORT_PLUGIN_CHAIN) {
				Zwas_Log::logException('Server remove preAction chain was aborted', $e);
				return $this;
			}
		} catch(Nodes_Model_Plugin_Exception $e) {
			throw new Nodes_Model_Delegate_Exception($e->getMessageObject());
		}
		
		// this is a representative, find a replacement candidate
		try {
			$this->model->replaceRepresentative();
		} catch(Zwas_Exception $e) {
			/// we've failed in the attempt to replace the representative.
			/// not much we can do about that...
		}
		// disconnect node
		$this->unlockNode();
		
		$copyOfNode = clone $this->node;
		$copyOfNode->setReadOnly(true);
		
		$this->removeNodeElement();
		
		// create event
		$this->createEvent($copyOfNode);
		if($this->model->hasNodes()) {
			// If the representative is the one which was removed, the saved data is no longer valid, as the reference is no longer a part
			// of the cluster.  Therefore, an update of the servers state is required
			$this->model->getModelHeartbeat()->analyzeNodesState(new Nodes_Collection($this->model->getEnabledNodes()));
			$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectNodeToSessionClustering());
			try {
				$pluginBroker->postAction($this->model, $this->model->getRespondingNodes());
			} catch(Zwas_Exception $e) {
				/// Silently ignore the error - we are past the point of no return
				Zwas_Log::logException(Zwas_Translate::_('Could not perform cleanup after removing a representative'), $e);
			}
		}
		return $this;
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Model_Delegate
	 * @throws Nodes_Model_Delegate_Exception
	 */
	protected function removeOfflineNode(Nodes_Element_Mapper $node) {
		$this->node = $node;
		$pluginBroker = new Nodes_Model_Plugin_Broker();
		
		$copyOfNode = clone $this->node;
		$copyOfNode->setReadOnly(true);
		
		$this->removeNodeElement();
		
		// create event
		$this->createEvent($copyOfNode);
		if ($this->model->hasNodes()) {
			$pluginBroker->registerPlugin(new Nodes_Model_Plugin_ConnectNodeToSessionClustering());
			try {
				$pluginBroker->postAction($this->model, $this->model->getRespondingNodes());
			} catch(Zwas_Exception $e) {
				/// Silently ignore the error - we are past the point of no return
				Zwas_Log::logException(Zwas_Translate::_('Could not perform cleanup after removing a representative'), $e);
			}
			
		}
		return $this;		
	}	
	
	/**
	 * @return Nodes_Model_Delegate_RemoveNode
	 * @throws Nodes_Model_Delegate_Exception
	 */
	private function unlockNode() {
		try {
			/// Any problem in the unlock method would return as an exception. Otherwise, the method always returns true.
			$this->commMapper->unlockNode($this->node->getAddress(), $this->node->getPassword());
		} catch(Zwas_Exception $e) {
			throw new Nodes_Model_Delegate_Exception(Zwas_Translate::_('Failed unlocking the server'));
		}
		return $this;
	}
	
	/**
	 * Remove a Nodel Element from the database
	 * @return Nodes_Element_Mapper
	 * @throws Nodes_Model_Delegate_Exception
	 */
	private function removeNodeElement() {
		$isRepresentative = $this->node->isRepresentative();
		
		// remove node from db
		try {
			$this->node->delete();
		} catch (Zend_Db_Table_Row_Exception $e) {
			Zwas_Log::logMessage("Could not remove the server by ID {$this->node->getId()}", Zwas_Exception::ERROR);
			throw new Nodes_Model_Delegate_Exception(Zwas_Translate::_(new Zwas_Text("You do not have permission to perform this action")));
		}
		
		return $this;
	}
}