<?php
class Nodes_Model_Heartbeat {
	
	/**
	 * @var Nodes_Model
	 */
	private $nodesModel;
	
	/**
	 * Perform a heartbeat collection process for the server passed from the outside.
	 * 
	 * @param Nodes_Element_Mapper $node
	 * @return Nodes_Element_Mapper
	 */
	public function updateHeartBeat(Nodes_Element_Mapper $node) {
		/// create a collection of self and run the heartbeat collection routine
		$selfCollection = new Nodes_Collection(array($node));
		$this->updateNodesState($selfCollection);
		return $node;
	}
	
	/**
	 * @param Nodes_Collection $nodes
	 * @return Nodes_Collection
	 * @throws Zwas_Exception
	 */
	public function updateNodesState(Nodes_Collection $nodes) {
		if ($nodes->count() === 0) {
			return $nodes;	
		}
		Zwas_Log::info(new Zwas_Text('--== Starting heartbeat collection for %s nodes ==--', array($nodes->count())));
		try {
			$heartbeats = $this->collectHeartBeats($nodes);
		} catch (Zwas_Exception $e) {
			throw new Zwas_Exception(new Zwas_Text('Error encountered while attempting to collect server states: %s', array($e->getMessage())));
		}
		
		Zwas_Log::debug(new Zwas_Text('Collected %s heartbeat responses', array(count($heartbeats))));
		
		$this->storeHeartBeats($nodes, $heartbeats);
		
		$this->analyzeNodesState($nodes);
		
		Zwas_Log::info('Heartbeat operation completed');
		return $nodes;
	}
	
	
	/**
	 * @param Nodes_Model $nodesModel
	 * @return Nodes_Model_Heartbeat
	 */
	public function setNodesModel(Nodes_Model $nodesModel) {
		$this->nodesModel = $nodesModel;
		return $this;
	}
	
	/**
	 * @param Nodes_Collection $nodes
	 * @throws Zwas_Exception
	 */
	public function analyzeNodesState(Nodes_Collection $nodes) {
		foreach ($nodes as $node) { 
			$this->setNodeStatus($node);
		}
		$nodes->save();
		// TRANSLATE
		Zwas_Log::info('Cluster members\' state analysis completed');
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @param array $extensions
	 */
	protected function setNodeExtensions(Nodes_Element_Mapper $node, array $extensions) {
		Context_Db::getNodesExtensionTable()->addExtensions($extensions, $node->getId());
		Zwas_Log::info(new Zwas_Text('Processed %s extensions for server %s', array(count($extensions), $node->getId())));
	}
	
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @param array $extensions
	 */
	protected function setNodeComponents(Nodes_Element_Mapper $node, array $components) {
		Context_Db::getNodesComponentTable()->addComponents($components, $node->getId());
		Zwas_Log::info(new Zwas_Text('Processed %s components for server %s', array(count($components), $node->getId())));
	}
	
	/**
	 * @param Nodes_Collection $nodes
	 * @param array $heartbeats
	 * @throws Zwas_Exception
	 */
	protected function storeHeartBeats(Nodes_Collection $nodes, array $heartbeats) {
		$offlineHeartbeat = new Node_Heartbeat();
		foreach ($nodes as $node) { /* @var $node Nodes_Element_Mapper */
			$nodeId = $node->getId();
			try {
				if ((isset($heartbeats[$nodeId])) && ($heartbeats[$nodeId] instanceof Node_Heartbeat)) {
					Zwas_Log::debug(new Zwas_Text("Starting heartbeat state storage for server %s", array($nodeId)));
					
					$heartbeat = $heartbeats[$nodeId]; /* @var $heartbeat Node_Heartbeat */
					$this->setNodeHeartbeat($node, $heartbeat);
					$this->storeNodeSignatures($node, $heartbeat);
					$this->setNodeEnvironment($node, $heartbeat);
					$this->setNodeExtensions($node, $heartbeat->getExtensions());
					$this->setNodeComponents($node, $heartbeat->getComponents());
				} else {
					Zwas_Log::debug(new Zwas_Text("No heartbeat found for server %s, using a default heartbeat", array($nodeId)));
					$this->setNodeHeartbeat($node, $offlineHeartbeat);
				}
				
				// Clean the flag of servers which were waiting for the next heartbeat to finish 
				if ($node->waitingForHeartbeat()) {
					$node->updateTransition(0);
				}
				Zwas_Log::debug(new Zwas_Text("Heartbeat state for server %s was stored successfully", array($nodeId)));
			} catch (Zwas_Exception $e) {
				Zwas_Log::logException('Could not store a node\'s state', $e);
			}
		}
		
		/// store everything in the database immediately
		/// avoid using an old representative in setNodeStatus
		$nodes->save();
		// TRANSLATE
		Zwas_Log::info('Stored heartbeat information for the cluster');
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 */
	protected function setNodeStatus(Nodes_Element_Mapper $node) {
		if (Nodes_Element::STATUS_OFFLINE == $node->getStatus()) {
			return;	//noting to be done
		}
		
		try {
			$masterSigsMatch = $node->isSignatureMatchingCluster();
			$masterDirectivesMatch = $node->isSignatureMatchingCluster(Node_HeartbeatGenerator::SIGNATURE_NAME_DIRECTIVES_MASTER);
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException(Zwas_Translate::_('Unable to compare signatures'), $e);
			$node->setStatus(Nodes_Element::STATUS_ERROR);
			return;
		}
		
		if ($masterSigsMatch) {
			Zwas_Log::debug(new Zwas_Text("Signatures for server %s match those of the cluster", array($node->getId())));
			$node->setStatus(Nodes_Element::STATUS_ONLINE);
			return;
		} else {
			Zwas_Log::debug(new Zwas_Text("Signatures for server %s do not match those of the cluster", array($node->getId())));
			$node->setStatus(Nodes_Element::STATUS_CONFIG_MISMATCH);
			return;
		}
	}
	
	/**
	 * Collect heartbeats
	 * @return array of nodeId => Node_Heartbeat
	 */
	protected function collectHeartBeats(Nodes_Collection $nodes) {
		$commMapper = $this->getNodesModel()->getCommMapper();
		$responses = $commMapper->distributeActionGetResponses($nodes->toArray(), 'Node', 'Heartbeat');
		
		$heartbeats = array();
		
		foreach ($responses as $response) { /* @var $response Nodes_Response */
			$nodeId = $response->getNodeId();
			if (! $response->isError()) {
				$heartbeat = $response->getData();
				
				if (! ($heartbeat instanceof Node_Heartbeat)) {
					try {
						$heartbeat = Node_Heartbeat::createFromXML($response->getData());
					} catch (Zwas_Exception $e) {
						Zwas_Log::logMessage(
							new Zwas_Text(Zwas_Translate::_('Received an unexpected response from server #%s'), array($nodeId)));
						
						continue;
					}
				}
				
				// There was a response from the node
				$nodes[$nodeId]->setStatus(Nodes_Element::STATUS_ONLINE);
				$heartbeats[$nodeId] = $heartbeat;
			} else {
				// there was no response from the node
				$nodes[$nodeId]->setStatus(Nodes_Element::STATUS_OFFLINE);
			}
		}
		return $heartbeats;
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @param Node_Heartbeat $heartbeat
	 */
	protected function setNodeHeartbeat(Nodes_Element_Mapper $node, Node_Heartbeat $heartbeat) {
		if (Nodes_Element::STATUS_OFFLINE != $node->getStatus()) {
			$node->setHeartbeatTime(time());
			$node->setAwaitRestart($heartbeat->awaitsRestart());
			$node->setScStatus($heartbeat->getScStatus());
			$node->setGuiTag($heartbeat->getGuiTag());
		} else {
			$node->setAwaitRestart(false);
			$node->setScStatus(0);
		}
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @param Node_Heartbeat $heartbeat
	 */
	protected function setNodeEnvironment(Nodes_Element_Mapper $node, Node_Heartbeat $heartbeat) {
		if (Nodes_Element::STATUS_OFFLINE != $node->getStatus()) {
			$nodesMapper = new Nodes_Mapper();
			$environment = $nodesMapper->addEnvironment($heartbeat->getNodeEnvironment());
			$node->setEnvironmentId($environment->getEnvironmentId());
		}
	}
	
	/**
	 * @param Nodes_Element_Mapper $node
	 * @param Node_Heartbeat $heartbeat
	 */
	protected function storeNodeSignatures(Nodes_Element_Mapper $node, Node_Heartbeat $heartbeat) {
		$validSignature = new Zend_Validate_StringLength(1);
		/// store all signatures into the signatures table
		$sigs = $heartbeat->getSignatures();
		if (is_array($sigs)) {
			Zwas_Log::debug(new Zwas_Text("Heartbeat for server %s contains %s signatures", array($node->getId(), count($sigs))));
			foreach($sigs as $name => $signature) {
				if($validSignature->isValid($signature)) {
					$node->setSignature($signature, $name);
				}
			}
		}
	}
	
	/**
	 * @return Nodes_Model
	 */
	private function getNodesModel() {
		if (is_null($this->nodesModel)) {
			$this->nodesModel = new Nodes_Model();
		}
		return $this->nodesModel;
	}
	
}