<?php

class Nodes_Table extends Zend_Db_Table_Abstract {
	
	protected $_name		= 'nodes';
	protected $_primary		= 'id';
	protected $_rowClass	= 'Nodes_Element';
	
	/**
	 * @return Zend_Db_Table_Rowset_Abstract
	 */
	public function getEnabledNodes() {
		return $this->fetchAll('disabled = 0');
	}
	
	/**
	 * @return integer
	 */
	public function countEnabledNodes() {
		$select = $this->select(self::SELECT_WITHOUT_FROM_PART);
		return $select->from($this->_name, 'COUNT(*)')
						->where('disabled = 0')
						->query()
						->fetchColumn(0);
	}
	
	/**
	 * 
	 * @param string $ipAddress
	 * @return Nodes_Element
	 */
	public function getNodeByAddress($ipAddress) {
		// Can't use the http prefix as it can be https as well
		return $this->fetchRow($this->getAdapter()->quoteInto('address LIKE ?', "%://$ipAddress:%"));
	}
	
	/**
	 * @return Zend_Db_Table_Rowset_Abstract
	 */
	public function getNodesList() {
		return $this->fetchAll();
	}
	
	/**
	 * @return Zend_Db_Table_Rowset_Abstract
	 */
	public function getRespondingNodes() {
		$select = $this->select();
		$select ->where('disabled = 0')
				->where('scStatus NOT IN (?)', array(ZwasComponents_SessionClustering_Model::SCD_STATUS_SHUTDOWN_IN_PROCESS, ZwasComponents_SessionClustering_Model::SCD_STATUS_STARTING_UP))
				->where('status != ?', Nodes_Element::STATUS_OFFLINE);
		return $this->fetchAll($select);
	}
	
	/**
	 * @return integer
	 */
	public function countRespondingNodes() {
		return (int)$this->select()->from($this->_name, 'COUNT(*) count_nodes')
						->where('disabled = 0')
						->where('scStatus NOT IN (?)', array(ZwasComponents_SessionClustering_Model::SCD_STATUS_SHUTDOWN_IN_PROCESS, ZwasComponents_SessionClustering_Model::SCD_STATUS_STARTING_UP))
						->where('status != ?', Nodes_Element::STATUS_OFFLINE)
						->query()->fetchColumn(0);
	}
	
	/**
	 * @return boolean
	 */
	public function hasDifferentEnvironmentId() {
		$adapter = $this->getAdapter();
		
		$distinct = $adapter->select()
			->distinct(true)
			->from($this->_name, 'environmentId')
			->where('disabled = 0')
			->query()
			->fetchAll();
			
		if (1 >= count($distinct)) {
			return false;
		}
		return true;
	}
	
	/**
	 * @return boolean
	 */
	public function hasNodesInRemovalTransition() {
		$where = $this->getAdapter()->quoteInto('transition = ?', Nodes_Element::SERVER_TRANSITION_TO_REMOVE);
		return (0 < count($this->fetchAll($where)));
	}	
	
	/**
	 * @return boolean
	 */
	public function hasNodesInDisableTransition() {
		$where = $this->getAdapter()->quoteInto('transition = ?', Nodes_Element::SERVER_TRANSITION_TO_DISABLE);
		return (0 < count($this->fetchAll($where)));
	}	
	
	/**
	 * @return Zend_Db_Table_Rowset
	 */
	public function getNodesListOrderedByName() {
		return $this->fetchAll(null, "name ASC");
	}
	
	/**
	 * @return Nodes_Element
	 */
	public function getNodeInShutdownProcess() {
		$where = $this->getAdapter()->quoteInto('scStatus = ?', ZwasComponents_SessionClustering_Model::SCD_STATUS_SHUTDOWN_IN_PROCESS);
		return $this->fetchRow($where);
	}
	
	/**
	 * @return boolean
	 */
	public function hasNodeInShutdownProcess() {
		return (! is_null($this->getNodeInShutdownProcess()));
	}
	
	/**
	 * @return Nodes_Element
	 */
	public function getNodeInStartupProcess() {
		$where = $this->getAdapter()->quoteInto('scStatus = ?', ZwasComponents_SessionClustering_Model::SCD_STATUS_STARTING_UP);
		return $this->fetchRow($where);
	}
	
	/**
	 * @return boolean
	 */
	public function hasNodeInStartupProcess() {
		return (! is_null($this->getNodeInStartupProcess()));
	}
	
	/**
	 * @return boolean
	 */
	public function hasUnstableNode() {
		$adapter = $this->getAdapter();
		if (0 == $this->countNodes()) {
			return false;
		}
		$status = $adapter->quoteInto('status != ?', Nodes_Element::STATUS_OFFLINE);
		$scStatusGS = $adapter->quote(ZwasComponents_SessionClustering_Model::SCD_STATUS_SHUTDOWN_IN_PROCESS);
		$scStatusSU = $adapter->quote(ZwasComponents_SessionClustering_Model::SCD_STATUS_STARTING_UP);
		
		return (! is_null($this->fetchRow("$status AND scStatus IN($scStatusGS, $scStatusSU)")));
	}
	
	/**
	 * @param integer $id
	 * @return Nodes_Element
	 */
	public function getNodeById($id) {
		return $this->find($id)->current();
	}
	
	/**
	 * @return integer
	 */
	public function countNodes() {
		return $this->select()->from($this->_name, 'COUNT(*) count_nodes')->query()->fetchColumn(0);
	}
	
	/**
	 * @param string	$name
	 * @param string	$uri
	 * @param string	$password
	 * @param boolean   $representative
	 * @param integer	$environmentId
	 * @return integer
	 */
	public function addNode($name, $uri, $password, $environmentId, $representative = false) {
		try {
		$id = $this->createRow()
				->setId(Metadata_Model::getNextServerId())
				->setName((string)$name)
				->setAddress((string)$uri)
				->setPassword((string)$password)
				->setEnvironmentId(intval($environmentId))
				->save();
		} catch (Zend_Db_Statement_Exception $e) {
			$previous = null;
			if ($e->hasChainedException()) {
				$previous = $e->getChainedException();/*@var PDOException $previous */
				$previous->getCode();
			}
			
			switch(is_null($previous) ? 0 : $previous->getCode()) {
				case 23000: /// duplicate key
					Zwas_Log::logException('Duplicate unique key, could not add the node', $e);
					throw new Zwas_Exception(Zwas_Translate::_('This name is already in use, select a different name and repeat your last action'), Zwas_Exception::ERROR);
					break;
				default:
					Zwas_Log::logException('Could not add the requested server to the list', $e);
					throw new Zwas_Exception(Zwas_Translate::_('Could not add the server to the list, if this problem persists go to the Zend Support Center at: http://www.zend.com/support-center'), Zwas_Exception::ERROR);
			}
			throw new Zwas_Exception($e->getMessage());
		}
		
		if ($representative) {
			$this->setRepresentative($id);
		}
		return $id;
	}
	
	/**
	 * Retrieves a valid, responsive representative
	 * @return Nodes_Element
	 * @throws Zwas_Exception if no representative is found or the rep is offline or disabled
	 */
	public function getRepresentative() {
		$representative = $this->getCurrentRepresentative();
		
		if ((! ($representative instanceof Nodes_Element))
			|| ($representative->getStatus() == Nodes_Element::STATUS_OFFLINE)
			|| ($representative->isDisabled())) {
				
			Zwas_Log::logMessage(Zwas_Translate::_('Failed locating the representative server'));
			throw new Zwas_Exception(Zwas_Translate::_('An internal application error has occurred.  If this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center'));
		}
		
		return $representative;
	}
	
	/**
	 * @return Nodes_Element|null
	 */
	public function getLastRespondingServer() {
		$adapter = $this->getAdapter();
		
		$exp = new Zend_Db_Expr(
			'(CASE WHEN ' . $adapter->quoteInto('status = ?', Nodes_Element::STATUS_ONLINE) . ' THEN 0
					WHEN ' . $adapter->quoteInto('status = ?', Nodes_Element::STATUS_CONFIG_MISMATCH) . ' THEN 1
					WHEN ' . $adapter->quoteInto('status = ?', Nodes_Element::STATUS_ERROR) . ' THEN 2 END) ASC'
		);
		
		$where = $this->select();
		$where->where('disabled = ?', 0);
		$where->where('status <> ?', Nodes_Element::STATUS_OFFLINE);
		$where->order(array($exp, 'metaHBdate DESC'));
		$row = $this->fetchRow($where);

		return $row;
	}

	/**
	 * @return integer
	 */
	public function getMaxServerId() {
		$row = $this->fetchRow('1', 'id DESC');
		
		if (is_null($row)) {
			return 0;
		}
		return $row->getId();
	}

	/**
	 * @return integer 
	 */
	public function getRepresentativeEnvironmentId() {
		$adapter = $this->getAdapter();
		
		$where = $this->select();
		$where->where('representative =?', 1);
		$row = $this->fetchRow($where); /* @var Nodes_Element $row */
		
		return $row->getEnvironmentId();
	}
	
	/**
	 * @return integer
	 */
	public function getMaxMetaHBdate() {
		$row = $this->fetchRow('1', 'metaHBdate DESC'); /** @var Nodes_Element $row **/
		
		if (is_null($row)) {
			return 0;
		}
		return $row->getHeartbeatTime();
	}
	
	/**
	 * @param integer $id
	 * @return boolean
	 */
	public function setRepresentative($id) {
		try {
			$this->getAdapter()->beginTransaction();
			//update all rows, unset the representative
			$this->update(array('representative' => 0), '');
			//set current node to representative
			$where = $this->getAdapter()->quoteInto('id = ?', $id);
			$result = $this->update(array('representative' => 1), $where);
			$this->getAdapter()->commit();
			Zwas_Log::logMessage('Representative was replaced', Zend_Log::INFO);
		} catch (Exception $e) {
			$this->getAdapter()->rollBack();
			Zwas_Log::logException('Error happens when set representative node', $e);
		}
		return $result > 0;
	}

	/**
	 * Retrieves the currently designated representative regardless of state
	 * @return Nodes_Element|null
	 */
	public function getCurrentRepresentative() {
		$representative = $this->fetchRow('representative = 1');
		if (! ($representative instanceof Nodes_Element)) {
			Zwas_Log::logMessage('The cluster has no representative server');
			throw new Zwas_Exception(Zwas_Translate::_('An internal application error has occurred.  If this problem persists, go to the Zend Support Center at: http://www.zend.com/support-center'));
		}
		
		return $representative;
	}
}