<?php
class Upgrade_Conversion_Db_2x0 extends Upgrade_Conversion_Db_Abstract {
	
	public function __construct() {
		$this->originVersion = '1.2.0';
		$this->targetVersion = '2.0';
	}
	
	/**
	 * @see Upgrade_Conversion_Db_Abstract::convert()
	 * @param Zend_Db_Adapter_Abstract $adapter
	 * @throws Conversion_Exception
	 */
	public function convert(Zend_Db_Adapter_Abstract $adapter) {
		try {
			$nodes = $this->retrieveAllNodes($adapter);
			$this->rebuildDatabase($adapter);
			$this->enterNewNodes($adapter, $nodes);
		} catch (Zend_Db_Exception $e) {
			throw new Upgrade_Conversion_Exception($e->getMessage(), Upgrade_Conversion_Exception::ERROR);
		} catch (Zwas_Exception $e) {
			throw new Upgrade_Conversion_Exception($e->getMessageObject(), Upgrade_Conversion_Exception::ERROR);
		}
	}
	
	/**
	 * @param Zend_Db_Adapter_Abstract $adapter
	 * @param array $nodes
	 * @throws Zend_Db_Exception
	 */
	protected function enterNewNodes(Zend_Db_Adapter_Abstract $adapter, array $nodes) {
		
		foreach ($nodes as $node) {
			
			if (array_key_exists('signature', $node)) {
				unset($node['signature']);
			}
			if (array_key_exists('lastUpdated', $node)) {
				unset($node['lastUpdated']);
			}
			
			$nodeNewProperties = array(
				'environmentId'	=> 0,
				'transition'	=> 0,
				'metaUdate'		=> time(),
				'metaCdate'		=> time(),
			);
			
			$uri = Zwas_Uri::factory($node['address']);
			if (false === $uri->getPort()) {
				$uri->setPort(10081);
			}
			
			if (false === $uri->getPath()) {
				$uri->setPath('/ZendServer');
			}
			
			$nodeNewProperties['networkHost']	= $uri->getHost();
			$nodeNewProperties['networkPort']	= $uri->getPort();
			$nodeNewProperties['networkScheme']	= $uri->getScheme();
			$nodeNewProperties['networkUri']	= $uri->getPath();
			
			$node = array_merge($node, $nodeNewProperties);
			$adapter->insert('nodes', $node);
		}
	}
	
	/**
	 * @param Zend_Db_Adapter_Abstract $adapter
	 * @return array
	 * @throws Zend_Db_Exception
	 */
	protected function retrieveAllNodes(Zend_Db_Adapter_Abstract $adapter) {
		$nodesSelect = $adapter->select();
		return $nodesSelect->from('nodes')->query(Zend_Db::FETCH_ASSOC)->fetchAll();
	}
	
	/**
	 * @param Zend_Db_Adapter_Abstract $adapter
	 * @throws Zend_Db_Exception
	 */
	protected function rebuildDatabase(Zend_Db_Adapter_Abstract $adapter) {
		$adapter->query('DROP TABLE nodes');
		$adapter->query(<<<CREATESQL
			CREATE TABLE "main"."nodes" (
				"id" INTEGER PRIMARY KEY NOT NULL ,
				"name" VARCHAR UNIQUE NOT NULL ,
				"address" VARCHAR UNIQUE NOT NULL ,
				"password" VARCHAR NOT NULL,
				"representative" INTEGER NOT NULL  DEFAULT 0,
				"status" INTEGER NOT NULL DEFAULT 0,
				"transition" INTEGER NOT NULL DEFAULT 0,
				"disabled" INTEGER NOT NULL DEFAULT 0,
				"scStatus" INTEGER NOT NULL DEFAULT 0,
				"awaitsRestart" BOOLEAN NOT NULL DEFAULT 0,
				"guiTag" VARCHAR,
				"environmentId" INTEGER NOT NULL,
				"metaUdate" INTEGER NOT NULL,
				"metaCdate" INTEGER NOT NULL,
				"metaHBdate" INTEGER,
				"networkScheme" VARCHAR NOT NULL,
				"networkHost" VARCHAR NOT NULL,
				"networkPort" INTEGER NOT NULL,
				"networkUri"  VARCHAR NOT NULL
			);
CREATESQL
		);
		$adapter->query('CREATE UNIQUE INDEX "main"."networkAddressKey" ON "nodes"("networkScheme", "networkHost", "networkPort", "networkUri")');
		
		// Add new tables
		$adapter->query(<<<CREATESQL
			CREATE TABLE "main"."environment" (
				"environmentId" INTEGER PRIMARY KEY NOT NULL,
				"clusterId" INTEGER NOT NULL DEFAULT 0,
				"profileId" INTEGER NOT NULL DEFAULT 0,
				"operatingSystem" VARCHAR DEFAULT NULL,
				"installationPath" VARCHAR DEFAULT NULL,
				"serverVersion" VARCHAR NOT NULL,
				"phpVersion" VARCHAR NOT NULL,
				"environmentHash" CHAR(40) NOT NULL
			);
CREATESQL
		);
		$adapter->query('CREATE UNIQUE INDEX "main"."environmentHashKey" ON "environment"("clusterId", "profileId", "environmentHash")');
		
		$adapter->query(<<<CREATESQL
			CREATE TABLE "extension" ( 
				"nodeId" INTEGER NOT NULL ,
				"name" VARCHAR NOT NULL, 
				"filename" VARCHAR NOT NULL, 
				"fileExists" BOOLEAN NOT NULL, 
				"loaded" BOOLEAN NOT NULL, 
				"iniStatus" INT NOT NULL, 
				"iniFile" VARCHAR NOT NULL, 
				"awaitsRestart" BOOLEAN NOT NULL,
				"errorString" VARCHAR NOT NULL,
				"extensionHash" CHAR(40) NOT NULL,
				"numOfDirectives" INT NOT NULL DEFAULT 0,
				PRIMARY KEY ("nodeId","name")
			);		
CREATESQL
		);
		
		$adapter->query(<<<CREATESQL
			CREATE TABLE "component" (
				"nodeId" INTEGER NOT NULL , 
				"name" VARCHAR NOT NULL , 
				"installed" BOOL NOT NULL , 
				"version" VARCHAR NOT NULL , 
				"errorString" VARCHAR NOT NULL ,
				"awaitsRestart" BOOL NOT NULL ,
				"daemonStatus" INTEGER DEFAULT NULL,
				"enabled" BOOL DEFAULT NULL,
				"componentHash" CHAR(40) NOT NULL ,
				PRIMARY KEY ("nodeId","name")
			);
CREATESQL
		);

		$adapter->query(<<<RENAMESQL
			ALTER TABLE 'signatures' RENAME TO 'nodesSignature'
RENAMESQL
		);
		$adapter->query('INSERT INTO "main"."metadata" (name, value, cdate) VALUES ("version", "'. $this->targetVersion .'", '.time().');');
		
	}
}