<?php
class Upgrade_Conversion_Db_Model {
	
	const BACKUP	= 'backup';
	const UPGRADE	= 'upgrade';
	/**
	 * @var string
	 */
	private $databasePath;
	
	/**
	 * @var string
	 */
	private $destinationPath;
	
	/**
	 * @var Zend_Db_Adapter_Abstract
	 */
	private $dbAdapter;
	
	/**
	 * @var array
	 */
	private $conversionPlan;
	
	/**
	 * @param string $filepath
	 * @param string $destinationPath
	 */
	public function __construct($filepath, $destinationPath) {
		Zwas_Log::info('--== Starting database upgrade script ==--');
		try {
			self::checkSourceFile($filepath);
			self::checkTargetDestination($destinationPath);
			$this->databasePath = self::createWorkCopy($filepath);
			if ($filepath == $destinationPath) {
				if (! Context_FileSystem::unlink($filepath)) {
					// TRANSLATE
					throw new Zwas_Exception('Could not remove the original file following relocation');
				}
				// TRANSLATE
				Zwas_Log::debug(new Zwas_Text('Original and destination paths are the same, removed the original file'));
			}
		} catch (Zwas_Exception $e) {
			Zwas_Log::err(new Zwas_Text('Failed during initialization: %s', array((string)$e->getMessageObject())));
			throw $e;
		}
		$this->destinationPath = $destinationPath;
		Zwas_Log::debug('Conversion model initialization completed');
	}
	
	private static function createWorkCopy($filepath) {
		$relocatedFilePath = $filepath. '.'.self::UPGRADE;
		if (! Context_FileSystem::copy($filepath, $relocatedFilePath)) {
			// TRANSLATE
			throw new Zwas_Exception('Could not relocate the original file');
		}
		Zwas_Log::debug(new Zwas_Text('Created working copy for the upgrade in %s', array($relocatedFilePath)));
		return $relocatedFilePath;
	}
	
	/**
	 * @return Zend_Db_Adapter_Abstract
	 */
	public function getAdapter() {
		if (is_null($this->dbAdapter)) {
			$this->dbAdapter = Zend_Db::factory('Pdo_Sqlite', array('dbname' => $this->databasePath));
		}
		return $this->dbAdapter;
	}
	
	/**
	 * @param Zend_Db_Adapter_Abstract $adapter
	 * @return Upgrade_Conversion_Db_Model
	 */
	public function setAdapter(Zend_Db_Adapter_Abstract $adapter) {
		$this->dbAdapter = $adapter;
		return $this;
	}
	
	/**
	 * Convert the database specific in the initialization filepath
	 * @return string
	 * @throws Conversion_Exception
	 */
	public function convert() {
		$originVersion = $this->retrieveDbVersion($this->getAdapter());
		Zwas_Log::info(new Zwas_Text('Current database version is %s', array($originVersion)));
		if ($originVersion == $this->latestTargetVersion()) {
			Zwas_Log::info(new Zwas_Text('No conversion is needed, schema version is already at %s', array($this->latestTargetVersion())));
			return $this->databasePath;
		}
		$conversionCollection = $this->buildConversionCollection();
		self::backupSourceFile($this->databasePath, $originVersion);
		$conversionCollection->setAdapter($this->getAdapter());
		try {
			$finalPath = $conversionCollection->convert($this->databasePath);
		} catch (Upgrade_Conversion_Exception $e) {
			Zwas_Log::err(new Zwas_Text('Conversion failed during execution: %s', array((string)$e->getMessageObject())));
			throw $e;
		}
		Zwas_Log::info(new Zwas_Text('Conversion from schema version %s to %s completed successfully', array($originVersion, $this->latestTargetVersion())));
		return $finalPath;
	}
	
	/**
	 * @param string $finalPath filepath to copy to the final destination
	 * @return boolean
	 */
	public function copyToFinalDestination($finalPath) {
		$finalDestination = $this->destinationPath;
		$copied = Context_FileSystem::copy($finalPath, $finalDestination);
		if ($copied) {
			// TRANSLATE
			Zwas_Log::debug('Copied database file to ' . $finalDestination);
		} else {
			// TRANSLATE
			Zwas_Log::debug('Database file could not be copied to ' . $finalDestination);
		}
		return $copied;
	}
	
	/**
	 * Find the latest version in the current plan
	 * @return string
	 */
	protected function latestTargetVersion() {
		return Context_Db_Factory::CLUSTER_DB_VERSION;
	}
	
	/**
	 * @param array $conversionPlan
	 * @return Upgrade_Conversion_Db_Model
	 */
	public function setConversionPlan($conversionPlan) {
		$this->conversionPlan = $conversionPlan;
		return $this;
	}

	/**
	 * Build a collection with a set of conversions to execute
	 * @return Upgrade_Conversion_Db_Collection
	 */
	protected function buildConversionCollection() {
		if (is_null($this->conversionPlan)) {
			$this->conversionPlan = new Upgrade_Conversion_Db_Collection(array(new Upgrade_Conversion_Db_2x0()));
		}
		return $this->conversionPlan;
	}
	
	/**	
	 * @param string $destinationPath
	 * @throws Conversion_Exception
	 * 
	 */
	protected static function checkTargetDestination($destinationPath) {
		$dataPath = dirname($destinationPath);
		
		try {
			$dataDir = Context_FileSystem::getDirectoryObject($dataPath);
		} catch (Zwas_Exception $e) {
			throw new Upgrade_Conversion_Exception((string)$e->getMessageObject(), Upgrade_Conversion_Exception::DIRECTORY_ERROR);
		}
		
		if (! $dataDir->isDir()) {
			throw new Upgrade_Conversion_Exception(
				new Zwas_Text(Zwas_Translate::_('The database path %s is not a valid directory'), array($dataPath))
				, Upgrade_Conversion_Exception::DIRECTORY_NOT_A_DIR);
		}
		
		$testWriteFile = Zwas_Path::create($dataPath, 'test');
		try {
			Context_FileSystem::getFileObject($testWriteFile, 'w');
		} catch (Zwas_Exception $e) {
			throw new Upgrade_Conversion_Exception((string)$e->getMessageObject(), Upgrade_Conversion_Exception::DIRECTORY_NOT_WRITABLE);
		}
		
		try {
			Context_FileSystem::unlink($testWriteFile);
		} catch (Zwas_Exception $e) {
			// TRANSLATE
			Zwas_Log::debug(new Zwas_Text('The test file \'%s\' could not be removed', array($testWriteFile)));
		}
		
		Zwas_Log::debug('Target Destination is verified to be accessible and writable');
	}
	
	/**
	 * Retrieve the cluster.db file's schema version
	 * @param Zend_Db_Adapter_Abstract $adapter
	 * @return string
	 * @throws Conversion_Exception
	 */
	protected function retrieveDbVersion(Zend_Db_Adapter_Abstract $adapter) {
		$metaDataTable = Context_Db::getMetadataTable()->setOptions(array(Zend_Db_Table_Abstract::ADAPTER => $adapter));/* @var $metaDataTable Metadata_Table */
		try {
			return $metaDataTable->getCurrentVersion();
		} catch (Zwas_Exception $e) {
			Zwas_Log::err(new Zwas_Text('Could not retrieve the schema version: %s', array($e->getMessageObject())));
			throw new Upgrade_Conversion_Exception(
				new Zwas_Text(Zwas_Translate::_('Could not retrieve the schema version: %s'), 
				array($e->getMessageObject()))
			);
		}
	}
	
	/**
	 * @param string $filepath
	 * @throws Conversion_Exception
	 */
	protected static function checkSourceFile($filepath) {
		$file = null;
		try {
			$file = Context_FileSystem::getFileObject($filepath);
		} catch (Zwas_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException(Zwas_Translate::_('File error'), $e);
			
			if (false !== strpos($filepath, '_')) {
				$message = new Zwas_Text(
					// TRANSLATE
					Zwas_Translate::_('%s, also filepath should not contain underscores'),
					array((string)$e->getMessageObject())
				); 
			} else {
				$message = (string)$e->getMessageObject();
			}
			throw new Upgrade_Conversion_Exception($message, Upgrade_Conversion_Exception::FILE_ERROR);
		}
		
		if (! $file->isFile()) {
			throw new Upgrade_Conversion_Exception(
				new Zwas_Text(Zwas_Translate::_('The filepath %s does not point to a file'), array($filepath))
				, Upgrade_Conversion_Exception::FILE_NOT_A_FILE);
		}
		
		if (! $file->isReadable()) {
			throw new Upgrade_Conversion_Exception(
				new Zwas_Text(Zwas_Translate::_('The file %s is not readable'), array($filepath))
				, Upgrade_Conversion_Exception::FILE_NOT_READABLE);
		}
		Zwas_Log::debug('Source file exists and is accessible');
	}

	/**
	 * @param string $filepath	
	 * @param string $dbVersion
	 * @return boolean
	 * @throws Conversion_Exception
	 */
	protected static function backupSourceFile($filepath, $dbVersion) {
		$basename = basename($filepath);
		$suffix = self::BACKUP;
		$copyFilename = Zwas_Path::create(dirname($filepath), "{$basename}.{$dbVersion}.{$suffix}");
		$copyBackup = Context_FileSystem::copy($filepath, $copyFilename);
		if ($copyBackup) {
			Zwas_Log::debug('Backed up file to '. $copyFilename);
		} else {
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('Back up operation failed'));
		}
		return $copyBackup;
	}
}

