<?php

class ZwasComponents_ZSM_Monitor_DatabaseManager {

	const MINIMUM_DB_VERSION		= 5;
	const ZEND_USERNAME				= 'zend';
	const ZEND_MONITOR_SCHEMA		= 'zend_monitor';
	const ZEND_MONITOR_DBTYPE		= 'MYSQL';
	const MONITOR_DBSCHEMA_FILENAME	= 'mysql_create_monitor_db.sql';
	
	/**
	 * @var Zend_Db_Adapter_Pdo_Mysql
	 */
	private $dbAdapter;
	/**
	 * @var Zend_Http_Client
	 */
	private $httpAdapter;
	
	/**
	 * @var string
	 */
	private $host;
	
	/**
	 * @var integer
	 */
	private $port;
	
	/**
	 * @var string
	 */
	private $user;
	
	/**
	 * @var string
	 */
	private $pass;
	
	/**
	 * @var string
	 */
	private $db;
	
	/**
	 * Username generated and stored during user creation
	 * @var string
	 */
	private $zendUser;
	
	/**
	 * User password generated and stored during user creation
	 * @var string
	 */
	private $zendUserPassword;
	
	/**
	 * The permission column names required by the creation process of the creator user
	 * so that we may create a new user and schema
	 * @var array
	 */
	private $requiredZendUserPermissions = array(
		'Create_priv' 				=> 'CREATE',
		'Drop_priv' 				=> 'DROP',
		'Alter_priv' 				=> 'ALTER',
		'Delete_priv' 				=> 'DELETE',
		'Index_priv' 				=> 'INDEX',
		'Insert_priv' 				=> 'INSERT',
		'Select_priv' 				=> 'SELECT',
		'Update_priv' 				=> 'UPDATE',
		'Create_tmp_table_priv' 	=> 'CREATE TEMPORARY TABLES',
		'Lock_Tables_priv' 			=> 'LOCK TABLES',
		'Create_view_priv' 			=> 'CREATE VIEW',
		'Show_view_priv' 			=> 'SHOW VIEW',
		'Alter_routine_priv' 		=> 'ALTER ROUTINE',
		'Create_routine_priv' 		=> 'CREATE ROUTINE',
		'Execute_priv' 				=> 'EXECUTE',
	);
	
	private $requiredCreatorPermissions = array(
		'Create_priv' 				=> 'CREATE',
		'Drop_priv' 				=> 'DROP',
		'Alter_priv' 				=> 'ALTER',
		'Delete_priv' 				=> 'DELETE',
		'Index_priv' 				=> 'INDEX',
		'Insert_priv' 				=> 'INSERT',
		'Select_priv' 				=> 'SELECT',
		'Update_priv' 				=> 'UPDATE',
		'Create_tmp_table_priv' 	=> 'CREATE TEMPORARY TABLES',
		'Lock_Tables_priv' 			=> 'LOCK TABLES',
		'Create_view_priv' 			=> 'CREATE VIEW',
		'Show_view_priv' 			=> 'SHOW VIEW',
		'Alter_routine_priv' 		=> 'ALTER ROUTINE',
		'Create_routine_priv' 		=> 'CREATE ROUTINE',
		'Execute_priv' 				=> 'EXECUTE',
		'Create_user_priv'			=> 'CREATE USER',
		'Reload_priv'				=> 'RELOAD',
		'Grant_priv'				=> 'GRANT OPTION',
	);
	
	/**
	 * @return the $zendUser
	 */
	public function getZendUser() {
		return $this->zendUser;
	}

	/**
	 * @return the $zendUserPassword
	 */
	public function getZendUserPassword() {
		return $this->zendUserPassword;
	}

	/**
	 * @param string $newHost
	 * @param integer $newPort
	 * @param string $newUser
	 * @param string $newPassword
	 * @param string $dbName
	 */
	public function __construct($newHost, $newPort, $newUser, $newPassword, $dbName = null) {
		$this->port = $newPort;
		$this->host = $newHost;
		$this->user = $newUser;
		$this->pass = $newPassword;
		$this->db = $dbName;
	}
	
	/**
	 * @return boolean
	 * @throws Zwas_Exception
	 */
	public function checkDatabaseTcp() {
		try {
			$adapter = $this->getHttpAdapter();
			$adapter->setConfig(array('timeout' => 2));
			$adapter->connect($this->host, $this->port);
		} catch (Zend_Http_Client_Adapter_Exception $e) {
			Zwas_Log::logException('Failed in tcp communications check', $e);
			$adapter->close();
			// TRANSLATE
			$errorObject = new Zwas_Text(Zwas_Translate::_('Could not connect to %s:%s'), array($this->host, $this->port));
			throw new Zwas_Exception($errorObject);
		}
		$adapter->close();
		return true;
	}
	
	/**
	 * Check the database credentials by creating a connection
	 * The connection is stored in the model for future use
	 * 
	 * @return boolean
	 * @throws Zwas_Exception|ZwasComponents_ZSM_Monitor_DatabaseManager_Exception
	 */
	public function checkDatabaseConnection() {
		$adapter = $this->getDbAdapter();
		try {
			$adapter->getConnection();
			$serverVersion = $adapter->getServerVersion();
			if (version_compare($serverVersion, self::MINIMUM_DB_VERSION, '<')) {
				// TRANSLATE
				throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('Installed database version must be at least %s'), array(self::MINIMUM_DB_VERSION)));
			}
		} catch (Zend_Db_Adapter_Exception $e) {
			// TRANSLATE
			$errorObject = new Zwas_Text(Zwas_Translate::_('Could not connect to the database at %s:%s (%s)'), array($this->host, $this->port, $e->getMessage()));
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception($errorObject);			
		}
		return true;
	}
	
	/**
	 * Check if zend's user was already created for this database
	 * @param string $password
	 * @param string $hostname
	 * @return boolean
	 * @throws ZwasComponents_ZSM_Monitor_DatabaseManager_Exception|Zwas_Exception
	 */
	public function createZendUser($password, $hostname) {
		$adapter = $this->getDbAdapter();
		if (! $adapter->isConnected()) {
			// TRANSLATE
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(Zwas_Translate::_('No database connection is available'));
		}
		
		/// Store the new credentials to be used by the system during normal operations
		$this->zendUser = self::ZEND_USERNAME;
		$this->zendUserPassword = $password;
		
		try {
			$select = $adapter->select();
			$statement = $select->from('user','count(*) userCount','mysql')
								->where("User = ?", $this->zendUser)
								->query();
			if (0 == $statement->fetchColumn(0)) {
				/// create a user
				$adapter->query("CREATE USER '$this->zendUser'@'localhost' IDENTIFIED BY '$password'");
				$adapter->query("CREATE USER '$this->zendUser'@'%' IDENTIFIED BY '$password'");
				$adapter->query("CREATE USER '$this->zendUser'@'$hostname' IDENTIFIED BY '$password'");
			} else {
				// TRANSLATE
				throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_("User '%s' already exists"), array($this->zendUser)));
			}
		} catch (Zend_Db_Adapter_Exception $e) {
			$subException = $e->getChainedException(); /* @var $subException PDOException */
			// TRANSLATE
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('User creation failed for user %s (%s)'), array($this->zendUser, $subException->getMessage())));
		} catch (Zend_Db_Exception $e) {
			// TRANSLATE
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('User creation failed for user %s (%s)'), array($this->zendUser, $e->getMessage())));
		}		
		return true;
	}
	
	/**
	 * Attempt to create the database schema as retrieved from the file
	 * in application/EE/models/ZwasComponents/ZSM/Monitor/zendMonitor.sql
	 * @param string $schemaName
	 * @param string $hostname
	 * @return boolean
	 * @throws Zwas_Exception|ZwasComponents_ZSM_Monitor_DatabaseManager_Exception
	 */
	public function createSchema($schemaName, $hostname) {
		$adapter = $this->getDbAdapter();
		if (! $adapter->isConnected()) {
			// TRANSLATE
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(Zwas_Translate::_('No database connection is available'));
		}
		$zendUsername = self::ZEND_USERNAME;
		$createdDatabase = false;
		try {
			try {
				$filepath = $this->getDatabaseSchemaFilePath();
				$createSchemaScriptFile = Context_FileSystem::getFileObject($filepath);
			} catch (Zwas_Exception $e) {
				// TRANSLATE
				throw new Zwas_Exception(Zwas_Translate::_('Unable to access the database structure script'));
			}
			
			try {
				/// create database
				$adapter->query("CREATE DATABASE IF NOT EXISTS `$schemaName`");
				/// "use" the database - unfortunately this is required
				/// the zend adapter does not allow us to switch schemas in mid stride
				$adapter->query("use `$schemaName`");
				$createdDatabase = true;
			} catch (Zend_Db_Exception $e) {
				// TRANSLATE
				throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to create a database named %s (%s)'), array($schemaName, $e->getMessage())));
			}
			
			try {
				/// create tables
				$queries = array();
				$queries = explode(';', $createSchemaScriptFile->readAll());
				foreach($queries as $query) {
					$query = trim($query);
					if(0 < strlen($query)) {
						$adapter->query($query);
					}
				}
			} catch (Zend_Db_Adapter_Exception $e) {
				$subException = $e->getChainedException(); /* @var $subException PDOException */
				// TRANSLATE
				throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to create a database named %s (%s)'), array($schemaName, $subException->getMessage())));
			} catch (Zend_Db_Exception $e) {
				// TRANSLATE
				throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to build the database schema (%s)'), array($e->getMessage())));
			}
			
			$permissionsList = self::getPermissionsList();
			
			try {
				/// grant permissions
				$adapter->query("GRANT $permissionsList ON `$schemaName`.* TO '$zendUsername'@'localhost'");
				$adapter->query("GRANT $permissionsList ON `$schemaName`.* TO '$zendUsername'@'%'");
				$adapter->query("GRANT $permissionsList ON `$schemaName`.* TO '$zendUsername'@'$hostname'");
			} catch (Zend_Db_Adapter_Exception $e) {
				$subException = $e->getChainedException(); /* @var $subException PDOException */
				// TRANSLATE
				throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to grant permissions to user %s (%s)'), array($zendUsername, $subException->getMessage())));
			} catch (Zend_Db_Exception $e) {
				// TRANSLATE
				throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to grant permissions to user %s (%s)'), array($zendUsername, $e->getMessage())));
			}
		} catch (Zwas_Exception $e) {
			if ($createdDatabase) {
				$this->cleanupCreateSchema($schemaName, $hostname);
			}
			throw $e;
		}
		
		return true;
	}
	
	/**
	 * Check the database schema's existence and structure
	 * 
	 * @return boolean
	 * @throws Zwas_Exception|ZwasComponents_ZSM_Monitor_DatabaseManager_Exception
	 */
	public function checkDatabaseSchema() {
		// Zend Monitor doesn't expose its required version number
		if (! defined('ZM_EVENTS_DB_SCHEMA_VERSION')) {
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('Missing database version'));
		}
		
		$adapter = $this->getDbAdapter();
		if (! $adapter->isConnected()) {
			// TRANSLATE
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(Zwas_Translate::_('No database connection is available'));
		}
		
		/// Store the new credentials to be used by the system during normal operations
		$this->zendUser = $this->user;
		$this->zendUserPassword = $this->pass;
		
		$serverVersion = $adapter->getServerVersion();
		if (version_compare($serverVersion, self::MINIMUM_DB_VERSION, '<')) {
			// TRANSLATE
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('Installed database version must be at least %s'), array(self::MINIMUM_DB_VERSION)));
		}
		
		/// check if schema version table exist
		$tables = $adapter->listTables();
		if ((! is_array($tables)) || (! in_array('schema_properties', $tables))) {
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(Zwas_Translate::_('Missing `schema_properties` table in the provided database'));
		}
		
		/// compare the schema's actual version with the required version
		try {
			$select = $adapter->select();
			$statement = $select->from('schema_properties','property_value','zend_monitor')
								->where('property = ?', 'SCHEMA_VERSION')
								->query();
			$version = $statement->fetchColumn(0);
		} catch (Zend_Db_Exception $e) {
			// TRANSLATE
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(Zwas_Translate::_('The `schema_properties` table in the provided database is invalid'));
		}

		if ((! $version) || (version_compare(ZM_EVENTS_DB_SCHEMA_VERSION, $version, '<>'))) {
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('The provided database\'s structure is incompatible with the current version of Zend Server Cluster Manager'));
		}
		
		return true;
	}
	
	/**
	 * Check if the provided connection has all needed permissions for bootstraping the database
	 * @param string $databaseName
	 * @param string $tableName
	 * @return boolean
	 * @throws Zwas_Exception|ZwasComponents_ZSM_Monitor_DatabaseManager_Exception
	 */
	public function checkCredentialsPermissions($databaseName, $tableName) {
		$adapter = $this->getDbAdapter();
		if (! $adapter->isConnected()) {
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(Zwas_Translate::_('No database connection is available'));
		}
		
		try {
			/// attempt to open the schema file
			$filepath = $this->getDatabaseSchemaFilePath();
			Context_FileSystem::getFileObject($filepath);
		} catch (Zwas_Exception $e) {
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('Unable to access the database structure script'));
		}
		
		$failedPermissions = false;
		$createdDatabase = false;
		$myself = null;
		try {
			/// check that we have SELECT on the mysql schema, retrieve the user's record
			$select = $adapter->select();
			$statement = $select->from('user', array_keys($this->requiredZendUserPermissions + $this->requiredCreatorPermissions), 'mysql')
					->where('CONCAT(user,\'@\',host) = CURRENT_USER()');
			
			/// Current is used because we expect only one row
			/// Avoid using fetch mode dependent calls - problems in PHP5.3
			$myself = current($adapter->fetchAssoc($statement));
		} catch (Zend_Db_Adapter_Exception $e) {
			$subException = $e->getChainedException(); /* @var $subException PDOException */
			// TRANSLATE
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to retrieve the mysql record for supplied user (%s)'), array($subException->getMessage())));
		} catch (Zend_Db_Exception $e) {
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to retrieve the mysql record for supplied user (%s)'), array($e->getMessage())));
		}
		
		foreach($this->requiredCreatorPermissions as $column => $privilege) {
			if ((! isset($myself[$column])) || ('Y' != $myself[$column])) {
				// TRANSLATE
				throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('The supplied credentials are missing the %s privilege'), array($privilege)));
			}
		}
		
		try {
			///try to create a random testing database for sanity
			$adapter->query("CREATE DATABASE `$databaseName`;");
		} catch (Zend_Db_Adapter_Exception $e) {
			$subException = $e->getChainedException(); /* @var $subException PDOException */
			// TRANSLATE
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to create a database named %s (%s)'), array($databaseName, $subException->getMessage())));
		} catch (Zend_Db_Exception $e) {
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to create a database named %s (%s)'), array($databaseName, $e->getMessage())));
		}
		
		try {
			///try to create a random table in the testing database
			$adapter->query("CREATE TABLE `$databaseName`.`$tableName` (temp INTEGER);");
		} catch (Zend_Db_Adapter_Exception $e) {
			$this->cleanupCheckCredentials($databaseName);
			$subException = $e->getChainedException(); /* @var $subException PDOException */
			// TRANSLATE
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to create a table named %s (%s)'), array("`$databaseName`.`$tableName`", $subException->getMessage())));
		} catch (Zend_Db_Exception $e) {
			$this->cleanupCheckCredentials($databaseName);
			throw new ZwasComponents_ZSM_Monitor_DatabaseManager_Exception(new Zwas_Text(Zwas_Translate::_('Unable to create a table named %s (%s)'), array("`$databaseName`.`$tableName`", $e->getMessage())));
		}
		
		$this->cleanupCheckCredentials($databaseName);
		return true;
	}
	
	/**
	 * @param array $config
	 * @return Zend_Db_Adapter_Abstract
	 */
	public function getDbAdapter($config = null) {
		if (is_null($this->dbAdapter)) {
			if (is_null($config)) {
				$config = array(
					'host' 		=> $this->host,
					'username' 	=> $this->user,
					'password' 	=> $this->pass,
					'port' 		=> $this->port
				);
				$config['dbname']= $this->db;
			}
			$this->dbAdapter = Zend_Db::factory('Pdo_Mysql', $config);
		}
		return $this->dbAdapter;
	}

	/**
	 * @return Zend_Http_Client_Adapter_Interface
	 */
	public function getHttpAdapter() {
		if (is_null($this->httpAdapter)) {
			$this->httpAdapter = new Zend_Http_Client_Adapter_Socket();
		}
		return $this->httpAdapter;
	}

	/**
	 * @param $dbAdapter the $dbAdapter to set
	 * @return ZwasComponents_ZSM_Monitor_Model
	 */
	public function setDbAdapter($dbAdapter) {
		$this->dbAdapter = $dbAdapter;
		return $this;
	}
	
	/**
	 * @return ZwasComponents_ZSM_Monitor_Model
	 */
	public function resetDbAdapter() {
		$this->dbAdapter = null;
		return $this;
	}

	/**
	 * @param Zend_Http_Client_Adapter_Interface $httpClient the $httpClient to set
	 * @return ZwasComponents_ZSM_Monitor_Model
	 */
	public function setHttpAdapter(Zend_Http_Client_Adapter_Interface $httpAdapter) {
		$this->httpAdapter = $httpAdapter;
		return $this;
	}
	
	/**
	 * @param string $databaseName
	 * @return boolean
	 */
	private function cleanupCheckCredentials($databaseName) {
		$adapter = $this->getDbAdapter();
		if (! $adapter->isConnected()) {
			return false;
		}
		try {
			$adapter->query("DROP DATABASE `$databaseName`;");
		} catch (Zend_Db_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException(new Zwas_Text(Zwas_Translate::_('Could not remove the priviledges probing database (%s)'), array($databaseName)), $e);
		}
		return true;
	}
	
	/**
	 * @param string $schemaName
	 * @param string $hostname
	 * @return boolean
	 */
	private function cleanupCreateSchema($schemaName, $hostname) {
		$zendUsername = self::ZEND_USERNAME;
		$adapter = $this->getDbAdapter();
		if (! $adapter->isConnected()) {
			// TRANSLATE
			return false;
		}
		try {
			$adapter->query("DROP DATABASE `$schemaName`");
		} catch (Zend_Db_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException(new Zwas_Text(Zwas_Translate::_("The system encountered an initialization error but could not clean up the new database schema '%s'"), array($schemaName)), $e);
		}
		
		try {
		/// also attempt to drop the user
			$adapter->query("DROP USER '$zendUsername'@'localhost'");
			$adapter->query("DROP USER '$zendUsername'@'%'");
			$adapter->query("DROP USER '$zendUsername'@'$hostname'");
		} catch (Zend_Db_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException(new Zwas_Text(Zwas_Translate::_("The system encountered an initialization error but could not clean up the user '%s'"), array($zendUsername)), $e);
		}
		
		try {
			/// verify cleanup
			$select = $adapter->select();
			$statement = $select->from('db', 'user', 'mysql')->where('user = ?', $zendUsername)->query();
			if(0 < $statement->rowCount()) {
				$adapter->query($adapter->quoteInto('DELETE FROM mysql.db WHERE User = ?', $zendUsername));
			}
		} catch (Zend_Db_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException(new Zwas_Text(Zwas_Translate::_("The system encountered an initialization error but could not clean up the privileges set for user '%s' on database '%s'"), array($zendUsername,$schemaName)), $e);
		}
		
		try {
			$adapter->query("FLUSH PRIVILEGES");
		} catch (Zend_Db_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException(new Zwas_Text(Zwas_Translate::_("The system encountered an initialization error but could not perform a FLUSH to remove the privileges"), array()), $e);
		}
		return true;
	}

	private function getDatabaseSchemaFilePath() {
		return Zwas_Path::create(Context_Api::getPhpMemory()->getCfgVar('zend.install_dir'), 'share', self::MONITOR_DBSCHEMA_FILENAME);
	}
	
	/**
	 * @return string
	 */
	private function getPermissionsList() {
		return implode(',',array_values($this->requiredZendUserPermissions));
	}
}
