<?php
class ZwasComponents_ZSM_Monitor_Model extends ZwasComponents_ZSM_ProbeModel {
	
	public function getController() {
		return 'Node-Monitor-Component';
	}
	
	/**
	 * Retrieve monitor_trace_mode directive value from the representative
	 * @return string
	 * @throws Zwas_Exception
	 */
	public function getMonitorMode() {
		$nodesModel = self::getNodesModel();
		try {
			$directives = $this->remoteAction($this->getController(), 'Get-Trace-Mode', array())->getData();
		} catch(Exception $e) {
			Zwas_Log::logException('Failed retrieving the trace mode directive from the cluster', $e);
			return ZwasComponents_Monitor_CodeTracingMode::EVENT_TRACING_MODE_UNKNOWN;
		}

		if (!is_array($directives) || !array_key_exists('traceMode', $directives)) {
			Zwas_Log::logMessage('Unrecognized structure received from representative: ' . print_r($directives, true), Zend_Log::WARN);
			return ZwasComponents_Monitor_CodeTracingMode::EVENT_TRACING_MODE_UNKNOWN;
		}
		return $directives['traceMode'];
	}
	
	/**
	 * Set the database ip address for the monitor daemon to report events to.
	 * The same IP address is propagagetd to nodes that are connected to the cluster.
	 * @param string $newHost an Ip address  
	 */
	public static function saveMonitorHost($newHost) {
		
		$validateLength = new Zend_Validate_StringLength(array('min' => 1, 'max' => 64));
		if (! $validateLength->isValid($newHost)) {
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('\'%s\' must be between 1 and 64 characters in length'), array($newHost)));
		}
		
		$validateAddress = new Zend_Validate_Hostname(array(
			'allow' => Zend_Validate_Hostname::ALLOW_IP | Zend_Validate_Hostname::ALLOW_LOCAL,
			'ip' => new Zend_Validate_Ip(array('allowipv4' => true, 'allowipv6' => true)),
		));
		
		if (! $validateAddress->isValid($newHost)) {
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('\'%s\' is not a valid IP address or host name'), array($newHost)));
		}
		
		try{ 
			ZwasComponents_Monitor_Daemon_Model::saveDirectives(array('zend_monitor.database.host_name' => $newHost));
		} catch (ZwasComponents_Monitor_Daemon_Exception $e) {
			// nothing to do - could not reload the Zend Monitor Daemon's configurations
		}
		
		$monitorUiModel = new ZwasComponents_MonitorUi_Model();
		$monitorUiModel->saveDirectives(array('zend_monitor.database.host_name' => $newHost));
	}
	
	/**
	 * Set the database ip, port and credentials for the monitor daemon to report events to.
	 * The same details are later to be propagagetd to nodes that are connected to the cluster.
	 * @param string $newHost
	 * @param string $newPort
	 * @param string $newUser
	 * @param string $newPassword
	 */
	public static function saveMonitorDbCredentials($newHost, $newPort, $newUser, $newPassword) {
		
		$directives = array(
			'zend_monitor.database.type' 		=> ZwasComponents_ZSM_Monitor_DatabaseManager::ZEND_MONITOR_DBTYPE,
			'zend_monitor.database.name' 		=> ZwasComponents_ZSM_Monitor_DatabaseManager::ZEND_MONITOR_SCHEMA,
			'zend_monitor.database.host_name' 	=> $newHost,
			'zend_monitor.database.user' 		=> $newUser,
			'zend_monitor.database.password' 	=> $newPassword,
			'zend_monitor.database.port' 		=> $newPort,
		);
		
		try{ 
			ZwasComponents_Monitor_Daemon_Model::saveDirectives($directives);
		} catch (ZwasComponents_Monitor_Daemon_Exception $e) {
			// nothing to do - could not reload the Zend Monitor Daemon's configurations
		}
		
		$monitorUiModel = new ZwasComponents_MonitorUi_Model();
		$monitorUiModel->saveDirectives($directives);
	}
	
	/**
	 * Attempt to create a new user and then create the zend_monitor
	 * schema in the database, using a power user that is provided
	 * @param ZwasComponents_ZSM_Monitor_DatabaseManager $manager
	 * @return boolean
	 * @throws Zwas_Exception
	 */
	public function createUserAndSchema(ZwasComponents_ZSM_Monitor_DatabaseManager $manager) {
		try {
			$manager->checkDatabaseTcp();
		} catch (Zwas_Exception $e) {
			// TRANSLATE
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('Could not establish a tcp connection: %s'), array($e->getMessageObject())));
		}
		
		try {
			$manager->checkDatabaseConnection();
		} catch (ZwasComponents_ZSM_Monitor_DatabaseManager_Exception $e) {
			Zwas_Log::logException('Failed in database connection check', $e);
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('The system could not connect to the supplied database to create the user and schema'));
		} catch (Zwas_Exception $e) {
			Zwas_Log::logException('Failed in database connection check', $e);
			// TRANSLATE
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('The system could not check the database connection: %s'), array($e->getMessageObject())));
		}

		try {
			$manager->checkCredentialsPermissions('zend_monitor_test', 'zend_monitor_test_table');
		} catch (ZwasComponents_ZSM_Monitor_DatabaseManager_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException('Credentials check failed', $e);
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('The system could not verify the supplied credentials\' permissions'));
		} catch (Zwas_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException('Credentials check failed', $e);
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('The system could not check the privileges provided for the current credentials: %s'), array($e->getMessageObject())));
		}
			
		$newPassword = Zwas_Random::randomPassword();
		$hostname = Context_Os::getHostname();
		
		try {
			$manager->createZendUser($newPassword, $hostname);
		} catch (ZwasComponents_ZSM_Monitor_DatabaseManager_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException("Error while attempting to create the user '{$manager->getZendUser()}'", $e);
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('The system could not create a user for the database'));
		} catch (Zwas_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException("Error while attempting to create the user '{$manager->getZendUser()}'", $e);
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('The system could not create the user \'%s\': %s'), array($manager->getZendUser(), $e->getMessageObject())));
		}
		
		try {
			$manager->createSchema(ZwasComponents_ZSM_Monitor_DatabaseManager::ZEND_MONITOR_SCHEMA, $hostname);
		} catch (ZwasComponents_ZSM_Monitor_DatabaseManager_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException('Error encountered while attempting to build the database schema', $e);
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('Ths system could not build the database schema'));
		} catch (Zwas_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException('Error encountered while attempting to build the database schema', $e);
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('The system could not create the schema structure in \'%s\': %s'), array(ZwasComponents_ZSM_Monitor_DatabaseManager::ZEND_MONITOR_SCHEMA, $e->getMessageObject())));
		}
		return true;
	}
	
	/**
	 * @param ZwasComponents_ZSM_Monitor_DatabaseManager $manager
	 * @return boolean
	 */
	public function checkUserAndSchema(ZwasComponents_ZSM_Monitor_DatabaseManager $manager) {
		try {
			$manager->checkDatabaseConnection();
		} catch (ZwasComponents_ZSM_Monitor_DatabaseManager_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException('Failed in database connection check', $e);
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('The system could not connect to the supplied database to validate the supplied credentials and the existing schema'));
		} catch (Zwas_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException('Failed in database connection check', $e);
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('The system could not check the database connection: %s'), array($e->getMessageObject())));
		}
		
		try {
			$manager->checkDatabaseSchema();
		} catch (ZwasComponents_ZSM_Monitor_DatabaseManager_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException('Failed in database structure check', $e);
			// TRANSLATE
			throw new Zwas_Exception(Zwas_Translate::_('The system could not validate the schema structure for the existing database'));
		} catch (Zwas_Exception $e) {
			// TRANSLATE
			Zwas_Log::logException('Failed in database structure check', $e);
			throw new Zwas_Exception(new Zwas_Text(Zwas_Translate::_('The system could not check the provided database structure: %s'), array($e->getMessageObject())));
		}
		return true;
	}
}