<?php

/**
 * 
 * This class aims to provide simple none atomic synchronization between processes
 * guaranteeing only that execution will be busy-waited while any other process executes it
 *
 * @uses Context_FileSystem
 */
class Zwas_Synchronize {

	/**
	 * @var integer
	 */
	private $pid;
	/**
	 * @var string
	 */
	private $context;
	/**
	 * @var integer
	 */
	private $timeout;
	/**
	 * @var Zwas_Synchronize
	 */
	private static $instance;
	
	/**
	 * @return Zwas_Synchronize
	 */
	public static function getInstance() {
		if (self::$instance instanceof Zwas_Synchronize) {
			return self::$instance;
		}
		// TRANSLATE
		throw new Zwas_Exception(Zwas_Translate::_('No Synchronization object is set'));
	}
	
	/**
	 * @param Zwas_Synchronize $sync
	 * @return Zwas_Synchronize
	 */
	public static function setInstance(Zwas_Synchronize $sync) {
		self::$instance = $sync;
		return $sync;
	}
	
	public static function resetInstance() {
		self::$instance = null;
	}
	
	/**
	 * Create a new synchronization instance and store it statically
	 * This method does not support nested synchronization!
	 * 
	 * @param string $context
	 * @param integer $timeout
	 * @return boolean
	 */
	public static function sync($context, $timeout) {
		return self::setInstance(new Zwas_Synchronize($context, $timeout))->wait();
	}
	
	/**
	 * @param string $context
	 * @param string $pid PID may also be * for the same of globing the markers directory
	 * @return string
	 */
	public static function getMarkerPath($context, $pid) {
		return Zwas_Path::create(ApplicationModel::getGuiTempDir(), "{$context}.{$pid}.sync");
	}
	
	/**
	 * @return boolean
	 */
	public static function release() {
		$result = self::getInstance()->signal();
		self::resetInstance();
		return $result;
	}
	
	/**
	 * @param string $this->context
	 * @param integer $this->timeout
	 */
	public function __construct($context, $timeout) {
		$this->pid = getmypid();
		$this->context = $context;
		$this->timeout = $timeout;
	}
	
	/**
	 * Attempt to gain control.
	 * 
	 * Returns true if access was acquired.
	 * False is returned if wait has timedout and forces acquisition.
	 * 
	 * @param string $this->context
	 * @param integer $this->timeout threshold in seconds for ignoring the maker
	 * @return boolean
	 */
	public function wait() {
		/// leave a note
		Context_FileSystem::touch(self::getMarkerPath($this->context, $this->pid));
		/// count the sync markers
		$syncs = $this->collectSyncMarkers($this->context, $this->timeout);
		$syncsCount = count($syncs);
		
		if (1 < $syncsCount) {
			/// more than one sync marker, back off!
			$this->signal();
			return false;
		} else {
			/// one sync marker only, we've acquired posession
			return true;
		}
	}
	
	/**
	 * @return boolean
	 */
	public function signal() {
		// remove my marker
		if (Context_FileSystem::fileExists(self::getMarkerPath($this->context, $this->pid))) {
			return Context_FileSystem::unlink(self::getMarkerPath($this->context, $this->pid));
		}
		return false;
	}
	
	/**
	 * @param string $context
	 * @param integer $timeout
	 * @return array
	 */
	private function collectSyncMarkers($context, $timeout) {
		$syncs = Context_FileSystem::getDirectoryObject(ApplicationModel::getGuiTempDir())->getGlob(basename(self::getMarkerPath($this->context, '*')));
		$validSyncs = array();
		foreach($syncs as $syncMarker) {
			$file = Context_FileSystem::getFileObject($syncMarker);
			if ($this->timeout < (time() - $file->getMTime())) {
				unset($file);
				Context_FileSystem::unlink($syncMarker);
			} else {
				unset($file);
				$validSyncs []= $syncMarker;
			}
		}
		return $validSyncs;
	}
}


